@@ -4,22 +4,20 @@
 import torch
 
 try:
-    import torch_ecg
-except:
+    import torch_ecg  # noqa: F401
+except ModuleNotFoundError:
     import sys
     from pathlib import Path
 
-    sys.path.append(Path(__file__).absolute().parent.parent)
-    import torch_ecg
+    sys.path.insert(0, str(Path(__file__).absolute().parents[1]))
 
 from torch_ecg.models.loss import (
-    BCEWithLogitsWithClassWeightLoss,
-    MaskedBCEWithLogitsLoss,
-    FocalLoss,
     AsymmetricLoss,
+    BCEWithLogitsWithClassWeightLoss,
+    FocalLoss,
+    MaskedBCEWithLogitsLoss,
 )
 
-
 inp = torch.tensor([[10.0, -10.0], [-10.0, 10.0]])
 targ_1 = torch.tensor([[1.0, 0.0], [0.0, 1.0]])
 targ_0 = torch.tensor([[0.0, 1.0], [1.0, 0.0]])
