@@ -1,20 +1,20 @@
 """
 """
 
-from random import choice, randint
-from typing import Any, NoReturn, Sequence, List, Tuple, Union, Optional
 from numbers import Real
+from random import choice, randint
+from typing import Any, List, NoReturn, Optional, Sequence, Tuple, Union
 
 import numpy as np
 import torch
-from torch import Tensor
-import torch.nn.functional as F
 import torch.multiprocessing as tmp
-from scipy.signal import resample, resample_poly
+import torch.nn.functional as F
+from scipy.signal import resample, resample_poly  # noqa: F401
+from torch import Tensor
 
-from .base import Augmenter
+from ..cfg import DEFAULTS
 from ..utils.misc import ReprMixin
-
+from .base import Augmenter
 
 __all__ = [
     "StretchCompress",
@@ -42,7 +42,7 @@ class StretchCompress(Augmenter):
     def __init__(
         self, ratio: Real = 6, prob: float = 0.5, inplace: bool = True, **kwargs: Any
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -70,7 +70,7 @@ class StretchCompress(Augmenter):
     def forward(
         self, sig: Tensor, *labels: Optional[Sequence[Tensor]], **kwargs: Any
     ) -> Tuple[Tensor, ...]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -175,7 +175,7 @@ class StretchCompress(Augmenter):
     def _sample_ratio(self) -> float:
         """ """
         return np.clip(
-            np.random.normal(self.ratio, 0.382 * self.ratio), 0, 2 * self.ratio
+            DEFAULTS.RNG.normal(self.ratio, 0.382 * self.ratio), 0, 2 * self.ratio
         )
 
     def _generate(
@@ -259,6 +259,7 @@ def _stretch_compress_one_batch_element(
     labels = list(labels)
     label_len = []
     n_labels = len(labels)
+    siglen = sig.shape[-1]
     for idx in range(n_labels):
         labels[idx] = labels[idx].permute(
             0, 2, 1
@@ -270,7 +271,7 @@ def _stretch_compress_one_batch_element(
             )
         label_len.append(ll)
     sign = choice([-1, 1])
-    ratio = np.clip(np.random.normal(ratio, 0.382 * ratio), 0, 2 * ratio)
+    ratio = np.clip(DEFAULTS.RNG.normal(ratio, 0.382 * ratio), 0, 2 * ratio)
     # print(f"batch_idx = {batch_idx}, sign = {sign}, ratio = {ratio}")
     new_len = int(round((1 + sign * ratio) * siglen))
     diff_len = abs(new_len - siglen)
@@ -349,7 +350,7 @@ class StretchCompressOffline(ReprMixin):
         overlap: float = 0.5,
         critical_overlap: float = 0.85,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -388,7 +389,7 @@ class StretchCompressOffline(ReprMixin):
         *labels: Sequence[np.ndarray],
         critical_points: Optional[Sequence[int]] = None,
     ) -> List[Tuple[Union[np.ndarray, int], ...]]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -467,7 +468,7 @@ class StretchCompressOffline(ReprMixin):
         start_idx: Optional[int] = None,
         end_idx: Optional[int] = None,
     ) -> Tuple[Union[np.ndarray, int], ...]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -543,12 +544,12 @@ class StretchCompressOffline(ReprMixin):
         return (aug_seg,) + tuple(aug_labels) + (start_idx, end_idx)
 
     def _sample_ratio(self) -> float:
-        """finished, checked,"""
-        if np.random.uniform() >= self.prob:
+        """ """
+        if DEFAULTS.RNG.uniform() >= self.prob:
             return 0
         else:
             return np.clip(
-                np.random.normal(self.ratio, 0.382 * self.ratio),
+                DEFAULTS.RNG.normal(self.ratio, 0.382 * self.ratio),
                 0.01 * self.ratio,
                 2 * self.ratio,
             )
