@@ -12,41 +12,37 @@ Remarks:
 
 """
 
-import os, pprint, time, posixpath, re, warnings
-from pathlib import Path
+import os
+import posixpath
+import pprint
+import re
+import time
+import warnings
 from abc import ABC, abstractmethod
-from collections import namedtuple
+from copy import deepcopy
 from dataclasses import dataclass
-from typing import Union, Optional, Any, List, NoReturn
-from numbers import Real
+from pathlib import Path
 from string import punctuation
+from typing import Any, List, NoReturn, Optional, Union
 
-import wfdb, requests
 import numpy as np
 import pandas as pd
+import requests
+import wfdb
 from pyedflib import EdfReader
 
-from ..utils import ecg_arrhythmia_knowledge as EAK
-from ..utils.misc import (
-    get_record_list_recursive3,
-    dict_to_str,
-    ReprMixin,
-    list_sum,
-)
+from ..utils import ecg_arrhythmia_knowledge as EAK  # noqa: F401
 from ..utils.download import http_get
+from ..utils.misc import ReprMixin, dict_to_str, get_record_list_recursive3
 from .aux_data import get_physionet_dbs
 
-
 __all__ = [
     "WFDB_Beat_Annotations",
     "WFDB_Non_Beat_Annotations",
     "WFDB_Rhythm_Annotations",
     "PhysioNetDataBase",
     "NSRRDataBase",
-    "ImageDataBase",
-    "AudioDataBase",
-    "OtherDataBase",
-    "ECGWaveForm",
+    "CPSCDataBase",
     "DEFAULT_FIG_SIZE_PER_SEC",
     "BeatAnn",
 ]
@@ -149,6 +145,13 @@ class _DataBase(ReprMixin, ABC):
         """
         self.db_name = db_name
         self.db_dir = Path(db_dir)
+        if not self.db_dir.exists():
+            self.db_dir.mkdir(parents=True, exist_ok=True)
+            warnings.warn(
+                f"{self.db_dir} does not exist, created it. "
+                "Please check if it is correct. "
+                "Or you may want to download the database into this folder."
+            )
         self.working_dir = Path(working_dir or os.getcwd())
         self.working_dir.mkdir(parents=True, exist_ok=True)
         self.data_ext = None
@@ -163,7 +166,7 @@ class _DataBase(ReprMixin, ABC):
         raise NotImplementedError
 
     @abstractmethod
-    def load_data(self, rec: str, **kwargs) -> Any:
+    def load_data(self, rec: Union[str, int], **kwargs) -> Any:
         """
         load data from the record `rec`
 
@@ -171,7 +174,7 @@ class _DataBase(ReprMixin, ABC):
         raise NotImplementedError
 
     @abstractmethod
-    def load_ann(self, rec: str, **kwargs) -> Any:
+    def load_ann(self, rec: Union[str, int], **kwargs) -> Any:
         """
         load annotations of the record `rec`
 
@@ -181,7 +184,7 @@ class _DataBase(ReprMixin, ABC):
         raise NotImplementedError
 
     def _auto_infer_units(self, sig: np.ndarray, sig_type: str = "ECG") -> str:
-        """finished, checked,
+        """
 
         automatically infer the units of `sig`,
         under the assumption that `sig` not being raw signal, with baseline removed
@@ -233,7 +236,7 @@ class _DataBase(ReprMixin, ABC):
     def get_arrhythmia_knowledge(
         cls, arrhythmias: Union[str, List[str]], **kwargs: Any
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         knowledge about ECG features of specific arrhythmias,
 
@@ -343,7 +346,7 @@ class PhysioNetDataBase(_DataBase):
             )
 
     def _ls_rec(self, db_name: Optional[str] = None, local: bool = True) -> NoReturn:
-        """finished, checked,
+        """
 
         find all records (relative path without file extension),
         and save into `self._all_records` for further use
@@ -363,13 +366,11 @@ class PhysioNetDataBase(_DataBase):
         try:
             self._all_records = wfdb.get_record_list(db_name or self.db_name)
             self._all_records = [Path(item).name for item in self._all_records]
-        except:
+        except Exception:
             self._ls_rec_local()
 
-    def _ls_rec_local(
-        self,
-    ) -> NoReturn:
-        """finished, checked,
+    def _ls_rec_local(self) -> NoReturn:
+        """
 
         find all records in `self.db_dir`
 
@@ -387,7 +388,7 @@ class PhysioNetDataBase(_DataBase):
         start = time.time()
         self._all_records = get_record_list_recursive3(self.db_dir, self.data_ext)
         print(f"Done in {time.time() - start:.3f} seconds!")
-        record_list_fp.write_text("\n".join(self._all_records) + "\n")
+        # record_list_fp.write_text("\n".join(self._all_records) + "\n")
 
     def get_subject_id(self, rec: str) -> int:
         """
@@ -421,7 +422,7 @@ class PhysioNetDataBase(_DataBase):
                 ]["db_description"].values[0]
                 print(short_description)
                 return
-            except:
+            except Exception:
                 pass
         info = "\n".join(self.__doc__.split("\n")[1:])
         print(info)
@@ -429,7 +430,7 @@ class PhysioNetDataBase(_DataBase):
     def helper(
         self, items: Union[List[str], str, type(None)] = None, **kwargs
     ) -> NoReturn:
-        """finished, checked, to be improved,
+        """to be improved,
 
         print corr. meanings of symbols belonging to `items`
 
@@ -506,7 +507,7 @@ class PhysioNetDataBase(_DataBase):
                 if k in a.keys() or "(" + k in a.keys():
                     try:
                         print(f"{k.split('(')[1]} stands for {a[k]}")
-                    except:
+                    except Exception:
                         print(f"{k} stands for {a['('+k]}")
 
     @property
@@ -515,7 +516,7 @@ class PhysioNetDataBase(_DataBase):
             return self._version
         try:
             self._version = wfdb.io.record.get_version(self.db_name)
-        except:
+        except Exception:
             warnings.warn(
                 "Cannot get the version number from PhysioNet! Defaults to '1.0.0'"
             )
@@ -540,12 +541,18 @@ class PhysioNetDataBase(_DataBase):
         if self._url_compressed is not None:
             return self._url_compressed
         domain = "https://physionet.org/static/published-projects/"
-        punct = re.sub("[-:]", "", punctuation)
-        db_desc = self.df_all_db_info[
-            self.df_all_db_info["db_name"] == self.db_name
-        ].iloc[0]["db_description"]
+        punct = re.sub("[\\-:]", "", punctuation)
+        try:
+            db_desc = self.df_all_db_info[
+                self.df_all_db_info["db_name"] == self.db_name
+            ].iloc[0]["db_description"]
+        except IndexError:
+            print(
+                f"\042{self.db_name}\042 is not in the database list hosted at PhysioNet!"
+            )
+            return ""
         db_desc = re.sub(f"[{punct}]+", "", db_desc).lower()
-        db_desc = re.sub("[\s:]+", "-", db_desc)
+        db_desc = re.sub("[\\s:]+", "-", db_desc)
         url = posixpath.join(domain, f"{self.db_name}/{db_desc}-{self.version}.zip")
         if requests.head(url).headers.get("Content-Type") == "application/zip":
             self._url_compressed = url
@@ -646,7 +653,7 @@ class NSRRDataBase(_DataBase):
         operation: str = "close",
         full_file_path: Optional[Union[str, Path]] = None,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -842,11 +849,6 @@ class CPSCDataBase(_DataBase):
             pp.pprint(methods)
 
 
-ECGWaveForm = namedtuple(
-    typename="ECGWaveForm",
-    field_names=["name", "onset", "offset", "peak", "duration"],
-)
-
 DEFAULT_FIG_SIZE_PER_SEC = 4.8
 
 
