@@ -2,34 +2,38 @@
 """
 """
 
-import math, time, json, warnings
-from pathlib import Path
-from typing import Union, Optional, Any, List, Tuple, Dict, Sequence, NoReturn
+import json
+import math
+import os
+import time
+import warnings
 from numbers import Real
+from pathlib import Path
+from typing import Any, Dict, List, NoReturn, Optional, Sequence, Tuple, Union
 
 import numpy as np
 import pandas as pd
-from scipy.signal import resample, resample_poly
+import scipy.io as sio
 import wfdb
+from scipy.signal import resample, resample_poly  # noqa: F401
 
 from ...cfg import CFG
 from ...utils.misc import (
-    get_record_list_recursive,
     get_record_list_recursive3,
-    ms2samples,
     list_sum,
+    ms2samples,
+    add_docstring,
 )
 from ...utils.utils_interval import generalized_intervals_intersection
-from ..base import (
+from ..base import (  # noqa: F401
+    DEFAULT_FIG_SIZE_PER_SEC,
     CPSCDataBase,
     PhysioNetDataBase,
-    DEFAULT_FIG_SIZE_PER_SEC,
     WFDB_Beat_Annotations,
     WFDB_Non_Beat_Annotations,
     WFDB_Rhythm_Annotations,
 )
 
-
 __all__ = [
     "CPSC2021",
     "compute_metrics",
@@ -104,9 +108,10 @@ class CPSC2021(PhysioNetDataBase):
 
     References
     ----------
-    [1] http://www.icbeb.org/CPSC2021
-    [2] https://www.physionet.org/content/cpsc2021/1.0.0/
-    [3] https://archive.physionet.org/physiobank/annotations.shtml
+    1. <a name="ref1"></a> http://www.icbeb.org/CPSC2021
+    2. <a name="ref2"></a> https://www.physionet.org/content/cpsc2021/1.0.0/
+    3. <a name="ref3"></a> https://archive.physionet.org/physiobank/annotations.shtml
+
     """
 
     def __init__(
@@ -116,7 +121,7 @@ class CPSC2021(PhysioNetDataBase):
         verbose: int = 2,
         **kwargs: Any,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -127,6 +132,7 @@ class CPSC2021(PhysioNetDataBase):
         verbose: int, default 2,
             log verbosity
         kwargs: auxilliary key word arguments
+
         """
         super().__init__(
             db_name="cpsc2021",
@@ -149,7 +155,7 @@ class CPSC2021(PhysioNetDataBase):
         self.ann_ext = "atr"
         self.header_ext = "hea"
         self.all_leads = ["I", "II"]
-        self.rec_patterns_with_ext = f"^data_(?:\d+)_(?:\d+).{self.rec_ext}$"
+        self.rec_patterns_with_ext = f"^data_(?:\\d+)_(?:\\d+).{self.rec_ext}$"
 
         self._labels_f2a = {  # fullname to abbreviation
             "non atrial fibrillation": "N",
@@ -199,10 +205,11 @@ class CPSC2021(PhysioNetDataBase):
         return self.__all_records
 
     def _ls_rec(self) -> NoReturn:
-        """finished, checked,
+        """
 
         list all the records and load into `self._all_records`,
         facilitating further uses
+
         """
         self._all_records = CFG({t: [] for t in self.db_tranches})
         self._all_subjects = CFG({t: [] for t in self.db_tranches})
@@ -262,6 +269,7 @@ class CPSC2021(PhysioNetDataBase):
 
         list all the records assuming the records
         are split into two folders (training_I and training_II)
+
         """
         fn = "RECORDS"
         rev_fn = "REVISED_RECORDS"
@@ -297,10 +305,7 @@ class CPSC2021(PhysioNetDataBase):
                 self.__revised_records.extend(record_list_fp.read_text().splitlines())
 
     def _aggregate_stats(self) -> NoReturn:
-        """finished, checked,
-
-        aggregate stats on the whole dataset
-        """
+        """aggregate stats on the whole dataset"""
         stats_file = "stats.csv"
         stats_file_fp = self.db_dir_base / stats_file
         if stats_file_fp.is_file():
@@ -361,10 +366,7 @@ class CPSC2021(PhysioNetDataBase):
         return self._stats
 
     def _ls_diagnoses_records(self) -> NoReturn:
-        """finished, checked,
-
-        list all the records for all diagnoses
-        """
+        """list all the records for all diagnoses"""
         fn = "diagnoses_records_list.json"
         dr_fp = self.db_dir_base / fn
         if dr_fp.is_file():
@@ -394,34 +396,37 @@ class CPSC2021(PhysioNetDataBase):
 
     @property
     def diagnoses_records_list(self):
-        """finished, checked"""
+        """ """
         if self._diagnoses_records_list is None:
             self._ls_diagnoses_records()
         return self._diagnoses_records_list
 
-    def get_subject_id(self, rec: str) -> str:
-        """finished, checked,
+    def get_subject_id(self, rec: Union[str, int]) -> str:
+        """
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
 
         Returns
         -------
         sid: str,
             subject id corresponding to the record
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         sid = rec.split("_")[1]
         return sid
 
-    def _get_path(self, rec: str, ext: Optional[str] = None) -> Path:
-        """finished, checked,
+    def _get_path(self, rec: Union[str, int], ext: Optional[str] = None) -> Path:
+        """
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         ext: str, optional,
             file extension of the path
 
@@ -430,6 +435,8 @@ class CPSC2021(PhysioNetDataBase):
         p: Path,
             path (with or without file extension) of the record
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         if ext:
             rec += f".{ext}"
         p = self.db_dirs[self._all_records_inv[rec]] / rec
@@ -437,17 +444,17 @@ class CPSC2021(PhysioNetDataBase):
 
     def _validate_samp_interval(
         self,
-        rec: str,
+        rec: Union[str, int],
         sampfrom: Optional[int] = None,
         sampto: Optional[int] = None,
     ) -> Tuple[int, int]:
-        """finished, checked,
+        """
         validate `sampfrom` and `sampto` so that they are reasonable
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         sampfrom: int, optional,
             start index of the data to be loaded
         sampto: int, optional,
@@ -460,7 +467,10 @@ class CPSC2021(PhysioNetDataBase):
             index sampling from
         st: int,
             index sampling to
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         sf, st = (
             sampfrom or 0,
             sampto or self.df_stats[self.df_stats.record == rec].iloc[0].sig_len,
@@ -471,7 +481,7 @@ class CPSC2021(PhysioNetDataBase):
 
     def load_data(
         self,
-        rec: str,
+        rec: Union[str, int],
         leads: Optional[Union[str, List[str]]] = None,
         data_format: str = "channel_first",
         units: str = "mV",
@@ -479,19 +489,19 @@ class CPSC2021(PhysioNetDataBase):
         sampto: Optional[int] = None,
         fs: Optional[Real] = None,
     ) -> np.ndarray:
-        """finished, checked,
+        """
 
-        load physical (converted from digital) ecg data,
+        load physical (converted from digital) ECG data,
         which is more understandable for humans
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         leads: str or list of str, optional,
             the leads to load
         data_format: str, default "channel_first",
-            format of the ecg data,
+            format of the ECG data,
             "channel_last" (alias "lead_last"), or
             "channel_first" (alias "lead_first")
         units: str, default "mV",
@@ -506,8 +516,11 @@ class CPSC2021(PhysioNetDataBase):
         Returns
         -------
         data: ndarray,
-            the ecg data
+            the ECG data
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         assert data_format.lower() in [
             "channel_first",
             "lead_first",
@@ -520,7 +533,7 @@ class CPSC2021(PhysioNetDataBase):
             _leads = [leads]
         else:
             _leads = leads
-        assert all([l in self.all_leads for l in _leads])
+        assert all([ld in self.all_leads for ld in _leads])
 
         rec_fp = self._get_path(rec)
         sf, st = self._validate_samp_interval(rec, sampfrom, sampto)
@@ -543,20 +556,20 @@ class CPSC2021(PhysioNetDataBase):
 
     def load_ann(
         self,
-        rec: str,
+        rec: Union[str, int],
         field: Optional[str] = None,
         sampfrom: Optional[int] = None,
         sampto: Optional[int] = None,
         **kwargs: Any,
     ) -> Union[dict, np.ndarray, List[List[int]], str]:
-        """finished, checked,
+        """
 
         load annotations of the record
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         field: str, optional
             field of the annotation, can be one of "rpeaks", "af_episodes", "label", "raw", "wfdb",
             if not specified, all fields of the annotation will be returned in the form of a dict,
@@ -580,6 +593,8 @@ class CPSC2021(PhysioNetDataBase):
         ann: dict, or list, or ndarray, or str,
             annotaton of the record
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         sf, st = self._validate_samp_interval(rec, sampfrom, sampto)
         ann = wfdb.rdann(
             str(self._get_path(rec)), extension=self.ann_ext, sampfrom=sf, sampto=st
@@ -605,28 +620,28 @@ class CPSC2021(PhysioNetDataBase):
 
         try:
             f = func[field.lower()]
-        except:
-            raise ValueError(f"invalid field")
+        except Exception:
+            raise ValueError("invalid field")
         ann = f(rec, ann, sf, st, **kwargs)
         return ann
 
     def load_rpeaks(
         self,
-        rec: str,
+        rec: Union[str, int],
         ann: Optional[wfdb.Annotation] = None,
         sampfrom: Optional[int] = None,
         sampto: Optional[int] = None,
-        zero_start: bool = False,
+        keep_original: bool = False,
         fs: Optional[Real] = None,
     ) -> np.ndarray:
-        """finished, checked,
+        """
 
         load position (in terms of samples) of rpeaks
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         ann: Annotation, optional,
             the wfdb Annotation of the record,
             if None, corresponding annotation file will be read
@@ -634,10 +649,9 @@ class CPSC2021(PhysioNetDataBase):
             start index of the data to be loaded
         sampto: int, optional,
             end index of the data to be loaded
-        zero_start: bool, default False,
-            if True, (relative) start index is zero,
-            otherwise, (relative) start index is `sampfrom`,
-            works only when `sampfrom` is positive
+        keep_original: bool, default False,
+            if True, indices will keep the same with the annotation file
+            otherwise subtract `sampfrom` if specified
         fs: real number, optional,
             if not None, positions of the loaded rpeaks will be ajusted according to this sampling frequency
 
@@ -645,7 +659,10 @@ class CPSC2021(PhysioNetDataBase):
         -------
         rpeaks: ndarray,
             position (in terms of samples) of rpeaks of the record
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         if ann is None:
             sf, st = self._validate_samp_interval(rec, sampfrom, sampto)
             ann = wfdb.rdann(
@@ -654,7 +671,7 @@ class CPSC2021(PhysioNetDataBase):
         critical_points = ann.sample
         symbols = ann.symbol
         rpeaks_valid = np.isin(symbols, list(WFDB_Beat_Annotations.keys()))
-        if sampfrom and zero_start:
+        if sampfrom and not keep_original:
             critical_points = critical_points - sampfrom
         if fs is not None and fs != self.fs:
             critical_points = np.round(
@@ -663,24 +680,39 @@ class CPSC2021(PhysioNetDataBase):
         rpeaks = critical_points[rpeaks_valid]
         return rpeaks
 
-    def load_af_episodes(
+    @add_docstring(load_rpeaks.__doc__)
+    def load_rpeak_indices(
         self,
-        rec: str,
+        rec: Union[str, int],
         ann: Optional[wfdb.Annotation] = None,
         sampfrom: Optional[int] = None,
         sampto: Optional[int] = None,
-        zero_start: bool = False,
+        keep_original: bool = False,
+        fs: Optional[Real] = None,
+    ) -> np.ndarray:
+        """
+        alias of `self.load_rpeaks`
+        """
+        return self.load_rpeaks(rec, ann, sampfrom, sampto, keep_original, fs)
+
+    def load_af_episodes(
+        self,
+        rec: Union[str, int],
+        ann: Optional[wfdb.Annotation] = None,
+        sampfrom: Optional[int] = None,
+        sampto: Optional[int] = None,
+        keep_original: bool = False,
         fs: Optional[Real] = None,
         fmt: str = "intervals",
     ) -> Union[List[List[int]], np.ndarray]:
-        """finished, checked,
+        """
 
         load the episodes of atrial fibrillation, in terms of intervals or mask
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         ann: Annotation, optional,
             the wfdb Annotation of the record,
             if None, corresponding annotation file will be read
@@ -690,10 +722,10 @@ class CPSC2021(PhysioNetDataBase):
         sampto: int, optional,
             end index of the data to be loaded,
             not used when `fmt` is "c_intervals"
-        zero_start: bool, default False,
-            if True, (relative) start index is zero,
-            otherwise, (relative) start index is `sampfrom`,
-            works only when `sampfrom` is positive and `fmt` is not "c_intervals"
+        keep_original: bool, default False,
+            if True, indices will keep the same with the annotation file
+            otherwise subtract `sampfrom` if specified
+            works only when `fmt` is not "c_intervals"
         fs: real number, optional,
             if not None, positions of the loaded intervals or mask will be ajusted according to this sampling frequency
         fmt: str, default "intervals",
@@ -703,14 +735,13 @@ class CPSC2021(PhysioNetDataBase):
         -------
         af_episodes: list or ndarray,
             episodes of atrial fibrillation, in terms of intervals or mask
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         header = wfdb.rdheader(str(self._get_path(rec)))
         label = self._labels_f2a[header.comments[0]]
         siglen = header.sig_len
-        # if ann is None or fmt.lower() in ["c_intervals",]:
-        #     _ann = wfdb.rdann(str(self._get_path(rec)), extension=self.ann_ext)
-        # else:
-        #     _ann = ann
         _ann = wfdb.rdann(str(self._get_path(rec)), extension=self.ann_ext)
         sf, st = self._validate_samp_interval(rec, sampfrom, sampto)
         aux_note = np.array(_ann.aux_note)
@@ -728,7 +759,7 @@ class CPSC2021(PhysioNetDataBase):
         ]:
             if sf > 0 or st < siglen:
                 raise ValueError(
-                    f"when `fmt` is `c_intervals`, `sampfrom` and `sampto` should never be used!"
+                    "when `fmt` is `c_intervals`, `sampfrom` and `sampto` should never be used!"
                 )
             af_episodes = [
                 [start, end] for start, end in zip(af_start_inds, af_end_inds)
@@ -758,7 +789,7 @@ class CPSC2021(PhysioNetDataBase):
                     for itv in intervals
                 ]
 
-        if zero_start:
+        if not keep_original:
             intervals = [[itv[0] - sf, itv[1] - sf] for itv in intervals]
             sf = 0
         af_episodes = intervals
@@ -775,13 +806,13 @@ class CPSC2021(PhysioNetDataBase):
 
     def load_label(
         self,
-        rec: str,
+        rec: Union[str, int],
         ann: Optional[wfdb.Annotation] = None,
         sampfrom: Optional[int] = None,
         sampto: Optional[int] = None,
         fmt: str = "a",
     ) -> str:
-        """finished, checked,
+        """
 
         load (classifying) label of the record,
         among the following three classes:
@@ -791,8 +822,8 @@ class CPSC2021(PhysioNetDataBase):
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         ann: Annotation, optional,
             not used, to keep in accordance with other methods
         sampfrom: int, optional,
@@ -809,7 +840,10 @@ class CPSC2021(PhysioNetDataBase):
         -------
         label: str,
             classifying label of the record
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         header = wfdb.rdheader(str(self._get_path(rec)))
         label = header.comments[0]
         if fmt.lower() in ["a", "abbr", "abbreviation"]:
@@ -821,16 +855,16 @@ class CPSC2021(PhysioNetDataBase):
         return label
 
     def gen_endpoint_score_mask(
-        self, rec: str, bias: dict = {1: 1, 2: 0.5}
+        self, rec: Union[str, int], bias: dict = {1: 1, 2: 0.5}
     ) -> Tuple[np.ndarray, np.ndarray]:
-        """finished, checked,
+        """
 
         generate the scoring mask for the onsets and offsets of af episodes,
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         bias: dict, default {1:1, 2:0.5},
             keys are bias (with ±) in terms of number of rpeaks
             values are corresponding scores
@@ -844,7 +878,10 @@ class CPSC2021(PhysioNetDataBase):
         ----
         the onsets in `af_intervals` are 0.15s ahead of the corresponding R peaks,
         while the offsets in `af_intervals` are 0.15s behind the corresponding R peaks,
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         masks = gen_endpoint_score_mask(
             siglen=self.df_stats[self.df_stats.record == rec].iloc[0].sig_len,
             critical_points=wfdb.rdann(
@@ -858,7 +895,7 @@ class CPSC2021(PhysioNetDataBase):
 
     def plot(
         self,
-        rec: str,
+        rec: Union[str, int],
         data: Optional[np.ndarray] = None,
         ann: Optional[Dict[str, np.ndarray]] = None,
         ticks_granularity: int = 0,
@@ -868,7 +905,7 @@ class CPSC2021(PhysioNetDataBase):
         waves: Optional[Dict[str, Sequence[int]]] = None,
         **kwargs,
     ) -> NoReturn:
-        """finished, checked, to improve,
+        """to improve,
 
         plot the signals of a record or external signals (units in μV),
         with metadata (labels, episodes of atrial fibrillation, etc.),
@@ -876,10 +913,10 @@ class CPSC2021(PhysioNetDataBase):
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         data: ndarray, optional,
-            (2-lead) ecg signal to plot,
+            (2-lead) ECG signal to plot,
             should be of the format "channel_first", and compatible with `leads`
             if given, data of `rec` will not be used,
             this is useful when plotting filtered data
@@ -915,7 +952,10 @@ class CPSC2021(PhysioNetDataBase):
         hence the isoelectric line is not plotted
 
         Contributors: Jeethan, and WEN Hao
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         if "plt" not in dir():
             import matplotlib.pyplot as plt
 
@@ -926,7 +966,7 @@ class CPSC2021(PhysioNetDataBase):
             _leads = [leads]
         else:
             _leads = leads
-        assert all([l in self.all_leads for l in _leads])
+        assert all([ld in self.all_leads for ld in _leads])
 
         if data is None:
             _data = self.load_data(
@@ -1124,8 +1164,7 @@ class CPSC2021(PhysioNetDataBase):
             plt.show()
 
     def _round(self, n: Real) -> int:
-        """finished, checked,
-
+        """
         dealing with round(0.5) = 0, hence keeping accordance with output length of `resample_poly`
         """
         return int(round(n + self._epsilon))
@@ -1137,7 +1176,7 @@ class CPSC2021(PhysioNetDataBase):
             return self._url_compressed
         # currently, cpsc2021 is not included in the list obtained
         # using `wfdb.get_dbs()`
-        self._url_compressed = f"https://www.physionet.org/static/published-projects/cpsc2021/paroxysmal-atrial-fibrillation-events-detection-from-dynamic-ecg-recordings-the-4th-china-physiological-signal-challenge-2021-{self.version}.zip"
+        self._url_compressed = f"https://www.physionet.org/static/published-projects/cpsc2021/paroxysmal-atrial-fibrillation-events-detection-from-dynamic-ECG-recordings-the-4th-china-physiological-signal-challenge-2021-{self.version}.zip"
         return self._url_compressed
 
 
@@ -1461,7 +1500,7 @@ def compute_challenge_metric(
     onset_score_range: Sequence[float],
     offset_score_range: Sequence[float],
 ) -> float:
-    """finished, checked,
+    """
 
     compute challenge metric for a single record
 
@@ -1500,7 +1539,7 @@ def gen_endpoint_score_mask(
     bias: dict = {1: 1, 2: 0.5},
     verbose: int = 0,
 ) -> Tuple[np.ndarray, np.ndarray]:
-    """finished, checked,
+    """
 
     generate the scoring mask for the onsets and offsets of af episodes,
 
