@@ -2,12 +2,13 @@
 data generator for feeding data into pytorch models
 """
 
-import json, time, textwrap
-from pathlib import Path
-import multiprocessing as mp
-from random import shuffle, randint, sample
+import json
+import textwrap
+import time
 from copy import deepcopy
-from typing import Union, Optional, List, Tuple, Dict, Sequence, Set, NoReturn
+from pathlib import Path
+from random import sample, shuffle
+from typing import List, NoReturn, Optional, Sequence, Set, Tuple
 
 import numpy as np
 
@@ -15,17 +16,18 @@ try:
     from tqdm.auto import tqdm
 except ModuleNotFoundError:
     from tqdm import tqdm
+
 import torch
 from torch.utils.data.dataset import Dataset
 
+from ...._preprocessors import PreprocManager
 from ....cfg import CFG
 from ....databases import CINC2021 as CR
-from ....utils.misc import ensure_siglen, dict_to_str, list_sum, ReprMixin
-from ....utils.utils_signal import normalize, remove_spikes_naive
 from ....utils.ecg_arrhythmia_knowledge import Standard12Leads
-from ...._preprocessors import PreprocManager
-
-from .cinc2021_cfg import CINC2021TrainCfg
+from ....utils.misc import ReprMixin, list_sum
+from torch_ecg.utils.utils_data import ensure_siglen
+from ....utils.utils_signal import remove_spikes_naive
+from .cinc2021_cfg import CINC2021TrainCfg  # noqa: F401
 
 
 __all__ = [
@@ -42,7 +44,7 @@ class CINC2021Dataset(ReprMixin, Dataset):
     def __init__(
         self, config: Optional[CFG] = None, training: bool = True, lazy: bool = True
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -57,6 +59,7 @@ class CINC2021Dataset(ReprMixin, Dataset):
             if True, the training set will be loaded, otherwise the test set
         lazy: bool, default True,
             if True, the data will not be loaded immediately,
+
         """
         super().__init__()
         self.config = deepcopy(config)
@@ -93,7 +96,7 @@ class CINC2021Dataset(ReprMixin, Dataset):
         self.siglen = self.config.input_len
         self.lazy = lazy
 
-        self._indices = [Standard12Leads.index(l) for l in self.config.leads]
+        self._indices = [Standard12Leads.index(ld) for ld in self.config.leads]
 
         self.records = self._train_test_split(
             self.config.train_ratio, force_recompute=False
@@ -138,18 +141,18 @@ class CINC2021Dataset(ReprMixin, Dataset):
         self._signals, self._labels = [], []
         with tqdm(desc="Loading data", total=len(fdr), unit="records") as pbar:
             for idx in range(len(fdr)):
-                s, l = fdr[idx]
+                sig, lb = fdr[idx]
                 # np.concatenate slows down the process severely
                 # self._signals = np.concatenate((self._signals, s), axis=0)
                 # self._labels = np.concatenate((self._labels, l), axis=0)
-                self._signals.append(s)
-                self._labels.append(l)
+                self._signals.append(sig)
+                self._labels.append(lb)
                 pbar.update(1)
         self._signals = np.concatenate(self._signals, axis=0).astype(self.dtype)
         self._labels = np.concatenate(self._labels, axis=0)
 
     def _load_one_record(self, rec: str) -> Tuple[np.ndarray, np.ndarray]:
-        """finished, checked,
+        """
 
         load a record from the database using data reader
 
@@ -169,6 +172,7 @@ class CINC2021Dataset(ReprMixin, Dataset):
             the values of the record
         labels: np.ndarray,
             the labels of the record
+
         """
         values = self.reader.load_resampled_data(
             rec,
@@ -177,8 +181,8 @@ class CINC2021Dataset(ReprMixin, Dataset):
             data_format=self.config.data_format,
             siglen=None,
         )
-        for l in range(values.shape[0]):
-            values[l] = remove_spikes_naive(values[l])
+        for idx in range(values.shape[0]):
+            values[idx] = remove_spikes_naive(values[idx])
         values, _ = self.ppm(values, self.config.fs)
         values = ensure_siglen(
             values,
@@ -202,7 +206,7 @@ class CINC2021Dataset(ReprMixin, Dataset):
         """ """
         prev_leads = self.config.leads
         self.config.leads = leads
-        self._indices = [prev_leads.index(l) for l in leads]
+        self._indices = [prev_leads.index(ld) for ld in leads]
         self._signals = self._signals[:, self._indices, :]
 
     def emtpy(self, leads: Optional[Sequence[str]] = None) -> NoReturn:
@@ -216,17 +220,17 @@ class CINC2021Dataset(ReprMixin, Dataset):
         )
 
     @classmethod
-    def from_extern(cls, ext_ds: "CINC2021", config: CFG) -> "CINC2021":
+    def from_extern(cls, ext_ds: "CINC2021Dataset", config: CFG) -> "CINC2021Dataset":
         """ """
         new_ds = cls(config, ext_ds.training, lazy=True)
-        indices = [ext_ds.config.leads.index(l) for l in new_ds.config.leads]
+        indices = [ext_ds.config.leads.index(ld) for ld in new_ds.config.leads]
         new_ds._signals = ext_ds._signals[:, indices, :]
         new_ds._labels = ext_ds._labels.copy()
         return new_ds
 
-    def reload_from_extern(self, ext_ds: "CINC2021") -> NoReturn:
+    def reload_from_extern(self, ext_ds: "CINC2021Dataset") -> NoReturn:
         """ """
-        indices = [ext_ds.config.leads.index(l) for l in self.config.leads]
+        indices = [ext_ds.config.leads.index(ld) for ld in self.config.leads]
         self._signals = ext_ds._signals[:, indices, :]
         self._labels = ext_ds._labels.copy()
 
@@ -241,7 +245,7 @@ class CINC2021Dataset(ReprMixin, Dataset):
         return self._labels
 
     def __getitem__(self, index: int) -> Tuple[np.ndarray, np.ndarray]:
-        """finished, checked,"""
+        """ """
         return self.signals[index], self.labels[index]
 
     def __len__(self) -> int:
@@ -251,7 +255,7 @@ class CINC2021Dataset(ReprMixin, Dataset):
     def _train_test_split(
         self, train_ratio: float = 0.8, force_recompute: bool = False
     ) -> List[str]:
-        """finished, checked,
+        """
 
         do train test split,
         it is ensured that both the train and the test set contain all classes
@@ -359,7 +363,7 @@ class CINC2021Dataset(ReprMixin, Dataset):
     def _check_train_test_split_validity(
         self, train_set: List[str], test_set: List[str], all_classes: Set[str]
     ) -> bool:
-        """finished, checked,
+        """
 
         the train-test split is valid iff
         records in both `train_set` and `test` contain all classes in `all_classes`
@@ -377,6 +381,7 @@ class CINC2021Dataset(ReprMixin, Dataset):
         -------
         is_valid: bool,
             the split is valid or not
+
         """
         train_classes = set(
             list_sum([self.reader.get_labels(rec, fmt="a") for rec in train_set])
@@ -400,9 +405,9 @@ class CINC2021Dataset(ReprMixin, Dataset):
         return is_valid
 
     def persistence(self) -> NoReturn:
-        """finished, checked,
-
+        """
         make the dataset persistent w.r.t. the tranches and the ratios in `self.config`
+
         """
         _TRANCHES = "ABEFG"
         if self.training:
@@ -432,10 +437,10 @@ class CINC2021Dataset(ReprMixin, Dataset):
         print(f"y saved to {filename}")
 
     def _check_nan(self) -> NoReturn:
-        """finished, checked,
-
+        """
         during training, sometimes nan values are encountered,
         which ruins the whole training process
+
         """
         for idx, (values, labels) in enumerate(self):
             if np.isnan(values).any():
@@ -485,8 +490,8 @@ class FastDataReader(ReprMixin, Dataset):
             data_format=self.config.data_format,
             siglen=None,
         )
-        for l in range(values.shape[0]):
-            values[l] = remove_spikes_naive(values[l])
+        for idx in range(values.shape[0]):
+            values[idx] = remove_spikes_naive(values[idx])
         if self.ppm:
             values, _ = self.ppm(values, self.config.fs)
         values = ensure_siglen(
@@ -534,6 +539,7 @@ def _load_record(reader: CR, rec: str, config: CFG) -> Tuple[np.ndarray, np.ndar
         the values of the record
     labels: np.ndarray,
         the labels of the record
+
     """
     values = reader.load_resampled_data(
         rec, leads=config.leads, data_format="channel_first", siglen=None
