@@ -33,15 +33,16 @@ for idx, seg in enumerate(err_list):
 ```
 
 and similarly for the task of `rr_lstm`
+
 """
 
-import os, json, re, time
-from pathlib import Path
-import multiprocessing as mp
-import random
-from itertools import repeat
+import json
+import os
+import re
+import time
 from copy import deepcopy
-from typing import Union, Optional, List, Tuple, Dict, Sequence, Set, NoReturn
+from pathlib import Path
+from typing import Dict, List, NoReturn, Optional, Sequence, Tuple, Union
 
 import numpy as np
 from scipy import signal as SS
@@ -50,23 +51,18 @@ try:
     from tqdm.auto import tqdm
 except ModuleNotFoundError:
     from tqdm import tqdm
+
 import torch
-from torch.utils.data.dataset import Dataset
 from scipy.io import loadmat, savemat
+from torch.utils.data.dataset import Dataset
 
-from ....cfg import CFG
-from ....databases import CPSC2021 as CR
 from ...._preprocessors import PreprocManager
-from ....utils.utils_interval import mask_to_intervals
-from ....utils.utils_signal import normalize, remove_spikes_naive
-from ....utils.misc import (
-    list_sum,
-    nildent,
-    uniform,
-    get_record_list_recursive3,
-    ReprMixin,
-)
-
+from ....databases import CPSC2021 as CR
+from ....utils.misc import ReprMixin, get_record_list_recursive3, list_sum, nildent
+from ....utils.utils_data import mask_to_intervals
+from ....utils.utils_signal import remove_spikes_naive
+from ....utils.utils_data import generate_weight_mask
+from ....cfg import CFG, DEFAULTS
 from .cpsc2021_cfg import CPSC2021TrainCfg
 
 
@@ -81,6 +77,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
     2. preprocessed ECGs are sliced with overlap to generate data and label for different tasks:
         the data files stores segments of fixed length of preprocessed ECGs,
         the annotation files contain "qrs_mask", and "af_mask"
+
     """
 
     __DEBUG__ = False
@@ -89,7 +86,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
     def __init__(
         self, config: CFG, task: str, training: bool = True, lazy: bool = True
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -104,6 +101,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             if True, the training set will be loaded, otherwise the test set
         lazy: bool, default False,
             if True, the data will not be loaded immediately,
+
         """
         super().__init__()
         self.config = deepcopy(config)
@@ -143,12 +141,12 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         # segments_dir for sliced segments of fixed length
         self.segments_base_dir = self.config.db_dir / "segments"
         self.segments_base_dir.mkdir(parents=True, exist_ok=True)
-        self.segment_name_pattern = "S_\d{1,3}_\d{1,2}_\d{7}"
+        self.segment_name_pattern = "S_[\\d]{1,3}_[\\d]{1,2}_[\\d]{7}"
         self.segment_ext = "mat"
         # rr_dir for sequence of rr intervals of fix length
         self.rr_seq_base_dir = self.config.db_dir / "rr_seq"
         self.rr_seq_base_dir.mkdir(parents=True, exist_ok=True)
-        self.rr_seq_name_pattern = "R_\d{1,3}_\d{1,2}_\d{7}"
+        self.rr_seq_name_pattern = "R_[\\d]{1,3}_[\\d]{1,2}_[\\d]{7}"
         self.rr_seq_ext = "mat"
 
         self._all_data = None
@@ -161,12 +159,15 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         self.__set_task(self.task, lazy=False)
 
     def __set_task(self, task: str, lazy: bool = True) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
         task: str,
             name of the task, can be one of `CPSC2021TrainCfg.tasks`
+        lazy: bool, default True,
+            if True, the data will not be loaded immediately,
+
         """
         assert task.lower() in CPSC2021TrainCfg.tasks, f"illegal task \042{task}\042"
         if (
@@ -204,11 +205,11 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                 [self.__all_segments[subject] for subject in self.subjects]
             )
             if self.__DEBUG__:
-                self.segments = random.sample(
+                self.segments = DEFAULTS.RNG_sample(
                     self.segments, int(len(self.segments) * 0.01)
-                )
+                ).tolist()
             if self.training:
-                random.shuffle(self.segments)
+                DEFAULTS.RNG.shuffle(self.segments)
             # preload data
             self.fdr = FastDataReader(
                 self.config,
@@ -246,9 +247,11 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                 [self.__all_rr_seq[subject] for subject in self.subjects]
             )
             if self.__DEBUG__:
-                self.rr_seq = random.sample(self.rr_seq, int(len(self.rr_seq) * 0.01))
+                self.rr_seq = DEFAULTS.RNG_sample(
+                    self.rr_seq, int(len(self.rr_seq) * 0.01)
+                ).tolist()
             if self.training:
-                random.shuffle(self.rr_seq)
+                DEFAULTS.RNG.shuffle(self.rr_seq)
             # preload data
             self.fdr = FastDataReader(
                 self.config,
@@ -278,14 +281,11 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             )
 
     def reset_task(self, task: str, lazy: bool = True) -> NoReturn:
-        """finished, checked,"""
+        """ """
         self.__set_task(task, lazy)
 
     def _ls_segments(self) -> NoReturn:
-        """finished, checked,
-
-        list all the segments
-        """
+        """list all the segments"""
         for item in ["data", "ann"]:
             self.segments_dirs[item] = CFG()
             for s in self.reader.all_subjects:
@@ -293,7 +293,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                 self.segments_dirs[item][s].mkdir(parents=True, exist_ok=True)
         if self.segments_json.is_file():
             self.__all_segments = json.loads(self.segments_json.read_text())
-            return
+            # return
         print(
             f"please allow the reader a few minutes to collect the segments from {self.segments_base_dir}..."
         )
@@ -312,16 +312,13 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             )
 
     def _ls_rr_seq(self) -> NoReturn:
-        """finished, checked,
-
-        list all the rr sequences
-        """
+        """list all the rr sequences"""
         for s in self.reader.all_subjects:
             self.rr_seq_dirs[s] = self.rr_seq_base_dir / s
             self.rr_seq_dirs[s].mkdir(parents=True, exist_ok=True)
         if self.rr_seq_json.is_file():
             self.__all_rr_seq = json.loads(self.rr_seq_json.read_text())
-            return
+            # return
         print(
             f"please allow the reader a few minutes to collect the rr sequences from {self.rr_seq_base_dir}..."
         )
@@ -378,7 +375,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                 )
 
     def _get_seg_data_path(self, seg: str) -> Path:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -389,13 +386,14 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         -------
         fp: Path,
             path of the data file of the segment
+
         """
         subject = seg.split("_")[1]
         fp = self.segments_dirs.data[subject] / f"{seg}.{self.segment_ext}"
         return fp
 
     def _get_seg_ann_path(self, seg: str) -> Path:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -406,13 +404,14 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         -------
         fp: Path,
             path of the annotation file of the segment
+
         """
         subject = seg.split("_")[1]
         fp = self.segments_dirs.ann[subject] / f"{seg}.{self.segment_ext}"
         return fp
 
     def _load_seg_data(self, seg: str) -> np.ndarray:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -423,13 +422,14 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         -------
         seg_data: ndarray,
             data of the segment, of shape (2, `self.seglen`)
+
         """
         seg_data_fp = self._get_seg_data_path(seg)
         seg_data = loadmat(str(seg_data_fp))["ecg"]
         return seg_data
 
     def _load_seg_ann(self, seg: str) -> dict:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -444,6 +444,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             - qrs_mask: mask of qrs complexes of the segment
             - af_mask: mask of af episodes of the segment
             - interval: interval ([start_idx, end_idx]) in the original ECG record of the segment
+
         """
         seg_ann_fp = self._get_seg_ann_path(seg)
         seg_ann = {
@@ -456,7 +457,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
     def _load_seg_mask(
         self, seg: str, task: Optional[str] = None
     ) -> Union[np.ndarray, Dict[str, np.ndarray]]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -471,6 +472,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         seg_mask: np.ndarray or dict,
             mask(s) of the segment,
             of shape (self.seglen, self.n_classes)
+
         """
         seg_mask = {
             k: v.reshape((self.seglen, -1))
@@ -495,7 +497,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         return seg_mask
 
     def _load_seg_seq_lab(self, seg: str, reduction: int) -> np.ndarray:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -510,6 +512,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         seq_lab: np.ndarray,
             label of the sequence,
             of shape (self.seglen//reduction, self.n_classes)
+
         """
         seg_mask = self._load_seg_mask(seg)
         seg_len, n_classes = seg_mask.shape
@@ -527,7 +530,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         return seq_lab
 
     def _get_rr_seq_path(self, seq_name: str) -> Path:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -538,13 +541,14 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         -------
         fp: Path,
             path of the annotation file of the rr_seq
+
         """
         subject = seq_name.split("_")[1]
         fp = self.rr_seq_dirs[subject] / f"{seq_name}.{self.rr_seq_ext}"
         return fp
 
     def _load_rr_seq(self, seq_name: str) -> Dict[str, np.ndarray]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -558,6 +562,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             - rr: the sequence of rr intervals, with units in seconds, of shape (self.seglen, 1)
             - label: label of the rr intervals, 0 for normal, 1 for af, of shape (self.seglen, self.n_classes)
             - interval: interval of the current rr sequence in the whole rr sequence in the original record
+
         """
         rr_seq_path = self._get_rr_seq_path(seq_name)
         rr_seq = {
@@ -569,7 +574,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         return rr_seq
 
     def persistence(self, force_recompute: bool = False, verbose: int = 0) -> NoReturn:
-        """finished, checked,
+        """
 
         make the dataset persistent w.r.t. the ratios in `self.config`
 
@@ -579,6 +584,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             if True, recompute regardless of possible existing files
         verbose: int, default 0,
             print verbosity
+
         """
         if verbose >= 1:
             print(" preprocessing data ".center("#", 110))
@@ -602,7 +608,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
     def _preprocess_data(
         self, force_recompute: bool = False, verbose: int = 0
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         preprocesses the ecg data in advance for further use,
         offline for `self.persistence`
@@ -613,6 +619,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             if True, recompute regardless of possible existing files
         verbose: int, default 0,
             print verbosity
+
         """
         for idx, rec in enumerate(self.reader.all_records):
             self._preprocess_one_record(
@@ -626,7 +633,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
     def _preprocess_one_record(
         self, rec: str, force_recompute: bool = False, verbose: int = 0
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         preprocesses the ecg data in advance for further use,
         offline for `self.persistence`
@@ -639,6 +646,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             if True, recompute regardless of possible existing files
         verbose: int, default 0,
             print verbosity
+
         """
         suffix = self._get_rec_suffix(self.allowed_preproc)
         save_fp = self.preprocess_dir / f"{rec}-{suffix}.{self.segment_ext}"
@@ -649,7 +657,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         savemat(save_fp, {"ecg": pps}, format="5")
 
     def load_preprocessed_data(self, rec: str) -> np.ndarray:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -660,6 +668,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         -------
         p_sig: ndarray,
             the pre-computed processed ECG
+
         """
         preproc = self.allowed_preproc
         suffix = self._get_rec_suffix(preproc)
@@ -674,7 +683,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         return p_sig
 
     def _get_rec_suffix(self, operations: List[str]) -> str:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -686,12 +695,13 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         -------
         suffix: str,
             suffix of the filename of the preprocessed ecg signal
+
         """
         suffix = "-".join(sorted([item.lower() for item in operations]))
         return suffix
 
     def _slice_data(self, force_recompute: bool = False, verbose: int = 0) -> NoReturn:
-        """finished, checked,
+        """
 
         slice all records into segments of length `self.seglen`,
         and perform data augmentations specified in `self.config`
@@ -702,6 +712,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             if True, recompute regardless of possible existing files
         verbose: int, default 0,
             print verbosity
+
         """
         self.__assert_task(
             [
@@ -721,8 +732,9 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             if verbose >= 1:
                 print(f"{idx+1}/{len(self.reader.all_records)} records", end="\r")
         if force_recompute:
-            with open(self.segments_json, "w") as f:
-                json.dump(self.__all_segments, f)
+            self.segments_json.write_text(
+                json.dump(self.__all_segments, ensure_ascii=False)
+            )
 
     def _slice_one_record(
         self,
@@ -731,7 +743,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         update_segments_json: bool = False,
         verbose: int = 0,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         slice one record into segments of length `self.seglen`,
         and perform data augmentations specified in `self.config`
@@ -748,6 +760,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             useful when slicing not all records
         verbose: int, default 0,
             print verbosity
+
         """
         self.__assert_task(
             [
@@ -812,7 +825,9 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                 0,
                 cp
                 - self.seglen
-                + random.randint(critical_forward_len[0], critical_forward_len[1]),
+                + DEFAULTS.RNG_randint(
+                    critical_forward_len[0], critical_forward_len[1]
+                ),
             )
             while start_idx <= min(cp - critical_forward_len[1], siglen - self.seglen):
                 new_seg = self.__generate_segment(
@@ -821,7 +836,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                     start_idx=start_idx,
                 )
                 segments.append(new_seg)
-                start_idx += random.randint(
+                start_idx += DEFAULTS.RNG_randint(
                     critical_forward_len[0], critical_forward_len[1]
                 )
 
@@ -835,7 +850,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         start_idx: Optional[int] = None,
         end_idx: Optional[int] = None,
     ) -> CFG:
-        """finished, checked,
+        """
 
         generate segment, with possible data augmentation
 
@@ -861,6 +876,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             - qrs_mask: mask of qrs complexes of the segment
             - af_mask: mask of af episodes of the segment
             - interval: interval ([start_idx, end_idx]) in the original ECG record of the segment
+
         """
         assert not all(
             [start_idx is None, end_idx is None]
@@ -868,10 +884,12 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         siglen = data.shape[1]
         # offline augmentations are done, including strech-or-compress, ...
         if self.config.stretch_compress != 0:
-            sign = random.sample(self.config.stretch_compress_choices, 1)[0]
+            sign = DEFAULTS.RNG_sample(self.config.stretch_compress_choices, 1)[0]
             if sign != 0:
                 sc_ratio = self.config.stretch_compress
-                sc_ratio = 1 + (random.uniform(sc_ratio / 4, sc_ratio) * sign) / 100
+                sc_ratio = (
+                    1 + (DEFAULTS.RNG.uniform(sc_ratio / 4, sc_ratio) * sign) / 100
+                )
                 sc_len = int(round(sc_ratio * self.seglen))
                 if start_idx is not None:
                     end_idx = start_idx + sc_len
@@ -915,7 +933,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             rec=rec,
             sampfrom=start_idx,
             sampto=end_idx,
-            zero_start=True,
+            keep_original=False,
         )
         seg_rpeaks = [
             int(round(r / sc_ratio))
@@ -935,7 +953,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             rec=rec,
             sampfrom=start_idx,
             sampto=end_idx,
-            zero_start=True,
+            keep_original=False,
             fmt="intervals",
         )
         seg_af_intervals = [
@@ -959,7 +977,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
     def __save_segments(
         self, rec: str, segments: List[CFG], update_segments_json: bool = False
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -969,16 +987,17 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             list of the segments (meta-)data
         update_segments_json: bool, default False,
             if True, the file `self.segments_json` will be updated
+
         """
         subject = self.reader.get_subject_id(rec)
         ordering = list(range(len(segments)))
-        random.shuffle(ordering)
+        DEFAULTS.RNG.shuffle(ordering)
         for i, idx in enumerate(ordering):
             seg = segments[idx]
             filename = f"{rec}_{i:07d}.{self.segment_ext}".replace("data", "S")
             data_path = self.segments_dirs.data[subject] / filename
             savemat(str(data_path), {"ecg": seg.data})
-            self.__all_segments[subject].append(Path(filename).with_suffix(""))
+            self.__all_segments[subject].append(Path(filename).with_suffix("").name)
             ann_path = self.segments_dirs.ann[subject] / filename
             savemat(
                 str(ann_path),
@@ -997,13 +1016,14 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             )
 
     def _clear_cached_segments(self, recs: Optional[Sequence[str]] = None) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
         recs: sequence of str, optional
             sequence of the records whose segments are to be cleared,
             defaults to all records
+
         """
         self.__assert_task(
             [
@@ -1024,7 +1044,10 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                     ]:
                         if self._get_rec_name(f) == rec:
                             os.remove(os.path.join(path, f))
-                            self.__all_segments[subject].remove(os.path.splitext(f)[0])
+                            if os.path.splitext(f)[0] in self.__all_segments[subject]:
+                                self.__all_segments[subject].remove(
+                                    os.path.splitext(f)[0]
+                                )
         else:
             for subject in self.reader.all_subjects:
                 for item in [
@@ -1036,7 +1059,8 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                         n for n in os.listdir(path) if n.endswith(self.segment_ext)
                     ]:
                         os.remove(os.path.join(path, f))
-                        self.__all_segments[subject].remove(os.path.splitext(f)[0])
+                        if os.path.splitext(f)[0] in self.__all_segments[subject]:
+                            self.__all_segments[subject].remove(os.path.splitext(f)[0])
         self.segments = list_sum(
             [self.__all_segments[subject] for subject in self.subjects]
         )
@@ -1044,7 +1068,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
     def _slice_rr_seq(
         self, force_recompute: bool = False, verbose: int = 0
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         slice sequences of rr intervals into fixed length (sub)sequences
 
@@ -1054,6 +1078,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             if True, recompute regardless of possible existing files
         verbose: int, default 0,
             print verbosity
+
         """
         self.__assert_task(["rr_lstm"])
         if force_recompute:
@@ -1068,8 +1093,9 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             if verbose >= 1:
                 print(f"{idx+1}/{len(self.reader.all_records)} records", end="\r")
         if force_recompute:
-            with open(self.rr_seq_json, "w") as f:
-                json.dump(self.__all_rr_seq, f)
+            self.rr_seq_json.write_text(
+                json.dumps(self.__all_rr_seq, ensure_ascii=False)
+            )
 
     def _slice_rr_seq_one_record(
         self,
@@ -1078,7 +1104,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         update_rr_seq_json: bool = False,
         verbose: int = 0,
     ) -> NoReturn:
-        """finished, checked,"""
+        """ """
         self.__assert_task(["rr_lstm"])
         subject = self.reader.get_subject_id(rec)
         rec_rr_seq = [
@@ -1123,14 +1149,15 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             )
             rr_seq.append(new_rr_seq)
         # the tail segment
-        end_idx = len(rr)
-        start_idx = start_idx + self.seglen
-        new_rr_seq = CFG(
-            rr=rr[start_idx:end_idx],
-            label=label_seq[start_idx:end_idx],
-            interval=[start_idx, end_idx],
-        )
-        rr_seq.append(new_rr_seq)
+        if end_idx < len(rr):
+            end_idx = len(rr)
+            start_idx = end_idx - self.seglen
+            new_rr_seq = CFG(
+                rr=rr[start_idx:end_idx],
+                label=label_seq[start_idx:end_idx],
+                interval=[start_idx, end_idx],
+            )
+            rr_seq.append(new_rr_seq)
 
         # special segments around critical_points with random forward_len in critical_forward_len
         for cp in critical_points:
@@ -1138,7 +1165,9 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                 0,
                 cp
                 - self.seglen
-                + random.randint(critical_forward_len[0], critical_forward_len[1]),
+                + DEFAULTS.RNG_randint(
+                    critical_forward_len[0], critical_forward_len[1]
+                ),
             )
             while start_idx <= min(cp - critical_forward_len[1], len(rr) - self.seglen):
                 end_idx = start_idx + self.seglen
@@ -1148,31 +1177,50 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                     interval=[start_idx, end_idx],
                 )
                 rr_seq.append(new_rr_seq)
-                start_idx += random.randint(
+                start_idx += DEFAULTS.RNG_randint(
                     critical_forward_len[0], critical_forward_len[1]
                 )
         # save rr sequences
+        self.__save_rr_seq(rec, rr_seq, update_rr_seq_json)
+
+    def __save_rr_seq(
+        self, rec: str, rr_seq: List[CFG], update_rr_seq_json: bool = False
+    ) -> NoReturn:
+        """
+
+        Parameters
+        ----------
+        rec: str,
+            filename of the record
+        rr_seq: list of dict,
+            list of the rr_seq (meta-)data
+        update_rr_seq_json: bool, default False,
+            if True, the file `self.rr_seq_json` will be updated
+
+        """
+        subject = self.reader.get_subject_id(rec)
         ordering = list(range(len(rr_seq)))
-        random.shuffle(ordering)
+        DEFAULTS.RNG.shuffle(ordering)
         for i, idx in enumerate(ordering):
             item = rr_seq[idx]
             filename = f"{rec}_{i:07d}.{self.rr_seq_ext}".replace("data", "R")
             data_path = self.rr_seq_dirs[subject] / filename
             savemat(str(data_path), item)
-            self.__all_rr_seq[subject].append(Path(filename).with_suffix(""))
+            self.__all_rr_seq[subject].append(Path(filename).with_suffix("").name)
         if update_rr_seq_json:
             self.rr_seq_json.write_text(
                 json.dumps(self.__all_rr_seq, ensure_ascii=False)
             )
 
     def _clear_cached_rr_seq(self, recs: Optional[Sequence[str]] = None) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
         recs: sequence of str, optional
             sequence of the records whose segments are to be cleared,
             defaults to all records
+
         """
         self.__assert_task(["rr_lstm"])
         if recs is not None:
@@ -1182,19 +1230,21 @@ class CPSC2021Dataset(ReprMixin, Dataset):
                 for f in [n for n in os.listdir(path) if n.endswith(self.rr_seq_ext)]:
                     if self._get_rec_name(f) == rec:
                         os.remove(os.path.join(path, f))
-                        self.__all_rr_seq[subject].remove(os.path.splitext(f)[0])
+                        if os.path.splitext(f)[0] in self.__all_rr_seq[subject]:
+                            self.__all_rr_seq[subject].remove(os.path.splitext(f)[0])
         else:
             for subject in self.reader.all_subjects:
                 path = str(self.rr_seq_dirs[subject])
                 for f in [n for n in os.listdir(path) if n.endswith(self.rr_seq_ext)]:
                     os.remove(os.path.join(path, f))
-                    self.__all_rr_seq[subject].remove(os.path.splitext(f)[0])
+                    if os.path.splitext(f)[0] in self.__all_rr_seq[subject]:
+                        self.__all_rr_seq[subject].remove(os.path.splitext(f)[0])
         self.rr_seq = list_sum(
             [self.__all_rr_seq[subject] for subject in self.subjects]
         )
 
     def _get_rec_name(self, seg_or_rr: str) -> str:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -1205,6 +1255,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         -------
         rec: str,
             name of the record that `seg` was generated from
+
         """
         rec = re.sub("[RS]", "data", os.path.splitext(seg_or_rr)[0])[:-8]
         return rec
@@ -1212,7 +1263,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
     def _train_test_split(
         self, train_ratio: float = 0.8, force_recompute: bool = False
     ) -> Dict[str, List[str]]:
-        """finished, checked,
+        """
 
         do train test split,
         it is ensured that both the train and the test set contain all classes
@@ -1230,6 +1281,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         split_res: dict,
             keys are "train" and "test",
             values are list of the subjects split for training or validation
+
         """
         start = time.time()
         print("\nstart performing train test split...\n")
@@ -1258,23 +1310,23 @@ class CPSC2021Dataset(ReprMixin, Dataset):
             normal_subjects = all_subjects - afp_subjects - aff_subjects
 
             test_set = (
-                random.sample(
+                DEFAULTS.RNG_sample(
                     afp_subjects,
                     max(1, int(round(len(afp_subjects) * _test_ratio / 100))),
-                )
-                + random.sample(
+                ).tolist()
+                + DEFAULTS.RNG_sample(
                     aff_subjects,
                     max(1, int(round(len(aff_subjects) * _test_ratio / 100))),
-                )
-                + random.sample(
+                ).tolist()
+                + DEFAULTS.RNG_sample(
                     normal_subjects,
                     max(1, int(round(len(normal_subjects) * _test_ratio / 100))),
-                )
+                ).tolist()
             )
             train_set = list(all_subjects - set(test_set))
 
-            random.shuffle(test_set)
-            random.shuffle(train_set)
+            DEFAULTS.RNG.shuffle(test_set)
+            DEFAULTS.RNG.shuffle(train_set)
 
             train_file.write_text(json.dumps(train_set, ensure_ascii=False))
             test_file.write_text(json.dumps(test_set, ensure_ascii=False))
@@ -1307,7 +1359,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         ), f"DO NOT call this method when the current task is {self.task}. Switch task using `reset_task`"
 
     def plot_seg(self, seg: str, ticks_granularity: int = 0) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -1316,6 +1368,7 @@ class CPSC2021Dataset(ReprMixin, Dataset):
         ticks_granularity: int, default 0,
             the granularity to plot axis ticks, the higher the more,
             0 (no ticks) --> 1 (major ticks) --> 2 (major + minor ticks)
+
         """
         seg_data = self._load_seg_data(seg)
         print(f"seg_data.shape = {seg_data.shape}")
@@ -1376,8 +1429,8 @@ class FastDataReader(ReprMixin, Dataset):
             subject = seg_name.split("_")[1]
             seg_data_fp = self.file_dirs.data[subject] / f"{seg_name}.{self.file_ext}"
             seg_data = loadmat(str(seg_data_fp))["ecg"]
-            for l in range(seg_data.shape[0]):
-                seg_data[l] = remove_spikes_naive(seg_data[l])
+            for idx in range(seg_data.shape[0]):
+                seg_data[idx] = remove_spikes_naive(seg_data[idx])
             seg_ann_fp = self.file_dirs.ann[subject] / f"{seg_name}.{self.file_ext}"
             seg_label = loadmat(str(seg_ann_fp))[self._seg_keys[self.task]].reshape(
                 (self.seglen, -1)
@@ -1388,7 +1441,7 @@ class FastDataReader(ReprMixin, Dataset):
                 seg_label = np.stack(
                     arrays=[
                         np.mean(
-                            seg_mask[reduction * idx : reduction * (idx + 1)],
+                            seg_data[reduction * idx : reduction * (idx + 1)],
                             axis=0,
                             keepdims=True,
                         ).astype(int)
@@ -1398,7 +1451,7 @@ class FastDataReader(ReprMixin, Dataset):
                 ).squeeze(axis=1)
             seg_data, _ = self.seg_ppm(seg_data, self.config.fs)
             if self.task == "main":
-                weight_mask = _generate_weight_mask(
+                weight_mask = generate_weight_mask(
                     target_mask=seg_label.squeeze(-1),
                     fg_weight=2,
                     fs=self.config.fs,
@@ -1417,7 +1470,7 @@ class FastDataReader(ReprMixin, Dataset):
             rr_seq = loadmat(str(rr_seq_path))
             rr_seq["rr"] = rr_seq["rr"].reshape((self.seglen, 1))
             rr_seq["label"] = rr_seq["label"].reshape((self.seglen, self.n_classes))
-            weight_mask = _generate_weight_mask(
+            weight_mask = generate_weight_mask(
                 target_mask=rr_seq["label"].squeeze(-1),
                 fg_weight=2,
                 fs=1 / 0.8,
@@ -1524,7 +1577,9 @@ class StandaloneSegmentSlicer(ReprMixin, Dataset):
                 0,
                 cp
                 - self.seglen
-                + random.randint(critical_forward_len[0], critical_forward_len[1]),
+                + DEFAULTS.RNG_randint(
+                    critical_forward_len[0], critical_forward_len[1]
+                ),
             )
             while start_idx <= min(cp - critical_forward_len[1], siglen - self.seglen):
                 new_seg = self.__generate_segment(
@@ -1533,7 +1588,7 @@ class StandaloneSegmentSlicer(ReprMixin, Dataset):
                     start_idx=start_idx,
                 )
                 segments.append(new_seg)
-                start_idx += random.randint(
+                start_idx += DEFAULTS.RNG_randint(
                     critical_forward_len[0], critical_forward_len[1]
                 )
 
@@ -1546,7 +1601,7 @@ class StandaloneSegmentSlicer(ReprMixin, Dataset):
         start_idx: Optional[int] = None,
         end_idx: Optional[int] = None,
     ) -> CFG:
-        """finished, checked,
+        """
 
         generate segment, with possible data augmentation
 
@@ -1572,6 +1627,7 @@ class StandaloneSegmentSlicer(ReprMixin, Dataset):
             - qrs_mask: mask of qrs complexes of the segment
             - af_mask: mask of af episodes of the segment
             - interval: interval ([start_idx, end_idx]) in the original ECG record of the segment
+
         """
         assert not all(
             [start_idx is None, end_idx is None]
@@ -1579,10 +1635,12 @@ class StandaloneSegmentSlicer(ReprMixin, Dataset):
         siglen = data.shape[1]
         # offline augmentations are done, including strech-or-compress, ...
         if self.config.stretch_compress != 0:
-            sign = random.sample(self.config.stretch_compress_choices, 1)[0]
+            sign = DEFAULTS.RNG_sample(self.config.stretch_compress_choices, 1)[0]
             if sign != 0:
                 sc_ratio = self.config.stretch_compress
-                sc_ratio = 1 + (random.uniform(sc_ratio / 4, sc_ratio) * sign) / 100
+                sc_ratio = (
+                    1 + (DEFAULTS.RNG.uniform(sc_ratio / 4, sc_ratio) * sign) / 100
+                )
                 sc_len = int(round(sc_ratio * self.seglen))
                 if start_idx is not None:
                     end_idx = start_idx + sc_len
@@ -1626,7 +1684,7 @@ class StandaloneSegmentSlicer(ReprMixin, Dataset):
             rec=rec,
             sampfrom=start_idx,
             sampto=end_idx,
-            zero_start=True,
+            keep_original=False,
         )
         seg_rpeaks = [
             int(round(r / sc_ratio))
@@ -1646,7 +1704,7 @@ class StandaloneSegmentSlicer(ReprMixin, Dataset):
             rec=rec,
             sampfrom=start_idx,
             sampto=end_idx,
-            zero_start=True,
+            keep_original=False,
             fmt="intervals",
         )
         seg_af_intervals = [
@@ -1668,7 +1726,7 @@ class StandaloneSegmentSlicer(ReprMixin, Dataset):
         return new_seg
 
     def load_preprocessed_data(self, rec: str) -> np.ndarray:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -1679,6 +1737,7 @@ class StandaloneSegmentSlicer(ReprMixin, Dataset):
         -------
         p_sig: ndarray,
             the pre-computed processed ECG
+
         """
         preproc = self.allowed_preproc
         suffix = _get_rec_suffix(preproc)
@@ -1701,7 +1760,7 @@ class StandaloneSegmentSlicer(ReprMixin, Dataset):
 
 
 def _get_rec_suffix(operations: List[str]) -> str:
-    """finished, checked,
+    """
 
     Parameters
     ----------
@@ -1712,66 +1771,7 @@ def _get_rec_suffix(operations: List[str]) -> str:
     -------
     suffix: str,
         suffix of the filename of the preprocessed ecg signal
+
     """
     suffix = "-".join(sorted([item.lower() for item in operations]))
     return suffix
-
-
-def _generate_weight_mask(
-    target_mask: np.ndarray,
-    fg_weight: float,
-    fs: int,
-    reduction: int,
-    radius: float,
-    boundary_weight: float,
-    plot: bool = False,
-) -> np.ndarray:
-    """finished, checked,
-
-    generate weight mask for a binary target mask,
-    accounting the foreground weight and boundary weight
-
-    Parameters
-    ----------
-    target_mask: ndarray,
-        the target mask, assumed to be 1d
-    fg_weight: float,
-        foreground weight, usually > 1
-    fs: int,
-        sampling frequency of the signal
-    reduction: int,
-        reduction ratio of the mask w.r.t. the signal
-    boundary_weight: float,
-        weight for the boundaries (positions where values change) of the target map
-    plot: bool, default False,
-        if True, target_mask and the result weight_mask will be plotted
-
-    Returns
-    -------
-    weight_mask: ndarray,
-        the weight mask
-    """
-    weight_mask = np.ones_like(target_mask, dtype=float)
-    sigma = int((radius * fs) / reduction)
-    weight = np.full_like(target_mask, fg_weight) - 1
-    weight_mask += (target_mask > 0.5) * weight
-    border = np.where(np.diff(target_mask) != 0)[0]
-    for idx in border:
-        # weight = np.zeros_like(target_mask, dtype=float)
-        # weight[max(0, idx-sigma): (idx+sigma)] = boundary_weight
-        weight = np.full_like(target_mask, boundary_weight, dtype=float)
-        weight = weight * np.exp(
-            -np.power(np.arange(len(target_mask)) - idx, 2) / sigma ** 2
-        )
-        weight_mask += weight
-    if plot:
-        import matplotlib.pyplot as plt
-
-        fig, ax = plt.subplots(figsize=(12, 6))
-        ax.plot(target_mask, label="target mask")
-        ax.plot(weight_mask, label="weight mask")
-        ax.set_xlabel("samples")
-        ax.set_ylabel("weight")
-        ax.legend(loc="best")
-        plt.show()
-    return weight_mask
