@@ -2,40 +2,43 @@
 """
 """
 
-import io, re, json, time, posixpath
-from pathlib import Path
+import io
+import json
+import posixpath
+import re
+import time
 from copy import deepcopy
 from datetime import datetime
-from typing import Union, Optional, Any, List, Dict, Tuple, Set, Sequence, NoReturn
-from numbers import Real, Number
+from numbers import Real
+from pathlib import Path
+from typing import Any, Dict, List, NoReturn, Optional, Sequence, Tuple, Union
 
 import numpy as np
 import pandas as pd
 import wfdb
 from scipy.io import loadmat
-from scipy.signal import resample, resample_poly
+from scipy.signal import resample, resample_poly  # noqa: F401
 
 from ...cfg import CFG
-from ...utils.misc import (
-    get_record_list_recursive,
-    get_record_list_recursive3,
-    ms2samples,
-    dict_to_str,
-    ensure_siglen,
-)
-from ...utils.download import http_get, _stem
 from ...utils import ecg_arrhythmia_knowledge as EAK
+from ...utils.download import _stem, http_get
+from ...utils.misc import (
+    add_docstring,
+    dict_to_str,
+    get_record_list_recursive3,
+    list_sum,
+    ms2samples,
+)
+from torch_ecg.utils.utils_data import ensure_siglen
 from ..aux_data.cinc2020_aux_data import (
+    df_weights_abbr,
     dx_mapping_all,
     dx_mapping_scored,
-    dx_mapping_unscored,
-    normalize_class,
-    abbr_to_snomed_ct_code,
-    df_weights_abbr,
     equiv_class_dict,
+    load_weights,
+    normalize_class,
 )
-from ..base import PhysioNetDataBase, DEFAULT_FIG_SIZE_PER_SEC
-
+from ..base import DEFAULT_FIG_SIZE_PER_SEC, PhysioNetDataBase
 
 __all__ = [
     "CINC2020",
@@ -157,14 +160,15 @@ class CINC2020(PhysioNetDataBase):
 
     References
     ----------
-    [1] https://physionetchallenges.github.io/2020/
-    [2] https://physionet.org/content/challenge-2020/1.0.1/
-    [3] http://2018.icbeb.org/#
-    [4] https://physionet.org/content/incartdb/1.0.0/
-    [5] https://physionet.org/content/ptbdb/1.0.0/
-    [6] https://physionet.org/content/ptb-xl/1.0.1/
-    [7] (deprecated) https://storage.cloud.google.com/physionet-challenge-2020-12-lead-ecg-public/
-    [8] (recommended) https://storage.cloud.google.com/physionetchallenge2021-public-datasets/
+    1. <a name="ref1"></a> https://physionetchallenges.github.io/2020/
+    2. <a name="ref2"></a> https://physionet.org/content/challenge-2020/1.0.1/
+    3. <a name="ref3"></a> http://2018.icbeb.org/#
+    4. <a name="ref4"></a> https://physionet.org/content/incartdb/1.0.0/
+    5. <a name="ref5"></a> https://physionet.org/content/ptbdb/1.0.0/
+    6. <a name="ref6"></a> https://physionet.org/content/ptb-xl/1.0.1/
+    7. <a name="ref7"></a> (deprecated) https://storage.cloud.google.com/physionet-challenge-2020-12-lead-ECG-public/
+    8. <a name="ref8"></a> (recommended) https://storage.cloud.google.com/physionetchallenge2021-public-datasets/
+
     """
 
     def __init__(
@@ -185,6 +189,7 @@ class CINC2020(PhysioNetDataBase):
         verbose: int, default 2,
             log verbosity
         kwargs: auxilliary key word arguments
+
         """
         super().__init__(
             db_name="challenge-2020",
@@ -222,6 +227,7 @@ class CINC2020(PhysioNetDataBase):
         self.db_dir_base = Path(db_dir)
         self.db_dirs = CFG({tranche: "" for tranche in self.db_tranches})
         self._all_records = None
+        self.__all_records = None
         self._ls_rec()  # loads file system structures into self.db_dirs and self._all_records
 
         self._diagnoses_records_list = None
@@ -271,19 +277,22 @@ class CINC2020(PhysioNetDataBase):
             "E08321",
         ]  # ref. ISSUES 4
 
-    def get_subject_id(self, rec: str) -> int:
-        """finished, checked,
+    def get_subject_id(self, rec: Union[str, int]) -> int:
+        """
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
 
         Returns
         -------
         sid: int,
             the `subject_id` corr. to `rec`
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         s2d = {"A": "11", "B": "12", "C": "21", "D": "31", "E": "32", "F": "41"}
         s2d = {self.rec_prefix[k]: v for k, v in s2d.items()}
         prefix = "".join(re.findall(r"[A-Z]", rec))
@@ -292,10 +301,10 @@ class CINC2020(PhysioNetDataBase):
         return sid
 
     def _ls_rec(self) -> NoReturn:
-        """finished, checked,
-
+        """
         list all the records and load into `self._all_records`,
         facilitating further uses
+
         """
         fn = "record_list.json"
         record_list_fp = self.db_dir_base / fn
@@ -318,7 +327,7 @@ class CINC2020(PhysioNetDataBase):
             )
             start = time.time()
             rec_patterns_with_ext = {
-                tranche: f"^{self.rec_prefix[tranche]}(?:\d+).{self.rec_ext}$"
+                tranche: f"^{self.rec_prefix[tranche]}(?:\\d+).{self.rec_ext}$"
                 for tranche in self.db_tranches
             }
             self._all_records = get_record_list_recursive3(
@@ -341,12 +350,10 @@ class CINC2020(PhysioNetDataBase):
                 ]
             print(f"Done in {time.time() - start:.5f} seconds!")
             record_list_fp.write_text(json.dumps(to_save))
+        self.__all_records = list_sum(self._all_records.values())
 
     def _ls_diagnoses_records(self) -> NoReturn:
-        """finished, checked,
-
-        list all the records for all diagnoses
-        """
+        """list all the records for all diagnoses"""
         fn = "diagnoses_records_list.json"
         dr_fp = self.db_dir_base / fn
         if dr_fp.is_file():
@@ -368,42 +375,46 @@ class CINC2020(PhysioNetDataBase):
             print(f"Done in {time.time() - start:.5f} seconds!")
             dr_fp.write_text(json.dumps(self._diagnoses_records_list))
         self._all_records = CFG(self._all_records)
+        self.__all_records = list_sum(self._all_records.values())
 
     @property
-    def diagnoses_records_list(self):
-        """finished, checked,"""
+    def diagnoses_records_list(self) -> Dict[str, List[str]]:
+        """ """
         if self._diagnoses_records_list is None:
             self._ls_diagnoses_records()
         return self._diagnoses_records_list
 
-    def _get_tranche(self, rec: str) -> str:
-        """finished, checked,
+    def _get_tranche(self, rec: Union[str, int]) -> str:
+        """
 
         get the tranche"s symbol (one of "A","B","C","D","E","F") of a record via its name
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
 
         Returns
         -------
         tranche, str,
             symbol of the tranche, ref. `self.rec_prefix`
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         prefix = "".join(re.findall(r"[A-Z]", rec))
         tranche = {v: k for k, v in self.rec_prefix.items()}[prefix]
         return tranche
 
-    def get_data_filepath(self, rec: str, with_ext: bool = True) -> str:
-        """finished, checked,
+    def get_data_filepath(self, rec: Union[str, int], with_ext: bool = True) -> str:
+        """
 
         get the absolute file path of the data file of `rec`
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         with_ext: bool, default True,
             if True, the returned file path comes with file extension,
             otherwise without file extension,
@@ -413,15 +424,18 @@ class CINC2020(PhysioNetDataBase):
         -------
         fp: str,
             absolute file path of the data file of the record
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         tranche = self._get_tranche(rec)
         fp = self.db_dirs[tranche] / f"{rec}.{self.rec_ext}"
         if not with_ext:
             fp = fp.with_suffix("")
         return str(fp)
 
-    def get_header_filepath(self, rec: str, with_ext: bool = True) -> str:
-        """finished, checked,
+    def get_header_filepath(self, rec: Union[str, int], with_ext: bool = True) -> str:
+        """
 
         get the absolute file path of the header file of `rec`
 
@@ -438,15 +452,19 @@ class CINC2020(PhysioNetDataBase):
         -------
         fp: str,
             absolute file path of the header file of the record
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         tranche = self._get_tranche(rec)
         fp = self.db_dirs[tranche] / f"{rec}.{self.ann_ext}"
         if not with_ext:
             fp = fp.with_suffix("")
         return str(fp)
 
-    def get_ann_filepath(self, rec: str, with_ext: bool = True) -> str:
-        """finished, checked,
+    @add_docstring(get_header_filepath.__doc__)
+    def get_ann_filepath(self, rec: Union[str, int], with_ext: bool = True) -> str:
+        """
         alias for `get_header_filepath`
         """
         fp = self.get_header_filepath(rec, with_ext=with_ext)
@@ -454,26 +472,26 @@ class CINC2020(PhysioNetDataBase):
 
     def load_data(
         self,
-        rec: str,
+        rec: Union[str, int],
         leads: Optional[Union[str, List[str]]] = None,
         data_format: str = "channel_first",
         backend: str = "wfdb",
         units: str = "mV",
         fs: Optional[Real] = None,
     ) -> np.ndarray:
-        """finished, checked,
+        """
 
-        load physical (converted from digital) ecg data,
+        load physical (converted from digital) ECG data,
         which is more understandable for humans
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         leads: str or list of str, optional,
             the leads to load
         data_format: str, default "channel_first",
-            format of the ecg data,
+            format of the ECG data,
             "channel_last" (alias "lead_last"), or
             "channel_first" (alias "lead_first")
         backend: str, default "wfdb",
@@ -486,8 +504,11 @@ class CINC2020(PhysioNetDataBase):
         Returns
         -------
         data: ndarray,
-            the ecg data
+            the ECG data
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         assert data_format.lower() in [
             "channel_first",
             "lead_first",
@@ -540,16 +561,16 @@ class CINC2020(PhysioNetDataBase):
         return data
 
     def load_ann(
-        self, rec: str, raw: bool = False, backend: str = "wfdb"
+        self, rec: Union[str, int], raw: bool = False, backend: str = "wfdb"
     ) -> Union[dict, str]:
-        """finished, checked,
+        """
 
         load annotations (header) stored in the .hea files
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         raw: bool, default False,
             if True, the raw annotations without parsing will be returned
         backend: str, default "wfdb", case insensitive,
@@ -560,7 +581,10 @@ class CINC2020(PhysioNetDataBase):
         -------
         ann_dict, dict or str,
             the annotations with items: ref. `self.ann_items`
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         tranche = self._get_tranche(rec)
         ann_fp = self.get_ann_filepath(rec, with_ext=True)
         header_data = Path(ann_fp).read_text().splitlines()
@@ -579,13 +603,13 @@ class CINC2020(PhysioNetDataBase):
             )
         return ann_dict
 
-    def _load_ann_wfdb(self, rec: str, header_data: List[str]) -> dict:
-        """finished, checked,
+    def _load_ann_wfdb(self, rec: Union[str, int], header_data: List[str]) -> dict:
+        """
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         header_data: list of str,
             list of lines read directly from a header file,
             complementary to data read using `wfdb.rdheader` if applicable,
@@ -595,7 +619,10 @@ class CINC2020(PhysioNetDataBase):
         -------
         ann_dict, dict,
             the annotations with items: ref. `self.ann_items`
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         header_fp = self.get_header_filepath(rec, with_ext=False)
         header_reader = wfdb.rdheader(header_fp)
         ann_dict = {}
@@ -616,37 +643,39 @@ class CINC2020(PhysioNetDataBase):
         )
         try:  # see NOTE. 1.
             ann_dict["age"] = int(
-                [l for l in header_reader.comments if "Age" in l][0].split(": ")[-1]
+                [line for line in header_reader.comments if "Age" in line][0].split(
+                    ": "
+                )[-1]
             )
-        except:
+        except Exception:
             ann_dict["age"] = np.nan
         try:
-            ann_dict["sex"] = [l for l in header_reader.comments if "Sex" in l][
-                0
-            ].split(": ")[-1]
-        except:
+            ann_dict["sex"] = [
+                line for line in header_reader.comments if "Sex" in line
+            ][0].split(": ")[-1]
+        except Exception:
             ann_dict["sex"] = "Unknown"
         try:
             ann_dict["medical_prescription"] = [
-                l for l in header_reader.comments if "Rx" in l
+                line for line in header_reader.comments if "Rx" in line
             ][0].split(": ")[-1]
-        except:
+        except Exception:
             ann_dict["medical_prescription"] = "Unknown"
         try:
-            ann_dict["history"] = [l for l in header_reader.comments if "Hx" in l][
-                0
-            ].split(": ")[-1]
-        except:
+            ann_dict["history"] = [
+                line for line in header_reader.comments if "Hx" in line
+            ][0].split(": ")[-1]
+        except Exception:
             ann_dict["history"] = "Unknown"
         try:
             ann_dict["symptom_or_surgery"] = [
-                l for l in header_reader.comments if "Sx" in l
+                line for line in header_reader.comments if "Sx" in line
             ][0].split(": ")[-1]
-        except:
+        except Exception:
             ann_dict["symptom_or_surgery"] = "Unknown"
 
         l_Dx = (
-            [l for l in header_reader.comments if "Dx" in l][0]
+            [line for line in header_reader.comments if "Dx" in line][0]
             .split(": ")[-1]
             .split(",")
         )
@@ -684,7 +713,7 @@ class CINC2020(PhysioNetDataBase):
         return ann_dict
 
     def _load_ann_naive(self, header_data: List[str]) -> dict:
-        """finished, checked,
+        """
 
         load annotations (header) using raw data read directly from a header file
 
@@ -697,6 +726,7 @@ class CINC2020(PhysioNetDataBase):
         -------
         ann_dict, dict,
             the annotations with items: ref. `self.ann_items`
+
         """
         ann_dict = {}
         (
@@ -716,37 +746,39 @@ class CINC2020(PhysioNetDataBase):
         )
         try:  # see NOTE. 1.
             ann_dict["age"] = int(
-                [l for l in header_data if l.startswith("#Age")][0].split(": ")[-1]
+                [line for line in header_data if line.startswith("#Age")][0].split(
+                    ": "
+                )[-1]
             )
-        except:
+        except Exception:
             ann_dict["age"] = np.nan
         try:
-            ann_dict["sex"] = [l for l in header_data if l.startswith("#Sex")][0].split(
-                ": "
-            )[-1]
-        except:
+            ann_dict["sex"] = [line for line in header_data if line.startswith("#Sex")][
+                0
+            ].split(": ")[-1]
+        except Exception:
             ann_dict["sex"] = "Unknown"
         try:
             ann_dict["medical_prescription"] = [
-                l for l in header_data if l.startswith("#Rx")
+                line for line in header_data if line.startswith("#Rx")
             ][0].split(": ")[-1]
-        except:
+        except Exception:
             ann_dict["medical_prescription"] = "Unknown"
         try:
-            ann_dict["history"] = [l for l in header_data if l.startswith("#Hx")][
-                0
-            ].split(": ")[-1]
-        except:
+            ann_dict["history"] = [
+                line for line in header_data if line.startswith("#Hx")
+            ][0].split(": ")[-1]
+        except Exception:
             ann_dict["history"] = "Unknown"
         try:
             ann_dict["symptom_or_surgery"] = [
-                l for l in header_data if l.startswith("#Sx")
+                line for line in header_data if line.startswith("#Sx")
             ][0].split(": ")[-1]
-        except:
+        except Exception:
             ann_dict["symptom_or_surgery"] = "Unknown"
 
         l_Dx = (
-            [l for l in header_data if l.startswith("#Dx")][0]
+            [line for line in header_data if line.startswith("#Dx")][0]
             .split(": ")[-1]
             .split(",")
         )
@@ -759,7 +791,7 @@ class CINC2020(PhysioNetDataBase):
         return ann_dict
 
     def _parse_diagnosis(self, l_Dx: List[str]) -> Tuple[dict, dict]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -772,6 +804,7 @@ class CINC2020(PhysioNetDataBase):
             diagnosis, including SNOMED CT Codes, fullnames and abbreviations of each diagnosis
         diag_scored_dict: dict,
             the scored items in `diag_dict`
+
         """
         diag_dict, diag_scored_dict = {}, {}
         try:
@@ -807,18 +840,14 @@ class CINC2020(PhysioNetDataBase):
                 for idx, item in enumerate(diag_dict["diagnosis_fullname"])
                 if scored_indices[idx]
             ]
-        except:  # the old version, the Dx"s are abbreviations
+        except Exception:  # the old version, the Dx"s are abbreviations
             diag_dict["diagnosis_abbr"] = diag_dict["diagnosis_code"]
             selection = dx_mapping_all["Abbreviation"].isin(diag_dict["diagnosis_abbr"])
             diag_dict["diagnosis_fullname"] = dx_mapping_all[selection]["Dx"].tolist()
-        # if not keep_original:
-        #     for idx, d in enumerate(ann_dict["diagnosis_abbr"]):
-        #         if d in ["Normal", "NSR"]:
-        #             ann_dict["diagnosis_abbr"] = ["N"]
         return diag_dict, diag_scored_dict
 
     def _parse_leads(self, l_leads_data: List[str]) -> pd.DataFrame:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -829,6 +858,7 @@ class CINC2020(PhysioNetDataBase):
         -------
         df_leads: DataFrame,
             infomation of each leads in the format of DataFrame
+
         """
         df_leads = pd.read_csv(
             io.StringIO("\n".join(l_leads_data)), delim_whitespace=True, header=None
@@ -884,23 +914,28 @@ class CINC2020(PhysioNetDataBase):
         df_leads.index.name = None
         return df_leads
 
-    def load_header(self, rec: str, raw: bool = False) -> Union[dict, str]:
+    @add_docstring(load_ann.__doc__)
+    def load_header(self, rec: Union[str, int], raw: bool = False) -> Union[dict, str]:
         """
         alias for `load_ann`, as annotations are also stored in header files
         """
         return self.load_ann(rec, raw)
 
     def get_labels(
-        self, rec: str, scored_only: bool = True, fmt: str = "s", normalize: bool = True
+        self,
+        rec: Union[str, int],
+        scored_only: bool = True,
+        fmt: str = "s",
+        normalize: bool = True,
     ) -> List[str]:
-        """finished, checked,
+        """
 
         read labels (diagnoses or arrhythmias) of a record
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         scored_only: bool, default True,
             only get the labels that are scored in the CINC2020 official phase
         fmt: str, default "a",
@@ -916,6 +951,7 @@ class CINC2020(PhysioNetDataBase):
         -------
         labels, list,
             the list of labels
+
         """
         ann_dict = self.load_ann(rec)
         if scored_only:
@@ -934,15 +970,15 @@ class CINC2020(PhysioNetDataBase):
             labels = [self.label_trans_dict.get(item, item) for item in labels]
         return labels
 
-    def get_fs(self, rec: str) -> Real:
-        """finished, checked,
+    def get_fs(self, rec: Union[str, int]) -> Real:
+        """
 
         get the sampling frequency of a record
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
 
         Returns
         -------
@@ -953,15 +989,17 @@ class CINC2020(PhysioNetDataBase):
         fs = self.fs[tranche]
         return fs
 
-    def get_subject_info(self, rec: str, items: Optional[List[str]] = None) -> dict:
-        """finished, checked,
+    def get_subject_info(
+        self, rec: Union[str, int], items: Optional[List[str]] = None
+    ) -> dict:
+        """
 
         read auxiliary information of a subject (a record) stored in the header files
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         items: list of str, optional,
             items of the subject's information (e.g. sex, age, etc.)
 
@@ -988,7 +1026,7 @@ class CINC2020(PhysioNetDataBase):
 
     def save_challenge_predictions(
         self,
-        rec: str,
+        rec: Union[str, int],
         output_dir: Union[str, Path],
         scores: List[Real],
         labels: List[int],
@@ -1010,7 +1048,10 @@ class CINC2020(PhysioNetDataBase):
             0 or 1, binary predictions
         classes: list of str,
             SNOMED CT Code of binary predictions
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         new_file = f"{rec}.csv"
         output_file = Path(output_dir) / new_file
 
@@ -1026,7 +1067,7 @@ class CINC2020(PhysioNetDataBase):
 
     def plot(
         self,
-        rec: str,
+        rec: Union[str, int],
         data: Optional[np.ndarray] = None,
         ann: Optional[Dict[str, np.ndarray]] = None,
         ticks_granularity: int = 0,
@@ -1035,7 +1076,7 @@ class CINC2020(PhysioNetDataBase):
         waves: Optional[Dict[str, Sequence[int]]] = None,
         **kwargs: Any,
     ) -> NoReturn:
-        """finished, checked, to improve,
+        """to improve,
 
         plot the signals of a record or external signals (units in μV),
         with metadata (fs, labels, tranche, etc.),
@@ -1043,10 +1084,10 @@ class CINC2020(PhysioNetDataBase):
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         data: ndarray, optional,
-            (12-lead) ecg signal to plot,
+            (12-lead) ECG signal to plot,
             should be of the format "channel_first", and compatible with `leads`
             if given, data of `rec` will not be used,
             this is useful when plotting filtered data
@@ -1078,7 +1119,10 @@ class CINC2020(PhysioNetDataBase):
         if not modifying this number, at most 40 seconds of signal could be plotted once
 
         Contributors: Jeethan, and WEN Hao
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         tranche = self._get_tranche(rec)
         if tranche in "CDE":
             physionet_lightwave_suffix = CFG(
@@ -1101,12 +1145,12 @@ class CINC2020(PhysioNetDataBase):
             _leads = [leads]
         else:
             _leads = leads
-        # assert all([l in self.all_leads for l in _leads])
+        # assert all([ld in self.all_leads for ld in _leads])
         assert set(_leads).issubset(self._all_leads_set)
 
         # lead_list = self.load_ann(rec)["df_leads"]["lead_name"].tolist()
-        # lead_indices = [lead_list.index(l) for l in _leads]
-        lead_indices = [self.all_leads.index(l) for l in _leads]
+        # lead_indices = [lead_list.index(ld) for ld in _leads]
+        lead_indices = [self.all_leads.index(ld) for ld in _leads]
         if data is None:
             _data = self.load_data(rec, data_format="channel_first", units="μV")[
                 lead_indices
@@ -1273,7 +1317,7 @@ class CINC2020(PhysioNetDataBase):
     def get_tranche_class_distribution(
         self, tranches: Sequence[str], scored_only: bool = True
     ) -> Dict[str, int]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -1285,7 +1329,9 @@ class CINC2020(PhysioNetDataBase):
         Returns
         -------
         distribution: dict,
-            keys are abbrevations of the classes, values are appearance of corr. classes in the tranche.
+            keys are abbrevations of the classes,
+            values are appearance of corr. classes in the tranche.
+
         """
         tranche_names = [self.tranche_names[t] for t in tranches]
         df = dx_mapping_scored if scored_only else dx_mapping_all
@@ -1300,7 +1346,7 @@ class CINC2020(PhysioNetDataBase):
     def get_arrhythmia_knowledge(
         arrhythmias: Union[str, List[str]], **kwargs
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         knowledge about ECG features of specific arrhythmias,
 
@@ -1308,6 +1354,7 @@ class CINC2020(PhysioNetDataBase):
         ----------
         arrhythmias: str, or list of str,
             the arrhythmia(s) to check, in abbreviations or in SNOMED CT Code
+
         """
         if isinstance(arrhythmias, str):
             d = [normalize_class(arrhythmias)]
@@ -1328,19 +1375,22 @@ class CINC2020(PhysioNetDataBase):
                 print("*" * 110)
 
     def load_resampled_data(
-        self, rec: str, data_format: str = "channel_first", siglen: Optional[int] = None
+        self,
+        rec: Union[str, int],
+        data_format: str = "channel_first",
+        siglen: Optional[int] = None,
     ) -> np.ndarray:
-        """finished, checked,
+        """
 
         resample the data of `rec` to 500Hz,
         or load the resampled data in 500Hz, if the corr. data file already exists
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         data_format: str, default "channel_first",
-            format of the ecg data,
+            format of the ECG data,
             "channel_last" (alias "lead_last"), or
             "channel_first" (alias "lead_first")
         siglen: int, optional,
@@ -1352,7 +1402,10 @@ class CINC2020(PhysioNetDataBase):
         -------
         data: ndarray,
             the resampled (and perhaps sliced) signal data
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         tranche = self._get_tranche(rec)
         if siglen is None:
             rec_fp = self.db_dirs[tranche] / f"{rec}_500Hz.npy"
@@ -1378,16 +1431,16 @@ class CINC2020(PhysioNetDataBase):
             data = data.T
         return data
 
-    def load_raw_data(self, rec: str, backend: str = "scipy") -> np.ndarray:
-        """finished, checked,
+    def load_raw_data(self, rec: Union[str, int], backend: str = "scipy") -> np.ndarray:
+        """
 
         load raw data from corresponding files with no further processing,
         in order to facilitate feeding data into the `run_12ECG_classifier` function
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         backend: str, default "scipy",
             the backend data reader, can also be "wfdb",
             note that "scipy" provides data in the format of "lead_first",
@@ -1397,8 +1450,11 @@ class CINC2020(PhysioNetDataBase):
         -------
         raw_data: ndarray,
             raw data (d_signal) loaded from corresponding data file,
-            without subtracting baseline nor dividing adc gain
+            without subtracting baseline nor dividing ADC gain
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         tranche = self._get_tranche(rec)
         if backend.lower() == "wfdb":
             rec_fp = self.get_data_filepath(rec, with_ext=False)
@@ -1410,7 +1466,7 @@ class CINC2020(PhysioNetDataBase):
         return raw_data
 
     def _check_nan(self, tranches: Union[str, Sequence[str]]) -> NoReturn:
-        """finished, checked,
+        """
 
         check if records from `tranches` has nan values
 
@@ -1422,6 +1478,7 @@ class CINC2020(PhysioNetDataBase):
         ----------
         tranches: str or sequence of str,
             tranches to check
+
         """
         for t in tranches:
             for rec in self.all_records[t]:
@@ -1431,7 +1488,7 @@ class CINC2020(PhysioNetDataBase):
 
     @property
     def url(self) -> List[str]:
-        domain = "https://storage.cloud.google.com/physionet-challenge-2020-12-lead-ecg-public/"
+        domain = "https://storage.cloud.google.com/physionet-challenge-2020-12-lead-ECG-public/"
         return [posixpath.join(domain, f) for f in self.data_files]
 
     data_files = [
@@ -1448,14 +1505,25 @@ class CINC2020(PhysioNetDataBase):
         for url in self.url:
             http_get(url, self.db_dir_base / _stem(url), extract=True)
 
+    def __len__(self) -> int:
+        """
+        number of records in the database
 
-from ..aux_data.cinc2020_aux_data import load_weights
+        """
+        return len(self.__all_records)
+
+    def __getitem__(self, index: int) -> str:
+        """
+        get the record name by index
+
+        """
+        return self.__all_records[index]
 
 
 def compute_all_metrics(
     classes: List[str], truth: Sequence, binary_pred: Sequence, scalar_pred: Sequence
 ) -> Tuple[float]:
-    """finished, checked,
+    """
 
     Parameters
     ----------
@@ -1471,12 +1539,20 @@ def compute_all_metrics(
     Returns
     -------
     auroc: float,
+        area under the receiver operating characteristic (ROC) curve
     auprc: float,
+        area under the precision-recall curve
     accuracy: float,
+        accuracy
     f_measure: float,
+        f1 score
     f_beta_measure: float,
+        f-beta score
     g_beta_measure: float,
+        g-beta score
     challenge_metric: float,
+        challenge metric, defined by a weight matrix
+
     """
     # normal_class = "426783006"
     normal_class = "NSR"
@@ -1520,7 +1596,7 @@ def compute_all_metrics(
 
 # Compute recording-wise accuracy.
 def compute_accuracy(labels: np.ndarray, outputs: np.ndarray) -> float:
-    """checked,"""
+    """ """
     num_recordings, num_classes = np.shape(labels)
 
     num_correct_recordings = 0
@@ -1535,7 +1611,7 @@ def compute_accuracy(labels: np.ndarray, outputs: np.ndarray) -> float:
 def compute_confusion_matrices(
     labels: np.ndarray, outputs: np.ndarray, normalize: bool = False
 ) -> np.ndarray:
-    """checked,"""
+    """ """
     # Compute a binary confusion matrix for each class k:
     #
     #     [TN_k FN_k]
@@ -1580,7 +1656,7 @@ def compute_confusion_matrices(
 
 # Compute macro F-measure.
 def compute_f_measure(labels: np.ndarray, outputs: np.ndarray) -> float:
-    """checked,"""
+    """ """
     num_recordings, num_classes = np.shape(labels)
 
     A = compute_confusion_matrices(labels, outputs)
@@ -1602,7 +1678,7 @@ def compute_f_measure(labels: np.ndarray, outputs: np.ndarray) -> float:
 def compute_beta_measures(
     labels: np.ndarray, outputs: np.ndarray, beta: Real
 ) -> Tuple[float, float]:
-    """checked,"""
+    """ """
     num_recordings, num_classes = np.shape(labels)
 
     A = compute_confusion_matrices(labels, outputs, normalize=True)
@@ -1611,9 +1687,9 @@ def compute_beta_measures(
     g_beta_measure = np.zeros(num_classes)
     for k in range(num_classes):
         tp, fp, fn, tn = A[k, 1, 1], A[k, 1, 0], A[k, 0, 1], A[k, 0, 0]
-        if (1 + beta ** 2) * tp + fp + beta ** 2 * fn:
-            f_beta_measure[k] = float((1 + beta ** 2) * tp) / float(
-                (1 + beta ** 2) * tp + fp + beta ** 2 * fn
+        if (1 + beta**2) * tp + fp + beta**2 * fn:
+            f_beta_measure[k] = float((1 + beta**2) * tp) / float(
+                (1 + beta**2) * tp + fp + beta**2 * fn
             )
         else:
             f_beta_measure[k] = float("nan")
@@ -1630,7 +1706,7 @@ def compute_beta_measures(
 
 # Compute macro AUROC and macro AUPRC.
 def compute_auc(labels: np.ndarray, outputs: np.ndarray) -> Tuple[float, float]:
-    """checked,"""
+    """ """
     num_recordings, num_classes = np.shape(labels)
 
     # Compute and summarize the confusion matrices for each class across at distinct output values.
@@ -1711,10 +1787,10 @@ def compute_auc(labels: np.ndarray, outputs: np.ndarray) -> Tuple[float, float]:
 def compute_modified_confusion_matrix(
     labels: np.ndarray, outputs: np.ndarray
 ) -> np.ndarray:
-    """checked,
-
+    """
     Compute a binary multi-class, multi-label confusion matrix,
     where the rows are the labels and the columns are the outputs.
+
     """
     num_recordings, num_classes = np.shape(labels)
     A = np.zeros((num_classes, num_classes))
@@ -1743,7 +1819,7 @@ def compute_challenge_metric(
     classes: List[str],
     normal_class: str,
 ) -> float:
-    """checked,"""
+    """ """
     num_recordings, num_classes = np.shape(labels)
     normal_index = classes.index(normal_class)
 
