@@ -2,20 +2,22 @@
 """
 """
 
-from pathlib import Path
-from typing import Union, Optional, Any, List, Tuple, Dict, Sequence, NoReturn
 from numbers import Real
+from pathlib import Path
+from typing import Any, Dict, List, NoReturn, Optional, Sequence, Union
 
 import numpy as np
 import pandas as pd
 import wfdb
+from scipy.signal import resample_poly
 
-from ...cfg import CFG
-from ..base import PhysioNetDataBase, ECGWaveForm
-
+from ...cfg import CFG, DEFAULTS
+from ...utils.utils_data import ECGWaveForm, masks_to_waveforms
+from ..base import PhysioNetDataBase
 
 __all__ = [
     "LUDB",
+    "compute_metrics",
 ]
 
 
@@ -143,8 +145,9 @@ class LUDB(PhysioNetDataBase):
 
     References
     ----------
-    [1] https://physionet.org/content/ludb/1.0.1/
-    [2] Kalyakulina, A., Yusipov, I., Moskalenko, V., Nikolskiy, A., Kozlov, A., Kosonogov, K., Zolotykh, N., & Ivanchenko, M. (2020). Lobachevsky University Electrocardiography Database (version 1.0.0).
+    1. <a name="ref1"></a> https://physionet.org/content/ludb/1.0.1/
+    2. <a name="ref2"></a> Kalyakulina, A., Yusipov, I., Moskalenko, V., Nikolskiy, A., Kozlov, A., Kosonogov, K., Zolotykh, N., & Ivanchenko, M. (2020). Lobachevsky University Electrocardiography Database (version 1.0.1).
+
     """
 
     def __init__(
@@ -154,7 +157,7 @@ class LUDB(PhysioNetDataBase):
         verbose: int = 2,
         **kwargs: Any,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -165,6 +168,7 @@ class LUDB(PhysioNetDataBase):
         verbose: int, default 2,
             log verbosity
         kwargs: auxilliary key word arguments
+
         """
         super().__init__(
             db_name="ludb",
@@ -193,7 +197,7 @@ class LUDB(PhysioNetDataBase):
             "V5",
             "V6",
         ]
-        self.all_leads_lower = [l.lower() for l in self.all_leads]
+        self.all_leads_lower = [ld.lower() for ld in self.all_leads]
         # Version 1.0.0 has different beat_ann_ext: [f"atr_{item}" for item in self.all_leads_lower]
         self.beat_ann_ext = [f"{item}" for item in self.all_leads_lower]
 
@@ -247,45 +251,48 @@ class LUDB(PhysioNetDataBase):
         """ """
         raise NotImplementedError
 
-    def _get_path(self, rec: str) -> str:
-        """finished, checked,
+    def _get_path(self, rec: Union[str, int]) -> str:
+        """
 
         get the path of a record, without file extension
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
 
         Returns
         -------
         rec_fp: str,
             path of the record, without file extension
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         rec_fp = str(self.db_dir / "data" / rec)
         return rec_fp
 
     def load_data(
         self,
-        rec: str,
+        rec: Union[str, int],
         leads: Optional[Union[str, List[str]]] = None,
         data_format="channel_first",
         units: str = "mV",
         fs: Optional[Real] = None,
     ) -> np.ndarray:
-        """finished, checked,
+        """
 
-        load physical (converted from digital) ecg data,
+        load physical (converted from digital) ECG data,
         which is more understandable for humans
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         leads: str or list of str, optional,
             the leads to load
         data_format: str, default "channel_first",
-            format of the ecg data,
+            format of the ECG data,
             "channel_last" (alias "lead_last"), or
             "channel_first" (alias "lead_first", original)
         units: str, default "mV",
@@ -296,8 +303,11 @@ class LUDB(PhysioNetDataBase):
         Returns
         -------
         data: ndarray,
-            the ecg data
+            the ECG data
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         assert data_format.lower() in [
             "channel_first",
             "lead_first",
@@ -325,16 +335,19 @@ class LUDB(PhysioNetDataBase):
         return data
 
     def load_ann(
-        self, rec: str, leads: Optional[Sequence[str]] = None, metadata: bool = False
+        self,
+        rec: Union[str, int],
+        leads: Optional[Sequence[str]] = None,
+        metadata: bool = False,
     ) -> dict:
-        """finished, checked,
+        """
 
         load the wave delineation, along with metadata if specified
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         leads: str or list of str, optional,
             the leads to load
         metadata: bool, default False,
@@ -344,17 +357,19 @@ class LUDB(PhysioNetDataBase):
         -------
         ann_dict: dict,
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         ann_dict = CFG()
         rec_fp = self._get_path(rec)
 
         # wave delineation annotations
         _leads = self._normalize_leads(leads, standard_ordering=True, lower_cases=False)
         _ann_ext = [
-            f"{l.lower()}" for l in _leads
+            f"{ld.lower()}" for ld in _leads
         ]  # for Version 1.0.0, it is f"{l.lower()}"
-        ann_dict["waves"] = CFG({l: [] for l in _leads})
-        for l, e in zip(_leads, _ann_ext):
-            ann = wfdb.rdann(rec_fp, extension=e)
+        ann_dict["waves"] = CFG({ld: [] for ld in _leads})
+        for ld, ext in zip(_leads, _ann_ext):
+            ann = wfdb.rdann(rec_fp, extension=ext)
             df_lead_ann = pd.DataFrame()
             symbols = np.array(ann.symbol)
             peak_inds = np.where(np.isin(symbols, ["p", "N", "t"]))[0]
@@ -404,7 +419,7 @@ class LUDB(PhysioNetDataBase):
                     peak=int(row.peak),
                     duration=row.duration,
                 )
-                ann_dict["waves"][l].append(w)
+                ann_dict["waves"][ld].append(w)
 
         if metadata:
             header_dict = self._load_header(rec)
@@ -412,38 +427,40 @@ class LUDB(PhysioNetDataBase):
 
         return ann_dict
 
-    def load_diagnoses(self, rec: str) -> List[str]:
-        """finished, checked,
+    def load_diagnoses(self, rec: Union[str, int]) -> List[str]:
+        """
 
         load diagnoses of the `rec`
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
 
         Returns
         -------
         diagnoses: list of str,
+            diagnoses of the record
+
         """
         diagnoses = self._load_header(rec)["diagnoses"]
         return diagnoses
 
     def load_masks(
         self,
-        rec: str,
+        rec: Union[str, int],
         leads: Optional[Sequence[str]] = None,
         mask_format: str = "channel_first",
         class_map: Optional[Dict[str, int]] = None,
     ) -> np.ndarray:
-        """finished, checked,
+        """
 
         load the wave delineation in the form of masks
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         leads: str or list of str, optional,
             the leads to load
         mask_format: str, default "channel_first",
@@ -458,7 +475,10 @@ class LUDB(PhysioNetDataBase):
         -------
         masks: ndarray,
             the masks corresponding to the wave delineation annotations of `rec`
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         _class_map = CFG(class_map) if class_map is not None else self.class_map
         _leads = self._normalize_leads(leads, standard_ordering=True, lower_cases=True)
         data = self.load_data(rec, leads=_leads, data_format="channel_first")
@@ -484,7 +504,7 @@ class LUDB(PhysioNetDataBase):
         class_map: Optional[Dict[str, int]] = None,
         fs: Optional[Real] = None,
     ) -> Dict[str, List[ECGWaveForm]]:
-        """finished, checked,
+        """
 
         convert masks into lists of waveforms
 
@@ -504,12 +524,13 @@ class LUDB(PhysioNetDataBase):
             if not set, `self.class_map` will be used
         fs: real number, optional,
             sampling frequency of the signal corresponding to the `masks`,
-            if is None, `self.fs` will be used, to compute `duration` of the ecg waveforms
+            if is None, `self.fs` will be used, to compute `duration` of the ECG waveforms
 
         Returns
         -------
         waves: dict,
             each item value is a list containing the `ECGWaveForm`s corr. to the lead (item key)
+
         """
         if masks.ndim == 1:
             _masks = masks[np.newaxis, ...]
@@ -570,20 +591,24 @@ class LUDB(PhysioNetDataBase):
             waves[lead_name].sort(key=lambda w: w.onset)
         return waves
 
-    def _load_header(self, rec: str) -> dict:
-        """finished, checked,
+    def _load_header(self, rec: Union[str, int]) -> dict:
+        """
 
         load header data into a dict
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
 
         Returns
         -------
         header_dict: dict,
+            the header data
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         header_dict = CFG({})
         rec_fp = self._get_path(rec)
         header_reader = wfdb.rdheader(rec_fp)
@@ -593,18 +618,22 @@ class LUDB(PhysioNetDataBase):
         header_dict["record_fmt"] = header_reader.fmt
         try:
             header_dict["age"] = int(
-                [l for l in header_reader.comments if "<age>" in l][0].split(": ")[-1]
+                [line for line in header_reader.comments if "<age>" in line][0].split(
+                    ": "
+                )[-1]
             )
-        except:
+        except Exception:
             header_dict["age"] = np.nan
         try:
-            header_dict["sex"] = [l for l in header_reader.comments if "<sex>" in l][
-                0
-            ].split(": ")[-1]
-        except:
+            header_dict["sex"] = [
+                line for line in header_reader.comments if "<sex>" in line
+            ][0].split(": ")[-1]
+        except Exception:
             header_dict["sex"] = ""
         d_start = [
-            idx for idx, l in enumerate(header_reader.comments) if "<diagnoses>" in l
+            idx
+            for idx, line in enumerate(header_reader.comments)
+            if "<diagnoses>" in line
         ][0] + 1
         header_dict["diagnoses"] = header_reader.comments[d_start:]
         return header_dict
@@ -615,7 +644,7 @@ class LUDB(PhysioNetDataBase):
         standard_ordering: bool = True,
         lower_cases: bool = False,
     ) -> List[str]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -625,34 +654,39 @@ class LUDB(PhysioNetDataBase):
             if True, the ordering will be re-aranged to be accordance with `self.all_leads`
         lower_cases: bool, default False,
             if True, all names of the leads will be in lower cases
+
+        Returns
+        -------
+        leads: list of str,
+            the leads in the standard names and order
         """
         if leads is None:
             _leads = self.all_leads_lower
         elif isinstance(leads, str):
             _leads = [leads.lower()]
         else:
-            _leads = [l.lower() for l in leads]
+            _leads = [ld.lower() for ld in leads]
 
         if standard_ordering:
-            _leads = [l for l in self.all_leads_lower if l in _leads]
+            _leads = [ld for ld in self.all_leads_lower if ld in _leads]
 
         if not lower_cases:
             _lead_indices = [
-                idx for idx, l in enumerate(self.all_leads_lower) if l in _leads
+                idx for idx, ld in enumerate(self.all_leads_lower) if ld in _leads
             ]
             _leads = [self.all_leads[idx] for idx in _lead_indices]
 
         return _leads
 
     def load_subject_info(
-        self, rec: str, fields: Optional[Union[str, Sequence[str]]] = None
+        self, rec: Union[str, int], fields: Optional[Union[str, Sequence[str]]] = None
     ) -> Union[dict, str]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         fields: str or sequence of str, optional,
             field(s) of the subject info of record `rec`,
             if not specified, all fields of the subject info will be returned
@@ -661,7 +695,10 @@ class LUDB(PhysioNetDataBase):
         -------
         info: dict or str,
             subject info of the given fields of the record
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         row = self._df_subject_info[self._df_subject_info.ID == rec]
         if row.empty:
             return {}
@@ -679,7 +716,7 @@ class LUDB(PhysioNetDataBase):
 
     def plot(
         self,
-        rec: str,
+        rec: Union[str, int],
         data: Optional[np.ndarray] = None,
         ticks_granularity: int = 0,
         leads: Optional[Union[str, List[str]]] = None,
@@ -687,7 +724,7 @@ class LUDB(PhysioNetDataBase):
         waves: Optional[ECGWaveForm] = None,
         **kwargs: Any,
     ) -> NoReturn:
-        """finished, checked, to improve,
+        """to improve,
 
         plot the signals of a record or external signals (units in μV),
         with metadata (fs, labels, tranche, etc.),
@@ -695,10 +732,10 @@ class LUDB(PhysioNetDataBase):
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         data: ndarray, optional,
-            12-lead ecg signal to plot,
+            12-lead ECG signal to plot,
             if given, data of `rec` will not be used,
             this is useful when plotting filtered data
         ticks_granularity: int, default 0,
@@ -723,7 +760,10 @@ class LUDB(PhysioNetDataBase):
         if not modifying this number, at most 40 seconds of signal could be plotted once
 
         Contributors: Jeethan, and WEN Hao
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         if "plt" not in dir():
             import matplotlib.pyplot as plt
 
@@ -731,8 +771,8 @@ class LUDB(PhysioNetDataBase):
         _leads = self._normalize_leads(leads, standard_ordering=True, lower_cases=False)
 
         # lead_list = self.load_ann(rec)["df_leads"]["lead_name"].tolist()
-        # _lead_indices = [lead_list.index(l) for l in leads]
-        _lead_indices = [self.all_leads.index(l) for l in _leads]
+        # _lead_indices = [lead_list.index(ld) for ld in leads]
+        _lead_indices = [self.all_leads.index(ld) for ld in _leads]
         if data is None:
             _data = self.load_data(rec, data_format="channel_first", units="μV")[
                 _lead_indices
@@ -754,18 +794,18 @@ class LUDB(PhysioNetDataBase):
             waves = self.load_ann(rec, leads=_leads)["waves"]
 
         if waves is not None:
-            pwaves = {l: [] for l in _leads}
-            qrs = {l: [] for l in _leads}
-            twaves = {l: [] for l in _leads}
-            for l, l_w in waves.items():
-                for w in l_w:
-                    itv = [w.onset, w.offset]
-                    if w.name == self._symbol_to_wavename["p"]:
-                        pwaves[l].append(itv)
-                    elif w.name == self._symbol_to_wavename["N"]:
-                        qrs[l].append(itv)
-                    elif w.name == self._symbol_to_wavename["t"]:
-                        twaves[l].append(itv)
+            pwaves = {ld: [] for ld in _leads}
+            qrs = {ld: [] for ld in _leads}
+            twaves = {ld: [] for ld in _leads}
+            for l_idx, l_w in waves.items():
+                for wv in l_w:
+                    itv = [wv.onset, wv.offset]
+                    if wv.name == self._symbol_to_wavename["p"]:
+                        pwaves[l_idx].append(itv)
+                    elif wv.name == self._symbol_to_wavename["N"]:
+                        qrs[l_idx].append(itv)
+                    elif wv.name == self._symbol_to_wavename["t"]:
+                        twaves[l_idx].append(itv)
 
         palette = {
             "pwaves": "green",
@@ -835,3 +875,354 @@ class LUDB(PhysioNetDataBase):
             plt.savefig(kwargs["save_path"], dpi=200, bbox_inches="tight")
         else:
             plt.show()
+
+
+__TOLERANCE = 150  # ms
+__WaveNames = ["pwave", "qrs", "twave"]
+
+
+def compute_metrics(
+    truth_masks: Sequence[np.ndarray],
+    pred_masks: Sequence[np.ndarray],
+    class_map: Dict[str, int],
+    fs: Real,
+    mask_format: str = "channel_first",
+) -> Dict[str, Dict[str, float]]:
+    """
+
+    compute metrics
+    (sensitivity, precision, f1_score, mean error and standard deviation of the mean errors)
+    for multiple evaluations
+
+    Parameters
+    ----------
+    truth_masks: sequence of ndarray,
+        a sequence of ground truth masks,
+        each of which can also hold multiple masks from different samples (differ by record or by lead)
+    pred_masks: sequence of ndarray,
+        predictions corresponding to `truth_masks`
+    class_map: dict,
+        class map, mapping names to waves to numbers from 0 to n_classes-1,
+        the keys should contain 'pwave', 'qrs', 'twave'
+    fs: real number,
+        sampling frequency of the signal corresponding to the masks,
+        used to compute the duration of each waveform,
+        hence the error and standard deviations of errors
+    mask_format: str, default "channel_first",
+        format of the mask, one of the following:
+        'channel_last' (alias 'lead_last'), or
+        'channel_first' (alias 'lead_first')
+
+    Returns
+    -------
+    scorings: dict,
+        with scorings of onsets and offsets of pwaves, qrs complexes, twaves,
+        each scoring is a dict consisting of the following metrics:
+        sensitivity, precision, f1_score, mean_error, standard_deviation
+
+    """
+    assert len(truth_masks) == len(pred_masks)
+    truth_waveforms, pred_waveforms = [], []
+    # compute for each element
+    for tm, pm in zip(truth_masks, pred_masks):
+        n_masks = (
+            tm.shape[0]
+            if mask_format.lower() in ["channel_first", "lead_first"]
+            else tm.shape[1]
+        )
+
+        new_t = masks_to_waveforms(tm, class_map, fs, mask_format)
+        new_t = [
+            new_t[f"lead_{idx+1}"] for idx in range(n_masks)
+        ]  # list of list of `ECGWaveForm`s
+        truth_waveforms += new_t
+
+        new_p = masks_to_waveforms(pm, class_map, fs, mask_format)
+        new_p = [
+            new_p[f"lead_{idx+1}"] for idx in range(n_masks)
+        ]  # list of list of `ECGWaveForm`s
+        pred_waveforms += new_p
+
+    scorings = compute_metrics_waveform(truth_waveforms, pred_waveforms, fs)
+
+    return scorings
+
+
+def compute_metrics_waveform(
+    truth_waveforms: Sequence[Sequence[ECGWaveForm]],
+    pred_waveforms: Sequence[Sequence[ECGWaveForm]],
+    fs: Real,
+) -> Dict[str, Dict[str, float]]:
+    """
+
+    compute the sensitivity, precision, f1_score, mean error and standard deviation of the mean errors,
+    of evaluations on a multiple samples (differ by records, or leads)
+
+    Parameters
+    ----------
+    truth_waveforms: sequence of sequence of `ECGWaveForm`s,
+        the ground truth,
+        each element is a sequence of `ECGWaveForm`s from the same sample
+    pred_waveforms: sequence of sequence of `ECGWaveForm`s,
+        the predictions corresponding to `truth_waveforms`,
+        each element is a sequence of `ECGWaveForm`s from the same sample
+    fs: real number,
+        sampling frequency of the signal corresponding to the waveforms,
+        used to compute the duration of each waveform,
+        hence the error and standard deviations of errors
+
+    Returns
+    -------
+    scorings: dict,
+        with scorings of onsets and offsets of pwaves, qrs complexes, twaves,
+        each scoring is a dict consisting of the following metrics:
+        sensitivity, precision, f1_score, mean_error, standard_deviation
+
+    """
+    truth_positive = CFG(
+        {
+            f"{wave}_{term}": 0
+            for wave in [
+                "pwave",
+                "qrs",
+                "twave",
+            ]
+            for term in ["onset", "offset"]
+        }
+    )
+    false_positive = CFG(
+        {
+            f"{wave}_{term}": 0
+            for wave in [
+                "pwave",
+                "qrs",
+                "twave",
+            ]
+            for term in ["onset", "offset"]
+        }
+    )
+    false_negative = CFG(
+        {
+            f"{wave}_{term}": 0
+            for wave in [
+                "pwave",
+                "qrs",
+                "twave",
+            ]
+            for term in ["onset", "offset"]
+        }
+    )
+    errors = CFG(
+        {
+            f"{wave}_{term}": []
+            for wave in [
+                "pwave",
+                "qrs",
+                "twave",
+            ]
+            for term in ["onset", "offset"]
+        }
+    )
+    # accumulating results
+    for tw, pw in zip(truth_waveforms, pred_waveforms):
+        s = _compute_metrics_waveform(tw, pw, fs)
+        for wave in [
+            "pwave",
+            "qrs",
+            "twave",
+        ]:
+            for term in ["onset", "offset"]:
+                truth_positive[f"{wave}_{term}"] += s[f"{wave}_{term}"][
+                    "truth_positive"
+                ]
+                false_positive[f"{wave}_{term}"] += s[f"{wave}_{term}"][
+                    "false_positive"
+                ]
+                false_negative[f"{wave}_{term}"] += s[f"{wave}_{term}"][
+                    "false_negative"
+                ]
+                errors[f"{wave}_{term}"] += s[f"{wave}_{term}"]["errors"]
+    scorings = CFG()
+    for wave in [
+        "pwave",
+        "qrs",
+        "twave",
+    ]:
+        for term in ["onset", "offset"]:
+            tp = truth_positive[f"{wave}_{term}"]
+            fp = false_positive[f"{wave}_{term}"]
+            fn = false_negative[f"{wave}_{term}"]
+            err = errors[f"{wave}_{term}"]
+            sensitivity = tp / (tp + fn + DEFAULTS.eps)
+            precision = tp / (tp + fp + DEFAULTS.eps)
+            f1_score = (
+                2 * sensitivity * precision / (sensitivity + precision + DEFAULTS.eps)
+            )
+            mean_error = np.mean(err) * 1000 / fs
+            standard_deviation = np.std(err) * 1000 / fs
+            scorings[f"{wave}_{term}"] = CFG(
+                sensitivity=sensitivity,
+                precision=precision,
+                f1_score=f1_score,
+                mean_error=mean_error,
+                standard_deviation=standard_deviation,
+            )
+
+    return scorings
+
+
+def _compute_metrics_waveform(
+    truths: Sequence[ECGWaveForm], preds: Sequence[ECGWaveForm], fs: Real
+) -> Dict[str, Dict[str, float]]:
+    """
+
+    compute the sensitivity, precision, f1_score, mean error and standard deviation of the mean errors,
+    of evaluations on a single sample (the same record, the same lead)
+
+    Parameters
+    ----------
+    truths: sequence of `ECGWaveForm`s,
+        the ground truth
+    preds: sequence of `ECGWaveForm`s,
+        the predictions corresponding to `truths`,
+    fs: real number,
+        sampling frequency of the signal corresponding to the waveforms,
+        used to compute the duration of each waveform,
+        hence the error and standard deviations of errors
+
+    Returns
+    -------
+    scorings: dict,
+        with scorings of onsets and offsets of pwaves, qrs complexes, twaves,
+        each scoring is a dict consisting of the following metrics:
+        truth_positive, false_negative, false_positive, errors,
+        sensitivity, precision, f1_score, mean_error, standard_deviation
+
+    """
+    pwave_onset_truths, pwave_offset_truths, pwave_onset_preds, pwave_offset_preds = (
+        [],
+        [],
+        [],
+        [],
+    )
+    qrs_onset_truths, qrs_offset_truths, qrs_onset_preds, qrs_offset_preds = (
+        [],
+        [],
+        [],
+        [],
+    )
+    twave_onset_truths, twave_offset_truths, twave_onset_preds, twave_offset_preds = (
+        [],
+        [],
+        [],
+        [],
+    )
+
+    for item in ["truths", "preds"]:
+        for w in eval(item):
+            for term in ["onset", "offset"]:
+                eval(f"{w.name}_{term}_{item}.append(w.{term})")
+
+    scorings = CFG()
+    for wave in [
+        "pwave",
+        "qrs",
+        "twave",
+    ]:
+        for term in ["onset", "offset"]:
+            (
+                truth_positive,
+                false_negative,
+                false_positive,
+                errors,
+                sensitivity,
+                precision,
+                f1_score,
+                mean_error,
+                standard_deviation,
+            ) = _compute_metrics_base(
+                eval(f"{wave}_{term}_truths"), eval(f"{wave}_{term}_preds"), fs
+            )
+            scorings[f"{wave}_{term}"] = CFG(
+                truth_positive=truth_positive,
+                false_negative=false_negative,
+                false_positive=false_positive,
+                errors=errors,
+                sensitivity=sensitivity,
+                precision=precision,
+                f1_score=f1_score,
+                mean_error=mean_error,
+                standard_deviation=standard_deviation,
+            )
+    return scorings
+
+
+def _compute_metrics_base(
+    truths: Sequence[Real], preds: Sequence[Real], fs: Real
+) -> Dict[str, float]:
+    """
+
+    Parameters
+    ----------
+    truths: sequence of real numbers,
+        ground truth of indices of corresponding critical points
+    preds: sequence of real numbers,
+        predicted indices of corresponding critical points
+    fs: real number,
+        sampling frequency of the signal corresponding to the critical points,
+        used to compute the duration of each waveform,
+        hence the error and standard deviations of errors
+
+    Returns
+    -------
+    tuple of metrics:
+        truth_positive, false_negative, false_positive, errors,
+        sensitivity, precision, f1_score, mean_error, standard_deviation
+        see ref. [1]
+
+    """
+    _tolerance = __TOLERANCE * fs / 1000
+    _truths = np.array(truths)
+    _preds = np.array(preds)
+    truth_positive, false_positive, false_negative = 0, 0, 0
+    errors = []
+    n_included = 0
+    for point in truths:
+        _pred = _preds[np.where(np.abs(_preds - point) <= _tolerance)[0].tolist()]
+        if len(_pred) > 0:
+            truth_positive += 1
+            idx = np.argmin(np.abs(_pred - point))
+            errors.append(_pred[idx] - point)
+        else:
+            false_negative += 1
+        n_included += len(_pred)
+
+    # false_positive = len(_preds) - n_included
+    false_positive = len(_preds) - truth_positive
+
+    # print(f"""
+    # truth_positive = {truth_positive}
+    # false_positive = {false_positive}
+    # false_negative = {false_negative}
+    # """)
+
+    # print(f"len(truths) = {len(truths)}, truth_positive + false_negative = {truth_positive + false_negative}")
+    # print(f"len(preds) = {len(preds)}, truth_positive + false_positive = {truth_positive + false_positive}")
+
+    sensitivity = truth_positive / (truth_positive + false_negative + DEFAULTS.eps)
+    precision = truth_positive / (truth_positive + false_positive + DEFAULTS.eps)
+    f1_score = 2 * sensitivity * precision / (sensitivity + precision + DEFAULTS.eps)
+    mean_error = np.mean(errors) * 1000 / fs
+    standard_deviation = np.std(errors) * 1000 / fs
+
+    return (
+        truth_positive,
+        false_negative,
+        false_positive,
+        errors,
+        sensitivity,
+        precision,
+        f1_score,
+        mean_error,
+        standard_deviation,
+    )
