@@ -2,24 +2,31 @@
 """
 """
 
-from pathlib import Path
-from collections import defaultdict
-from typing import Union, Optional, Any, List, NoReturn, Dict, Sequence
+from collections import defaultdict, Counter
 from numbers import Real
+from pathlib import Path
+from typing import Any, Dict, List, NoReturn, Optional, Sequence, Union
 
-import wfdb
 import numpy as np
 import pandas as pd
+import wfdb
+from scipy.signal import resample_poly
 
+try:
+    from tqdm.auto import tqdm
+except ModuleNotFoundError:
+    from tqdm import tqdm
+
+from ...cfg import CFG
+from ...utils.misc import get_record_list_recursive3
+from ...utils.utils_interval import generalized_intervals_intersection
 from ..base import (
+    BeatAnn,
     PhysioNetDataBase,
     WFDB_Beat_Annotations,
     WFDB_Non_Beat_Annotations,
-    BeatAnn,
+    WFDB_Rhythm_Annotations,
 )
-from ...utils.misc import get_record_list_recursive3
-from ...utils.utils_interval import generalized_intervals_intersection
-
 
 __all__ = [
     "MITDB",
@@ -47,11 +54,14 @@ class MITDB(PhysioNetDataBase):
 
     Usage
     -----
-    1.
+    1. Beat classification
+    2. rhythm classification (segmentation)
+    3. R peaks detection
 
     References
     ----------
-    [1] https://physionet.org/content/mitdb/1.0.0/
+    1. <a name="ref1"></a> https://physionet.org/content/mitdb/1.0.0/
+
     """
 
     def __init__(
@@ -73,6 +83,7 @@ class MITDB(PhysioNetDataBase):
         verbose: int, default 2,
             log verbosity
         kwargs: auxilliary key word arguments
+
         """
         super().__init__(
             db_name="mitdb",
@@ -118,28 +129,58 @@ class MITDB(PhysioNetDataBase):
             "PSE",
             "TS",
         ]
-        self.rhythm_types_map = {
-            item.lstrip("("): i
-            for i, item in enumerate(self.rhythm_types)
-            if item.startswith("(")
-        }
+        self.rhythm_types = [
+            rt.lstrip("(") for rt in self.rhythm_types if rt in WFDB_Rhythm_Annotations
+        ]
+        self.rhythm_types_map = {rt: idx for idx, rt in enumerate(self.rhythm_types)}
         self._rhythm_ignore_index = -100
 
         self.all_leads = ["MLII", "V1", "V2", "V4", "V5"]
 
         self._ls_rec()
 
+        self._stats = pd.DataFrame()
+        self._stats_columns = ["record", "beat_num", "beat_type_num", "rhythm_len"]
+        self._aggregate_stats()
+
     def _ls_rec(self) -> NoReturn:
         """ """
         super()._ls_rec()
         if len(self._all_records) == 0:
             self._all_records = get_record_list_recursive3(
-                self.db_dir, f"^[\d]{{3}}.{self.data_ext}$"
+                self.db_dir, f"^[\\d]{{3}}.{self.data_ext}$"
             )
 
+    def _aggregate_stats(self) -> NoReturn:
+        """ """
+        self._stats = pd.DataFrame(columns=self._stats_columns)
+        with tqdm(range(len(self)), desc="Aggregating stats", unit="records") as pbar:
+            for idx in pbar:
+                rec_ann = self.load_ann(idx)
+                beat_type_num = {
+                    k: v
+                    for k, v in Counter(
+                        [item.symbol for item in rec_ann["beat"]]
+                    ).most_common()
+                }
+                beat_num = sum(beat_type_num.values())
+                rhythm_len = {
+                    k: sum([itv[1] - itv[0] for itv in v])
+                    for k, v in rec_ann["rhythm"].items()
+                }
+                self._stats = self._stats.append(
+                    dict(
+                        record=self[idx],
+                        beat_num=beat_num,
+                        beat_type_num=beat_type_num,
+                        rhythm_len=rhythm_len,
+                    ),
+                    ignore_index=True,
+                )
+
     def load_data(
         self,
-        rec: str,
+        rec: Union[str, int],
         leads: Optional[Union[str, List[str]]] = None,
         sampfrom: Optional[int] = None,
         sampto: Optional[int] = None,
@@ -147,15 +188,15 @@ class MITDB(PhysioNetDataBase):
         units: str = "mV",
         fs: Optional[Real] = None,
     ) -> np.ndarray:
-        """finished, checked,
+        """
 
-        load physical (converted from digital) ecg data,
+        load physical (converted from digital) ECG data,
         which is more understandable for humans
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         leads: str or list of str, optional,
             the leads to load
         sampfrom: int, optional,
@@ -163,7 +204,7 @@ class MITDB(PhysioNetDataBase):
         sampto: int, optional,
             end index of the data to be loaded
         data_format: str, default "channel_first",
-            format of the ecg data,
+            format of the ECG data,
             "channel_last" (alias "lead_last"), or
             "channel_first" (alias "lead_first")
         units: str, default "mV",
@@ -174,8 +215,11 @@ class MITDB(PhysioNetDataBase):
         Returns
         -------
         data: ndarray,
-            the ecg data
+            the ECG data
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         fp = str(self.db_dir / rec)
         if not leads:
             _leads = self._get_lead_names(rec)
@@ -202,15 +246,17 @@ class MITDB(PhysioNetDataBase):
 
     def load_ann(
         self,
-        rec: str,
+        rec: Union[str, int],
         sampfrom: Optional[int] = None,
         sampto: Optional[int] = None,
-        rhythm_format: str = "interval",
+        rhythm_format: str = "intervals",
+        rhythm_types: Optional[Sequence[str]] = None,
         beat_format: str = "beat",
+        beat_types: Optional[Sequence[str]] = None,
         extended_beats: bool = False,
         keep_original: bool = False,
     ) -> dict:
-        """finished, checked,
+        """
 
         load rhythm and beat annotations,
         which are stored in the `aux_note`, `symbol` attributes of corresponding annotation files.
@@ -218,16 +264,22 @@ class MITDB(PhysioNetDataBase):
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         sampfrom: int, optional,
             start index of the annotations to be loaded
         sampto: int, optional,
             end index of the annotations to be loaded
-        rhythm_format: str, default "interval", case insensitive,
+        rhythm_format: str, default "intervals", case insensitive,
             format of returned annotation, can also be "mask"
+        rhythm_types: list of str, optional,
+            defaults to `self.rhythm_types`
+            if not None, only the rhythm annotations with the specified types will be returned
         beat_format: str, default "beat", case insensitive,
             format of returned annotation, can also be "dict"
+        beat_types: list of str, optional,
+            defaults to `self.beat_types`
+            if not None, only the beat annotations with the specified types will be returned
         keep_original: bool, default False,
             if True, indices will keep the same with the annotation file
             otherwise subtract `sampfrom` if specified
@@ -238,11 +290,14 @@ class MITDB(PhysioNetDataBase):
             the annotations of `rhythm` and `beat`, with
             `rhythm` annotatoins in the format of `intervals`, or `mask`;
             `beat` annotations in the format of `dict` or `BeatAnn`
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         assert rhythm_format.lower() in [
-            "interval",
+            "intervals",
             "mask",
-        ], f"`rhythm_format` must be one of ['interval', 'mask'], got {rhythm_format}"
+        ], f"`rhythm_format` must be one of ['intervals', 'mask'], got {rhythm_format}"
         assert beat_format.lower() in [
             "beat",
             "dict",
@@ -258,23 +313,33 @@ class MITDB(PhysioNetDataBase):
         sample_inds = wfdb_ann.sample
         indices = np.where((sample_inds >= sf) & (sample_inds < st))[0]
 
+        if beat_types is None:
+            beat_types = self.beat_types
+
         beat_ann = [
             BeatAnn(i, s)
             for i, s in zip(sample_inds[indices], np.array(wfdb_ann.symbol)[indices])
+            if s in beat_types
         ]
 
+        if rhythm_types is None:
+            rhythm_types = self.rhythm_types
+            rhythm_types_map = self.rhythm_types_map
+        else:
+            rhythm_types = [rt.lstrip("(") for rt in rhythm_types]
+            rhythm_types_map = {rt: idx for idx, rt in enumerate(rhythm_types)}
+
         rhythm_intervals = defaultdict(list)
         start_idx, rhythm = None, None
         for ra, si in zip(wfdb_ann.aux_note, sample_inds):
-            ra = ra.rstrip("\x00")
-            if ra.startswith("("):
+            ra = ra.rstrip("\x00").lstrip("(")
+            if ra in rhythm_types:
                 if start_idx is not None:
                     rhythm_intervals[rhythm].append([start_idx, si])
                 start_idx = si
                 rhythm = ra.lstrip("(")
         if start_idx is not None:
             rhythm_intervals[rhythm].append([start_idx, si])
-            # rhythm_intervals[rhythm].append([start_idx, sig_len])
         rhythm_intervals = {
             k: np.array(generalized_intervals_intersection(v, [[sf, st]]))
             for k, v in rhythm_intervals.items()
@@ -302,7 +367,7 @@ class MITDB(PhysioNetDataBase):
 
         ann = {}
         ann["beat"] = beat_ann
-        if rhythm_format.lower() == "interval":
+        if rhythm_format.lower() == "intervals":
             ann["rhythm"] = rhythm_intervals
         else:
             ann["rhythm"] = rhythm_mask
@@ -310,29 +375,33 @@ class MITDB(PhysioNetDataBase):
 
     def load_rhythm_ann(
         self,
-        rec: str,
+        rec: Union[str, int],
         sampfrom: Optional[int] = None,
         sampto: Optional[int] = None,
-        rhythm_format: str = "interval",
+        rhythm_format: str = "intervals",
+        rhythm_types: Optional[Sequence[str]] = None,
         keep_original: bool = False,
     ) -> Union[Dict[str, list], np.ndarray]:
-        """finished, checked,
+        """
 
         load rhythm annotations,
         which are stored in the `aux_note` attribute of corresponding annotation files.
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         sampfrom: int, optional,
             start index of the annotations to be loaded
         sampto: int, optional,
             end index of the annotations to be loaded
-        rhythm_format: str, default "interval", case insensitive,
+        rhythm_format: str, default "intervals", case insensitive,
             format of returned annotation, can also be "mask"
         beat_format: str, default "beat", case insensitive,
             format of returned annotation, can also be "dict"
+        rhythm_types: list of str, optional,
+            defaults to `self.rhythm_types`
+            if not None, only the rhythm annotations with the specified types will be returned
         keep_original: bool, default False,
             if True, indices will keep the same with the annotation file
             otherwise subtract `sampfrom` if specified
@@ -341,38 +410,46 @@ class MITDB(PhysioNetDataBase):
         -------
         ann, dict or ndarray,
             the annotations in the format of intervals, or in the format of mask
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         return self.load_ann(
             rec,
             sampfrom,
             sampto,
             rhythm_format=rhythm_format,
+            rhythm_types=rhythm_types,
             keep_original=keep_original,
         )["rhythm"]
 
     def load_beat_ann(
         self,
-        rec: str,
+        rec: Union[str, int],
         sampfrom: Optional[int] = None,
         sampto: Optional[int] = None,
         beat_format: str = "beat",
+        beat_types: Optional[Sequence[str]] = None,
         keep_original: bool = False,
     ) -> Union[Dict[str, np.ndarray], List[BeatAnn]]:
-        """finished, checked,
+        """
 
         load beat annotations,
         which are stored in the `symbol` attribute of corresponding annotation files
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         sampfrom: int, optional,
             start index of the annotations to be loaded
         sampto: int, optional,
             end index of the annotations to be loaded
         beat_format: str, default "beat", case insensitive,
             format of returned annotation, can also be "dict"
+        beat_types: list of str, optional,
+            defaults to `self.beat_types`
+            if not None, only the beat annotations with the specified types will be returned
         keep_original: bool, default False,
             if True, indices will keep the same with the annotation file
             otherwise subtract `sampfrom` if specified
@@ -381,23 +458,27 @@ class MITDB(PhysioNetDataBase):
         -------
         ann, dict or list,
             locations (indices) of the all the beat types ("A", "N", "Q", "V",)
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         return self.load_ann(
             rec,
             sampfrom,
             sampto,
             beat_format=beat_format,
+            beat_types=beat_types,
             keep_original=keep_original,
         )["beat"]
 
     def load_rpeak_indices(
         self,
-        rec: str,
+        rec: Union[str, int],
         sampfrom: Optional[int] = None,
         sampto: Optional[int] = None,
         keep_original: bool = False,
     ) -> np.ndarray:
-        """finished, checked,
+        """
 
         load rpeak indices, or equivalently qrs complex locations,
         which are stored in the `symbol` attribute of corresponding annotation files,
@@ -405,8 +486,8 @@ class MITDB(PhysioNetDataBase):
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
         sampfrom: int, optional,
             start index of the annotations to be loaded
         sampto: int, optional,
@@ -419,7 +500,10 @@ class MITDB(PhysioNetDataBase):
         -------
         rpeak_inds, ndarray,
             locations (indices) of the all the rpeaks (qrs complexes)
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         fp = str(self.db_dir / rec)
         wfdb_ann = wfdb.rdann(fp, extension=self.ann_ext)
         header = wfdb.rdheader(fp)
@@ -439,24 +523,113 @@ class MITDB(PhysioNetDataBase):
             rpeak_inds -= sf
         return rpeak_inds
 
-    def _get_lead_names(self, rec: str) -> List[str]:
+    def _get_lead_names(self, rec: Union[str, int]) -> List[str]:
         """
 
         Parameters
         ----------
-        rec: str,
-            name of the record
+        rec: str or int,
+            name or index of the record
 
-        returns:
+        Returns
+        -------
         list of str:
             a list of names of the leads contained in the record
+
         """
+        if isinstance(rec, int):
+            rec = self[rec]
         fp = str(self.db_dir / rec)
         return wfdb.rdheader(fp).sig_name
 
+    @property
+    def df_stats(self) -> pd.DataFrame:
+        """ """
+        if self._stats.empty:
+            self._aggregate_stats()
+        return self._stats
+
+    @property
+    def df_stats_expanded(self) -> pd.DataFrame:
+        """ """
+        df = self.df_stats.copy(deep=True)
+        for bt in self.beat_types:
+            df[f"beat_{bt}"] = df["beat_type_num"].apply(lambda d: d.get(bt, 0))
+        for rt in self.rhythm_types:
+            df[f"rhythm_{rt}"] = df["rhythm_len"].apply(lambda d: d.get(rt, 0))
+        return df.drop(columns=["beat_num", "beat_type_num", "rhythm_len"])
+
+    @property
+    def df_stats_expanded_boolean(self) -> pd.DataFrame:
+        """ """
+        df = self.df_stats_expanded.copy(deep=True)
+        for col in df.columns:
+            if col == "record":
+                continue
+            df[col] = df[col].apply(lambda x: int(x > 0))
+        return df
+
+    @property
+    def db_stats(self) -> Dict[str, Dict[str, int]]:
+        """ """
+        if self._stats.empty:
+            self._aggregate_stats()
+        rhythm_len = defaultdict(int)
+        for rl_dict in self._stats["rhythm_len"]:
+            for k, v in rl_dict.items():
+                rhythm_len[k] += v
+        beat_type_num = defaultdict(int)
+        for btn_dict in self._stats["beat_type_num"]:
+            for k, v in btn_dict.items():
+                beat_type_num[k] += v
+        return CFG(rhythm_len=dict(rhythm_len), beat_type_num=dict(beat_type_num))
+
+    def _categorize_records(self, by: str) -> Dict[str, List[str]]:
+        """
+
+        categorize records by specific attributes
+
+        Parameters
+        ----------
+        by: str,
+            the attribute to categorize the records,
+            can be one of "beat", "rhythm", case insensitive
+
+        Returns
+        -------
+        dict of list of str:
+            a dict of lists of record names,
+
+        """
+        assert by.lower() in [
+            "beat",
+            "rhythm",
+        ], f"`by` should be one of 'beat' or 'rhythm', but got {by}"
+        key = dict(beat="beat_type_num", rhythm="rhythm_len")[by.lower()]
+        return CFG(
+            {
+                item: [
+                    row["record"]
+                    for _, row in self.df_stats.iterrows()
+                    if item in row[key]
+                ]
+                for item in self.db_stats[key]
+            }
+        )
+
+    @property
+    def beat_types_records(self) -> Dict[str, List[str]]:
+        """ """
+        return self._categorize_records("beat")
+
+    @property
+    def rhythm_types_records(self) -> Dict[str, List[str]]:
+        """ """
+        return self._categorize_records("rhythm")
+
     def plot(
         self,
-        rec: str,
+        rec: Union[str, int],
         data: Optional[np.ndarray] = None,
         ann: Optional[Dict[str, np.ndarray]] = None,
         beat_ann: Optional[Dict[str, np.ndarray]] = None,
