@@ -1,14 +1,10 @@
 """
 """
 
-from itertools import repeat
 from copy import deepcopy
 
-import numpy as np
-
 from ...cfg import CFG
 
-
 __all__ = [
     # vanilla densenet
     "densenet_vanilla",
