@@ -1,36 +1,38 @@
 """
 The core part of the SOTA model (framework) of CPSC2020
 
-Its key points: 
+Its key points:
 1. ECG signal pre-processing: filters out the high frequency noise and baseline drift in the ECG signal
 2. QRS complex detection
 3. noisy heartbeat recognition: transient noise and artifacts
 4. atrial fibrillation recognition: remove SPB and carefully distinguish PVC and AF beats with aberrant ventricular conduction in episodes with atrial fibrillation
 5. PVC and SPB model detection: DenseNet
 6. post-processing with clinical rules: a set of clinical experiences and rules including rhythm and morphological rules to suppress false positives and search for false negatives of PVC and SPB detection
+
 """
 
 import math
 from copy import deepcopy
 from itertools import repeat
-from typing import Union, Optional, Sequence, NoReturn
+from typing import NoReturn, Optional, Sequence, Union
 
-import numpy as np
 import torch
-from torch import nn
-from torch import Tensor
+from torch import Tensor, nn
 
 from ...cfg import CFG, DEFAULTS
-from ...utils.utils_nn import compute_module_size, SizeMixin
-from ...utils.misc import dict_to_str, list_sum
-from ...models._nets import (
+from ...models._nets import (  # noqa: F401
     Conv_Bn_Activation,
     DownSample,
+    GlobalContextBlock,
     NonLocalBlock,
     SEBlock,
-    GlobalContextBlock,
 )
-
+from ...utils.misc import dict_to_str, list_sum, add_docstring
+from ...utils.utils_nn import (
+    SizeMixin,
+    compute_sequential_output_shape,
+    compute_sequential_output_shape_docstring,
+)
 
 if DEFAULTS.torch_dtype == torch.float64:
     torch.set_default_tensor_type(torch.DoubleTensor)
@@ -45,12 +47,13 @@ __all__ = [
 ]
 
 
-class DenseBasicBlock(SizeMixin, nn.Module):
-    """finished, checked,
+class DenseBasicBlock(nn.Module, SizeMixin):
+    """
 
     the basic building block for DenseNet,
     consisting of normalization -> activation -> convolution (-> dropout (optional)),
     the output Tensor is the concatenation of old features (input) with new features
+
     """
 
     __DEBUG__ = True
@@ -71,7 +74,7 @@ class DenseBasicBlock(SizeMixin, nn.Module):
         dropout: float = 0.0,
         **config,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -93,6 +96,7 @@ class DenseBasicBlock(SizeMixin, nn.Module):
         config: dict,
             other hyper-parameters, including
             activation choices, memory_efficient choices, etc.
+
         """
         super().__init__()
         self.__in_channels = in_channels
@@ -110,7 +114,7 @@ class DenseBasicBlock(SizeMixin, nn.Module):
             stride=1,
             dilation=1,
             groups=self.__groups,
-            batch_norm=True,
+            norm=True,
             activation=self.config.activation.lower(),
             kw_activation=self.config.kw_activation,
             bias=bias,
@@ -122,7 +126,7 @@ class DenseBasicBlock(SizeMixin, nn.Module):
             self.dropout = None
 
     def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -133,6 +137,7 @@ class DenseBasicBlock(SizeMixin, nn.Module):
         -------
         output: Tensor,
             of shape (batch_size, n_channels, seq_len)
+
         """
         new_features = self.bac(input)
         if self.dropout:
@@ -163,7 +168,7 @@ class DenseBasicBlock(SizeMixin, nn.Module):
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -175,15 +180,16 @@ class DenseBasicBlock(SizeMixin, nn.Module):
         Returns
         -------
         output_shape: sequence,
-            the output shape of this block, given `seq_len` and `batch_size`
+            the output shape, given `seq_len` and `batch_size`
+
         """
         out_channels = self.__in_channels + self.__growth_rate
         output_shape = (batch_size, out_channels, seq_len)
         return output_shape
 
 
-class DenseBottleNeck(SizeMixin, nn.Module):
-    """finished, checked,
+class DenseBottleNeck(nn.Module, SizeMixin):
+    """
 
     bottleneck modification of `DenseBasicBlock`,
     with an additional prefixed sequence of
@@ -209,7 +215,7 @@ class DenseBottleNeck(SizeMixin, nn.Module):
         dropout: float = 0.0,
         **config,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -233,6 +239,7 @@ class DenseBottleNeck(SizeMixin, nn.Module):
         config: dict,
             other hyper-parameters, including
             activation choices, memory_efficient choices, etc.
+
         """
         super().__init__()
         self.__in_channels = in_channels
@@ -251,7 +258,7 @@ class DenseBottleNeck(SizeMixin, nn.Module):
             stride=1,
             dilation=1,
             groups=groups,
-            batch_norm=True,
+            norm=True,
             activation=self.config.activation.lower(),
             kw_activation=self.config.kw_activation,
             bias=bias,
@@ -264,7 +271,7 @@ class DenseBottleNeck(SizeMixin, nn.Module):
             stride=1,
             dilation=1,
             groups=self.__groups,
-            batch_norm=True,
+            norm=True,
             activation=self.config.activation.lower(),
             kw_activation=self.config.kw_activation,
             bias=bias,
@@ -276,7 +283,7 @@ class DenseBottleNeck(SizeMixin, nn.Module):
             self.dropout = None
 
     def bn_function(self, input: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         the `not memory_efficient` way
 
@@ -289,12 +296,13 @@ class DenseBottleNeck(SizeMixin, nn.Module):
         -------
         bottleneck_output: Tensor,
             of shape (batch_size, n_channels, seq_len)
+
         """
         bottleneck_output = self.neck_conv(input)
         return bottleneck_output
 
     def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -305,6 +313,7 @@ class DenseBottleNeck(SizeMixin, nn.Module):
         -------
         output: Tensor,
             of shape (batch_size, n_channels, seq_len)
+
         """
         if self.config.memory_efficient:
             raise NotImplementedError
@@ -339,7 +348,7 @@ class DenseBottleNeck(SizeMixin, nn.Module):
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -351,18 +360,20 @@ class DenseBottleNeck(SizeMixin, nn.Module):
         Returns
         -------
         output_shape: sequence,
-            the output shape of this block, given `seq_len` and `batch_size`
+            the output shape, given `seq_len` and `batch_size`
+
         """
         out_channels = self.__in_channels + self.__growth_rate
         output_shape = (batch_size, out_channels, seq_len)
         return output_shape
 
 
-class DenseMacroBlock(SizeMixin, nn.Sequential):
-    """finished, checked,
+class DenseMacroBlock(nn.Sequential, SizeMixin):
+    """
 
     macro blocks for `DenseNet`,
     stacked sequence of builing blocks of similar pattern
+
     """
 
     __DEBUG__ = True
@@ -381,7 +392,7 @@ class DenseMacroBlock(SizeMixin, nn.Sequential):
         dropout: float = 0.0,
         **config,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -409,6 +420,7 @@ class DenseMacroBlock(SizeMixin, nn.Sequential):
             other hyper-parameters, including
             extra kw for `DenseBottleNeck`, and
             activation choices, memory_efficient choices, etc.
+
         """
         super().__init__()
         self.__in_channels = in_channels
@@ -446,52 +458,21 @@ class DenseMacroBlock(SizeMixin, nn.Sequential):
                 ),
             )
 
-    def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
-
-        Parameters
-        ----------
-        input: Tensor,
-            of shape (batch_size, n_channels, seq_len)
-
-        Returns
-        -------
-        output: Tensor,
-            of shape (batch_size, n_channels, seq_len)
-        """
-        output = super().forward(input)
-        return output
-
+    @add_docstring(compute_sequential_output_shape_docstring)
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
-
-        Parameters
-        ----------
-        seq_len: int,
-            length of the 1d sequence
-        batch_size: int, optional,
-            the batch size, can be None
-
-        Returns
-        -------
-        output_shape: sequence,
-            the output shape of this block, given `seq_len` and `batch_size`
-        """
-        _seq_len = seq_len
-        for module in self:
-            output_shape = module.compute_output_shape(_seq_len, batch_size)
-            _, _, _seq_len = output_shape
-        return output_shape
+        """ """
+        return compute_sequential_output_shape(self, seq_len, batch_size)
 
 
-class DenseTransition(SizeMixin, nn.Sequential):
-    """finished, checked,
+class DenseTransition(nn.Sequential, SizeMixin):
+    """
 
     transition blocks between `DenseMacroBlock`s,
     used to perform sub-sampling,
     and compression of channels if specified
+
     """
 
     __DEBUG__ = True
@@ -511,7 +492,7 @@ class DenseTransition(SizeMixin, nn.Sequential):
         bias: bool = False,
         **config,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -530,6 +511,7 @@ class DenseTransition(SizeMixin, nn.Sequential):
         config: dict,
             other parameters, including
             activation choices, subsampling mode (method), etc.
+
         """
         super().__init__()
         self.__in_channels = in_channels
@@ -555,7 +537,7 @@ class DenseTransition(SizeMixin, nn.Sequential):
                 stride=1,
                 dilation=1,
                 groups=self.__groups,
-                batch_norm=True,
+                norm=True,
                 activation=self.config.activation.lower(),
                 kw_activation=self.config.kw_activation,
                 bias=bias,
@@ -571,48 +553,16 @@ class DenseTransition(SizeMixin, nn.Sequential):
             ),
         )
 
-    def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
-
-        Parameters
-        ----------
-        input: Tensor,
-            of shape (batch_size, n_channels, seq_len)
-
-        Returns
-        -------
-        output: Tensor,
-            of shape (batch_size, n_channels, seq_len)
-        """
-        output = super().forward(input)
-        return output
-
+    @add_docstring(compute_sequential_output_shape_docstring)
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
-
-        Parameters
-        ----------
-        seq_len: int,
-            length of the 1d sequence
-        batch_size: int, optional,
-            the batch size, can be None
-
-        Returns
-        -------
-        output_shape: sequence,
-            the output shape of this block, given `seq_len` and `batch_size`
-        """
-        _seq_len = seq_len
-        for module in self:
-            output_shape = module.compute_output_shape(_seq_len, batch_size)
-            _, _, _seq_len = output_shape
-        return output_shape
+        """ """
+        return compute_sequential_output_shape(self, seq_len, batch_size)
 
 
-class DenseNet(SizeMixin, nn.Sequential):
-    """finished, checked,
+class DenseNet(nn.Sequential, SizeMixin):
+    """
 
     The core part of the SOTA model (framework) of CPSC2020
 
@@ -633,6 +583,7 @@ class DenseNet(SizeMixin, nn.Sequential):
     ----
     1. for `groups` > 1, the concatenated output should be re-organized in the channel dimension?
     2. memory-efficient mode, i.e. storing the `new_features` in a shared memory instead of stacking in newly created `Tensor`s after each mini-block
+
     """
 
     __DEBUG__ = True
@@ -647,7 +598,7 @@ class DenseNet(SizeMixin, nn.Sequential):
     )
 
     def __init__(self, in_channels: int, **config) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -689,6 +640,7 @@ class DenseNet(SizeMixin, nn.Sequential):
             block: dict,
                 other parameters that can be set for the building blocks
             for a full list of configurable parameters, ref. corr. config file
+
         """
         super().__init__()
         self.__in_channels = in_channels
@@ -709,7 +661,7 @@ class DenseNet(SizeMixin, nn.Sequential):
                 stride=1,
                 dilation=1,
                 groups=self.config.groups,
-                batch_norm=True,
+                norm=True,
                 activation=self.config.activation.lower(),
                 kw_activation=self.config.kw_activation,
                 bias=self.config.bias,
@@ -781,41 +733,13 @@ class DenseNet(SizeMixin, nn.Sequential):
                 _, macro_in_channels, _ = dt.compute_output_shape()
                 self.add_module(f"transition_{idx}", dt)
 
-    def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
-
-        Parameters
-        ----------
-        input: Tensor,
-            of shape (batch_size, n_channels, seq_len)
-
-        Returns
-        -------
-        output: Tensor,
-            of shape (batch_size, n_channels, seq_len)
-        """
-        output = super().forward(input)
-        return output
-
+    @add_docstring(compute_sequential_output_shape_docstring)
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
+        """ """
+        return compute_sequential_output_shape(self, seq_len, batch_size)
 
-        Parameters
-        ----------
-        seq_len: int,
-            length of the 1d sequence
-        batch_size: int, optional,
-            the batch size, can be None
-
-        Returns
-        -------
-        output_shape: sequence,
-            the output shape of this block, given `seq_len` and `batch_size`
-        """
-        _seq_len = seq_len
-        for module in self:
-            output_shape = module.compute_output_shape(_seq_len, batch_size)
-            _, _, _seq_len = output_shape
-        return output_shape
+    @property
+    def in_channels(self) -> int:
+        return self.__in_channels
