@@ -4,29 +4,25 @@ Higher Order ResNet
 References
 ----------
 [1] Luo, Z., Sun, Z., Zhou, W., & Kamata, S. I. (2021). Rethinking ResNets: Improved Stacking Strategies With High Order Schemes. arXiv preprint arXiv:2103.15244.
+
 """
 
-from copy import deepcopy
-from itertools import repeat
-from typing import Union, Optional, Sequence, NoReturn
+from typing import NoReturn
 
-import numpy as np
 import torch
 from torch import nn
-from torch import Tensor
 
-from ...cfg import CFG, DEFAULTS
-from ...utils.utils_nn import compute_module_size, SizeMixin
-from ...utils.misc import dict_to_str
-from ...models._nets import (
+from ...cfg import DEFAULTS
+from ...models._nets import (  # noqa: F401
     Activations,
     Conv_Bn_Activation,
     DownSample,
-    ZeroPadding,
+    GlobalContextBlock,
     NonLocalBlock,
     SEBlock,
-    GlobalContextBlock,
+    ZeroPadding,
 )
+from ...utils.utils_nn import SizeMixin
 
 if DEFAULTS.torch_dtype == torch.float64:
     torch.set_default_tensor_type(torch.DoubleTensor)
@@ -39,7 +35,7 @@ __all__ = [
 ]
 
 
-class MidPointResNet(SizeMixin, nn.Module):
+class MidPointResNet(nn.Module, SizeMixin):
     """ """
 
     def __init__(self, in_channels: int, **config) -> NoReturn:
@@ -47,7 +43,7 @@ class MidPointResNet(SizeMixin, nn.Module):
         raise NotImplementedError
 
 
-class RK4ResNet(SizeMixin, nn.Module):
+class RK4ResNet(nn.Module, SizeMixin):
     """ """
 
     def __init__(self, in_channels: int, **config) -> NoReturn:
@@ -55,7 +51,7 @@ class RK4ResNet(SizeMixin, nn.Module):
         raise NotImplementedError
 
 
-class RK8ResNet(SizeMixin, nn.Module):
+class RK8ResNet(nn.Module, SizeMixin):
     """ """
 
     def __init__(self, in_channels: int, **config) -> NoReturn:
