@@ -5,32 +5,29 @@ whose performance however seems exceeded by newer networks
 
 from copy import deepcopy
 from itertools import repeat
-from typing import Union, Optional, Sequence, NoReturn
 from numbers import Real
+from typing import NoReturn, Optional, Sequence, Union
 
-import numpy as np
 import torch
-from torch import nn
-from torch import Tensor
 import torch.nn.functional as F
+from torch import Tensor, nn
 
 from ...cfg import CFG, DEFAULTS
-from ...utils.utils_nn import compute_module_size, SizeMixin
-from ...utils.misc import dict_to_str
-from ...models._nets import (
+from ...models._nets import (  # noqa: F401
     Activations,
     Conv_Bn_Activation,
-    MultiConv,
     DownSample,
-    ZeroPadding,
-    NonLocalBlock,
-    SEBlock,
-    GlobalContextBlock,
+    MultiConv,
     SpaceToDepth,
-    DropPath,
+    ZeroPadding,
     make_attention_layer,
 )
-
+from ...utils.misc import dict_to_str, add_docstring
+from ...utils.utils_nn import (
+    SizeMixin,
+    compute_sequential_output_shape,
+    compute_sequential_output_shape_docstring,
+)
 
 if DEFAULTS.torch_dtype == torch.float64:
     torch.set_default_tensor_type(torch.DoubleTensor)
@@ -43,10 +40,10 @@ __all__ = [
 ]
 
 
-class ResNetBasicBlock(SizeMixin, nn.Module):
-    """finished, checked,
-
+class ResNetBasicBlock(nn.Module, SizeMixin):
+    """
     building blocks for `ResNet`, as implemented in ref. [2] of `ResNet`
+
     """
 
     __DEBUG__ = False
@@ -64,7 +61,7 @@ class ResNetBasicBlock(SizeMixin, nn.Module):
         attn: Optional[dict] = None,
         **config,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -93,6 +90,7 @@ class ResNetBasicBlock(SizeMixin, nn.Module):
             other hyper-parameters, including
             increase channel method, subsample method, dropouts,
             activation choices, weight initializer, and short cut patterns, etc.
+
         """
         super().__init__()
         if dilation > 1:
@@ -144,7 +142,7 @@ class ResNetBasicBlock(SizeMixin, nn.Module):
                     kernel_size=self.__kernel_size,
                     stride=(self.__stride if i == 0 else 1),
                     groups=self.__groups,
-                    batch_norm=True,
+                    norm=True,
                     activation=conv_activation,
                     kw_activation=self.config.kw_activation,
                     kernel_initializer=self.config.kernel_initializer,
@@ -177,7 +175,7 @@ class ResNetBasicBlock(SizeMixin, nn.Module):
             self.out_dropout = None
 
     def _make_shortcut_layer(self) -> Union[nn.Module, None]:
-        """finished, checked,"""
+        """ """
         if self.__DEBUG__:
             print(
                 f"down_scale = {self.__down_scale}, increase_channels = {self.__increase_channels}"
@@ -189,7 +187,7 @@ class ResNetBasicBlock(SizeMixin, nn.Module):
                     in_channels=self.__in_channels,
                     out_channels=self.__out_channels,
                     groups=self.__groups,
-                    batch_norm=True,
+                    norm=True,
                     mode=self.config.subsample_mode,
                     filt_size=self.config.get("filt_size", 3),  # for blur pool
                 )
@@ -202,7 +200,7 @@ class ResNetBasicBlock(SizeMixin, nn.Module):
                         down_scale=self.__down_scale,
                         in_channels=self.__in_channels,
                         out_channels=self.__in_channels,
-                        batch_norm=batch_norm,
+                        norm=batch_norm,
                         mode=self.config.subsample_mode,
                         filt_size=self.config.get("filt_size", 3),  # for blur pool
                     ),
@@ -213,7 +211,7 @@ class ResNetBasicBlock(SizeMixin, nn.Module):
         return shortcut
 
     def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -224,6 +222,7 @@ class ResNetBasicBlock(SizeMixin, nn.Module):
         -------
         out: Tensor,
             of shape (batch_size, n_channels, seq_len)
+
         """
         identity = input
 
@@ -251,7 +250,7 @@ class ResNetBasicBlock(SizeMixin, nn.Module):
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -264,6 +263,7 @@ class ResNetBasicBlock(SizeMixin, nn.Module):
         -------
         output_shape: sequence,
             the output shape of this block, given `seq_len` and `batch_size`
+
         """
         _seq_len = seq_len
         for module in self.main_stream:
@@ -274,11 +274,12 @@ class ResNetBasicBlock(SizeMixin, nn.Module):
         return output_shape
 
 
-class ResNetBottleNeck(SizeMixin, nn.Module):
-    """finished, checked,
+class ResNetBottleNeck(nn.Module, SizeMixin):
+    """
 
     bottle neck blocks for `ResNet`, as implemented in ref. [2] of `ResNet`,
     as for 1D ECG, should be of the "baby-giant-baby" pattern?
+
     """
 
     __DEBUG__ = False
@@ -300,7 +301,7 @@ class ResNetBottleNeck(SizeMixin, nn.Module):
         attn: Optional[dict] = None,
         **config,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -338,6 +339,7 @@ class ResNetBottleNeck(SizeMixin, nn.Module):
             other hyper-parameters, including
             increase channel method, subsample method, dropout,
             activation choices, weight initializer, and short cut patterns, etc.
+
         """
         super().__init__()
         self.__num_convs = 3
@@ -363,7 +365,9 @@ class ResNetBottleNeck(SizeMixin, nn.Module):
         ]
         if self.__DEBUG__:
             print(
-                f"__DEFAULT_BASE_WIDTH__ = {self.__DEFAULT_BASE_WIDTH__}, in_channels = {in_channels}, num_filters = {num_filters}, base_width = {base_width}, neck_num_filters = {neck_num_filters}"
+                f"__DEFAULT_BASE_WIDTH__ = {self.__DEFAULT_BASE_WIDTH__}, "
+                f"in_channels = {in_channels}, num_filters = {num_filters}, "
+                f"base_width = {base_width}, neck_num_filters = {neck_num_filters}."
             )
         self.__base_filter_length = base_filter_length
         self.__kernel_size = [
@@ -424,7 +428,7 @@ class ResNetBottleNeck(SizeMixin, nn.Module):
                     kernel_size=self.__kernel_size[i],
                     stride=(self.__stride if i == self.config.subsample_at else 1),
                     groups=self.__groups[i],
-                    batch_norm=True,
+                    norm=True,
                     activation=conv_activation,
                     kw_activation=self.config.kw_activation,
                     kernel_initializer=self.config.kernel_initializer,
@@ -457,7 +461,7 @@ class ResNetBottleNeck(SizeMixin, nn.Module):
             self.out_dropout = None
 
     def _make_shortcut_layer(self) -> Union[nn.Module, None]:
-        """finished, checked,"""
+        """ """
         if self.__DEBUG__:
             print(
                 f"down_scale = {self.__down_scale}, increase_channels = {self.__increase_channels}"
@@ -469,7 +473,7 @@ class ResNetBottleNeck(SizeMixin, nn.Module):
                     in_channels=self.__in_channels,
                     out_channels=self.__out_channels[-1],
                     groups=self.__base_groups,
-                    batch_norm=True,
+                    norm=True,
                     mode=self.config.subsample_mode,
                     filt_size=self.config.get("filt_size", 3),  # for blur pool
                 )
@@ -482,7 +486,7 @@ class ResNetBottleNeck(SizeMixin, nn.Module):
                         down_scale=self.__down_scale,
                         in_channels=self.__in_channels,
                         out_channels=self.__in_channels,
-                        batch_norm=batch_norm,
+                        norm=batch_norm,
                         mode=self.config.subsample_mode,
                         filt_size=self.config.get("filt_size", 3),  # for blur pool
                     ),
@@ -493,7 +497,7 @@ class ResNetBottleNeck(SizeMixin, nn.Module):
         return shortcut
 
     def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -504,6 +508,7 @@ class ResNetBottleNeck(SizeMixin, nn.Module):
         -------
         out: Tensor,
             of shape (batch_size, n_channels, seq_len)
+
         """
         identity = input
 
@@ -531,7 +536,7 @@ class ResNetBottleNeck(SizeMixin, nn.Module):
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -544,6 +549,7 @@ class ResNetBottleNeck(SizeMixin, nn.Module):
         -------
         output_shape: sequence,
             the output shape of this block, given `seq_len` and `batch_size`
+
         """
         _seq_len = seq_len
         for module in self.main_stream:
@@ -554,7 +560,7 @@ class ResNetBottleNeck(SizeMixin, nn.Module):
         return output_shape
 
 
-class ResNetMacroBlock(SizeMixin, nn.Sequential):
+class ResNetMacroBlock(nn.Sequential, SizeMixin):
     """NOT finished, NOT checked,"""
 
     __DEBUG__ = True
@@ -569,12 +575,12 @@ class ResNetMacroBlock(SizeMixin, nn.Sequential):
         """ """
         raise NotImplementedError
 
-    def compute_output_shape():
+    def compute_output_shape(self):
         """ """
         raise NotImplementedError
 
 
-class ResNetStem(SizeMixin, nn.Sequential):
+class ResNetStem(nn.Sequential, SizeMixin):
     """
     the input stem of ResNet
     """
@@ -593,7 +599,33 @@ class ResNetStem(SizeMixin, nn.Sequential):
         groups: int = 1,
         **config,
     ) -> NoReturn:
-        """ """
+        f"""
+
+        Parameters
+        ----------
+        in_channels: int,
+            the number of input channels
+        out_channels: int or sequence of int,
+            the number of output channels
+        filter_lengths: int or sequence of int,
+            the length of the filter, or equivalently,
+            the kernel size(s) of the convolutions
+        conv_stride: int,
+            the stride of the convolution
+        pool_size: int,
+            the size of the pooling window
+        pool_stride: int,
+            the stride of the pooling window
+        subsample_mode: str,
+            the mode of subsampling, can be one of
+            {DownSample.__MODES__},
+            or "s2d" (with aliases "space_to_depth", "SpaceToDepth")
+        groups: int,
+            the number of groups for the convolution
+        config: dict,
+            the other configs for convolution and pooling
+
+        """
         super().__init__()
         self.__in_channels = in_channels
         self.__out_channels = out_channels
@@ -640,32 +672,16 @@ class ResNetStem(SizeMixin, nn.Sequential):
                 ),
             )
 
+    @add_docstring(compute_sequential_output_shape_docstring)
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
-
-        Parameters
-        ----------
-        seq_len: int,
-            length of the 1d sequence
-        batch_size: int, optional,
-            the batch size, can be None
-
-        Returns
-        -------
-        output_shape: sequence,
-            the output shape of this block, given `seq_len` and `batch_size`
-        """
-        _seq_len = seq_len
-        for module in self:
-            output_shape = module.compute_output_shape(_seq_len, batch_size)
-            _, _, _seq_len = output_shape
-        return output_shape
+        """ """
+        return compute_sequential_output_shape(self, seq_len, batch_size)
 
 
-class ResNet(SizeMixin, nn.Sequential):
-    """finished, checked,
+class ResNet(nn.Sequential, SizeMixin):
+    """
 
     References
     ----------
@@ -676,6 +692,7 @@ class ResNet(SizeMixin, nn.Sequential):
     ----
     1. check performances of activations other than "nn.ReLU", especially mish and swish
     2. add functionality of "replace_stride_with_dilation"
+
     """
 
     __DEBUG__ = False
@@ -690,7 +707,7 @@ class ResNet(SizeMixin, nn.Sequential):
     )
 
     def __init__(self, in_channels: int, **config) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -698,7 +715,7 @@ class ResNet(SizeMixin, nn.Sequential):
             number of channels in the input
         config: dict,
             other hyper-parameters of the Module, ref. corresponding config file
-            key word arguments that have to be set:
+            keyword arguments that have to be set:
             bias: bool,
                 if True, each convolution will have a bias term
             num_blocks: sequence of int,
@@ -719,6 +736,7 @@ class ResNet(SizeMixin, nn.Sequential):
             block: dict,
                 other parameters that can be set for the building blocks
             for a full list of configurable parameters, ref. corr. config file
+
         """
         super().__init__()
         self.__in_channels = in_channels
@@ -797,37 +815,41 @@ class ResNet(SizeMixin, nn.Sequential):
             )
         else:
             self.__filter_lengths = self.config.filter_lengths
-        assert len(self.__filter_lengths) == len(
-            self.config.num_blocks
-        ), f"`config.filter_lengths` indicates {len(self.__filter_lengths)} macro blocks, while `config.num_blocks` indicates {len(self.config.num_blocks)}"
+        assert len(self.__filter_lengths) == len(self.config.num_blocks), (
+            f"`config.filter_lengths` indicates {len(self.__filter_lengths)} macro blocks, "
+            f"while `config.num_blocks` indicates {len(self.config.num_blocks)}"
+        )
         if isinstance(self.config.subsample_lengths, int):
             self.__subsample_lengths = list(
                 repeat(self.config.subsample_lengths, len(self.config.num_blocks))
             )
         else:
             self.__subsample_lengths = self.config.subsample_lengths
-        assert len(self.__subsample_lengths) == len(
-            self.config.num_blocks
-        ), f"`config.subsample_lengths` indicates {len(self.__subsample_lengths)} macro blocks, while `config.num_blocks` indicates {len(self.config.num_blocks)}"
+        assert len(self.__subsample_lengths) == len(self.config.num_blocks), (
+            f"`config.subsample_lengths` indicates {len(self.__subsample_lengths)} macro blocks, "
+            f"while `config.num_blocks` indicates {len(self.config.num_blocks)}"
+        )
         self.__num_filters = self.config.get(
             "num_filters",
             [
-                (2 ** i) * self.input_stem.compute_output_shape()[1]
+                (2**i) * self.input_stem.compute_output_shape()[1]
                 for i in range(len(self.config.num_blocks))
             ],
         )
-        assert len(self.__num_filters) == len(
-            self.config.num_blocks
-        ), f"`config.num_filters` indicates {len(self.__num_filters)} macro blocks, while `config.num_blocks` indicates {len(self.config.num_blocks)}"
+        assert len(self.__num_filters) == len(self.config.num_blocks), (
+            f"`config.num_filters` indicates {len(self.__num_filters)} macro blocks, "
+            f"while `config.num_blocks` indicates {len(self.config.num_blocks)}"
+        )
         if isinstance(self.config.dropouts, Real):
             self.__dropouts = list(
                 repeat(self.config.dropouts, len(self.config.num_blocks))
             )
         else:
             self.__dropouts = self.config.dropouts
-        assert len(self.__dropouts) == len(
-            self.config.num_blocks
-        ), f"`config.dropouts` indicates {len(self.__dropouts)} macro blocks, while `config.num_blocks` indicates {len(self.config.num_blocks)}"
+        assert len(self.__dropouts) == len(self.config.num_blocks), (
+            f"`config.dropouts` indicates {len(self.__dropouts)} macro blocks, "
+            f"while `config.num_blocks` indicates {len(self.config.num_blocks)}"
+        )
 
         # grouped resnet (basic) blocks,
         # number of channels are doubled at the first block of each macro-block
@@ -842,18 +864,20 @@ class ResNet(SizeMixin, nn.Sequential):
                 block_filter_lengths = list(repeat(macro_filter_lengths, nb))
             else:
                 block_filter_lengths = macro_filter_lengths
-            assert (
-                len(block_filter_lengths) == nb
-            ), f"at the {macro_idx}-th macro block, `macro_subsample_lengths` indicates {len(macro_subsample_lengths)} building blocks, while `config.num_blocks[{macro_idx}]` indicates {nb}"
+            assert len(block_filter_lengths) == nb, (
+                f"at the {macro_idx}-th macro block, `macro_subsample_lengths` indicates {len(macro_subsample_lengths)} "
+                f"building blocks, while `config.num_blocks[{macro_idx}]` indicates {nb}"
+            )
             if isinstance(macro_subsample_lengths, int):
                 # subsample at the first building block
                 block_subsample_lengths = list(repeat(1, nb))
                 block_subsample_lengths[0] = macro_subsample_lengths
             else:
                 block_subsample_lengths = macro_subsample_lengths
-            assert (
-                len(block_subsample_lengths) == nb
-            ), f"at the {macro_idx}-th macro block, `macro_subsample_lengths` indicates {len(macro_subsample_lengths)} building blocks, while `config.num_blocks[{macro_idx}]` indicates {nb}"
+            assert len(block_subsample_lengths) == nb, (
+                f"at the {macro_idx}-th macro block, `macro_subsample_lengths` indicates {len(macro_subsample_lengths)} "
+                f"building blocks, while `config.num_blocks[{macro_idx}]` indicates {nb}"
+            )
             if isinstance(self.building_block, Sequence):
                 bb = self.building_block[macro_idx]
                 bb_config = self.config.block[macro_idx]
@@ -880,43 +904,13 @@ class ResNet(SizeMixin, nn.Sequential):
                 block_in_channels = block_num_filters * bb.expansion
             macro_in_channels = macro_num_filters * bb.expansion
 
-    def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
-
-        Parameters
-        ----------
-        input: Tensor,
-            of shape (batch_size, n_channels, seq_len)
-
-        Returns
-        -------
-        output: Tensor,
-            of shape (batch_size, n_channels, seq_len)
-        """
-        output = input
-        for module in self:
-            output = module(output)
-        return output
-
+    @add_docstring(compute_sequential_output_shape_docstring)
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
+        """ """
+        return compute_sequential_output_shape(self, seq_len, batch_size)
 
-        Parameters
-        ----------
-        seq_len: int,
-            length of the 1d sequence
-        batch_size: int, optional,
-            the batch size, can be None
-
-        Returns
-        -------
-        output_shape: sequence,
-            the output shape of this block, given `seq_len` and `batch_size`
-        """
-        _seq_len = seq_len
-        for module in self:
-            output_shape = module.compute_output_shape(_seq_len, batch_size)
-            _, _, _seq_len = output_shape
-        return output_shape
+    @property
+    def in_channels(self) -> int:
+        return self.__in_channels
