@@ -4,31 +4,30 @@ usually the SOTA image classifier,
 however seems not have been used in physiological signal processing tasks
 """
 
-import math
 from copy import deepcopy
 from itertools import repeat
-from typing import Union, Optional, Sequence, NoReturn
 from numbers import Real
+from typing import NoReturn, Optional, Sequence, Union
 
-import numpy as np
 import torch
-from torch import nn
-from torch import Tensor
-import torch.nn.functional as F
+from torch import Tensor, nn
 
 from ...cfg import CFG, DEFAULTS
-from ...utils.utils_nn import compute_module_size, SizeMixin
-from ...utils.misc import dict_to_str, list_sum
-from ...models._nets import (
+from ...models._nets import (  # noqa: F401
     Conv_Bn_Activation,
-    SeparableConv,
-    MultiConv,
     DownSample,
+    GlobalContextBlock,
+    MultiConv,
     NonLocalBlock,
     SEBlock,
-    GlobalContextBlock,
+    SeparableConv,
+)
+from ...utils.misc import dict_to_str, add_docstring
+from ...utils.utils_nn import (
+    SizeMixin,
+    compute_sequential_output_shape,
+    compute_sequential_output_shape_docstring,
 )
-
 
 if DEFAULTS.torch_dtype == torch.float64:
     torch.set_default_tensor_type(torch.DoubleTensor)
@@ -55,7 +54,7 @@ _DEFAULT_CONV_CONFIGS = CFG(
 )
 
 
-class XceptionMultiConv(SizeMixin, nn.Module):
+class XceptionMultiConv(nn.Module, SizeMixin):
     """
 
     -> n(2 or 3) x (activation -> norm -> sep_conv) (-> optional sub-sample) ->
@@ -77,7 +76,7 @@ class XceptionMultiConv(SizeMixin, nn.Module):
         dropouts: Union[Sequence[float], float] = 0.0,
         **config,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -161,7 +160,7 @@ class XceptionMultiConv(SizeMixin, nn.Module):
                 out_channels=self.__num_filters[-1],
                 groups=self.__groups,
                 kernel_size=1,
-                batch_norm=self.config.batch_norm,
+                norm=self.config.get("norm", self.config.get("batch_norm")),
                 mode="conv",
             )
         else:
@@ -169,7 +168,7 @@ class XceptionMultiConv(SizeMixin, nn.Module):
             self.shortcut = None
 
     def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -193,7 +192,7 @@ class XceptionMultiConv(SizeMixin, nn.Module):
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -215,7 +214,7 @@ class XceptionMultiConv(SizeMixin, nn.Module):
         return output_shape
 
 
-class XceptionEntryFlow(SizeMixin, nn.Sequential):
+class XceptionEntryFlow(nn.Sequential, SizeMixin):
     """
 
     Entry flow of the Xception model,
@@ -242,7 +241,7 @@ class XceptionEntryFlow(SizeMixin, nn.Sequential):
         block_dropouts: Union[float, Sequence[float]] = 0.0,
         **config,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -379,7 +378,7 @@ class XceptionEntryFlow(SizeMixin, nn.Sequential):
                 )
 
     def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -397,7 +396,7 @@ class XceptionEntryFlow(SizeMixin, nn.Sequential):
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -420,7 +419,7 @@ class XceptionEntryFlow(SizeMixin, nn.Sequential):
         return output_shape
 
 
-class XceptionMiddleFlow(SizeMixin, nn.Sequential):
+class XceptionMiddleFlow(nn.Sequential, SizeMixin):
     """
 
     Middle flow of the Xception model,
@@ -441,7 +440,7 @@ class XceptionMiddleFlow(SizeMixin, nn.Sequential):
         block_dropouts: Union[float, Sequence[float]] = 0.0,
         **config,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -527,7 +526,7 @@ class XceptionMiddleFlow(SizeMixin, nn.Sequential):
                 )
 
     def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -545,7 +544,7 @@ class XceptionMiddleFlow(SizeMixin, nn.Sequential):
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -568,7 +567,7 @@ class XceptionMiddleFlow(SizeMixin, nn.Sequential):
         return output_shape
 
 
-class XceptionExitFlow(SizeMixin, nn.Sequential):
+class XceptionExitFlow(nn.Sequential, SizeMixin):
     """
 
     Exit flow of the Xception model,
@@ -594,7 +593,7 @@ class XceptionExitFlow(SizeMixin, nn.Sequential):
         block_dropouts: Union[float, Sequence[float]] = 0.0,
         **config,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -726,7 +725,7 @@ class XceptionExitFlow(SizeMixin, nn.Sequential):
         )
 
     def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -744,7 +743,7 @@ class XceptionExitFlow(SizeMixin, nn.Sequential):
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -767,7 +766,7 @@ class XceptionExitFlow(SizeMixin, nn.Sequential):
         return output_shape
 
 
-class Xception(SizeMixin, nn.Sequential):
+class Xception(nn.Sequential, SizeMixin):
     """
 
     References
@@ -781,7 +780,7 @@ class Xception(SizeMixin, nn.Sequential):
     __name__ = "Xception"
 
     def __init__(self, in_channels: int, **config) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -823,7 +822,7 @@ class Xception(SizeMixin, nn.Sequential):
         )
 
     def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -838,25 +837,13 @@ class Xception(SizeMixin, nn.Sequential):
         output = super().forward(input)
         return output
 
+    @add_docstring(compute_sequential_output_shape_docstring)
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
+        """ """
+        return compute_sequential_output_shape(self, seq_len, batch_size)
 
-        Parameters
-        ----------
-        seq_len: int,
-            length of the 1d sequence
-        batch_size: int, optional,
-            the batch size, can be None
-
-        Returns
-        -------
-        output_shape: sequence,
-            the output shape of this `MultiConv` layer, given `seq_len` and `batch_size`
-        """
-        _seq_len = seq_len
-        for module in self:
-            output_shape = module.compute_output_shape(_seq_len, batch_size)
-            _, _, _seq_len = output_shape
-        return output_shape
+    @property
+    def in_channels(self) -> int:
+        return self.__in_channels
