@@ -1,13 +1,11 @@
 """custom loss functions"""
 
 from numbers import Real
-from typing import Union, Sequence, Tuple, List, Optional, NoReturn, Any
+from typing import Any, NoReturn, Optional, Union
 
 import torch
-from torch import nn
-from torch import Tensor
 import torch.nn.functional as F
-
+from torch import Tensor, nn
 
 __all__ = [
     "WeightedBCELoss",
@@ -26,7 +24,7 @@ def weighted_binary_cross_entropy(
     size_average: bool = True,
     reduce: bool = True,
 ) -> Tensor:
-    """finished, checked,
+    """
 
     Parameters
     ----------
@@ -68,7 +66,7 @@ def weighted_binary_cross_entropy(
 
 
 class WeightedBCELoss(nn.Module):
-    """finished, checked,
+    """
 
     Reference (original source):
     https://github.com/pytorch/pytorch/issues/5660#issuecomment-403770305
@@ -141,12 +139,12 @@ class WeightedBCELoss(nn.Module):
 
 
 class BCEWithLogitsWithClassWeightLoss(nn.BCEWithLogitsLoss):
-    """finished, checked,"""
+    """ """
 
     __name__ = "BCEWithLogitsWithClassWeightsLoss"
 
     def __init__(self, class_weight: Tensor) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -177,16 +175,16 @@ class BCEWithLogitsWithClassWeightLoss(nn.BCEWithLogitsLoss):
 
 
 class MaskedBCEWithLogitsLoss(nn.BCEWithLogitsLoss):
-    """finished, checked,"""
+    """ """
 
     __name__ = "MaskedBCEWithLogitsLoss"
 
     def __init__(self) -> NoReturn:
-        """finished, checked,"""
+        """ """
         super().__init__(reduction="none")
 
     def forward(self, input: Tensor, target: Tensor, weight_mask: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -240,7 +238,7 @@ class FocalLoss(nn.modules.loss._WeightedLoss):
         multi_label: bool = True,
         **kwargs: Any
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -290,7 +288,7 @@ class FocalLoss(nn.modules.loss._WeightedLoss):
         return self.class_weight
 
     def forward(self, input: Tensor, target: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -323,7 +321,7 @@ class FocalLoss(nn.modules.loss._WeightedLoss):
 
 
 class AsymmetricLoss(nn.Module):
-    """finished, checked,
+    r"""
 
     The asymmetric loss is defined as
 
@@ -353,7 +351,7 @@ class AsymmetricLoss(nn.Module):
         reduction: str = "mean",
         implementation: str = "alibaba-miil",
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -413,7 +411,7 @@ class AsymmetricLoss(nn.Module):
             ) = self.xs_neg = self.loss = self.loss_pos = self.loss_neg = None
 
     def forward(self, input: Tensor, target: Tensor) -> Tensor:
-        """ " finished, checked,
+        """
 
         Parameters
         ----------
@@ -433,7 +431,7 @@ class AsymmetricLoss(nn.Module):
             return self._forward_deep_psp(input, target)
 
     def _forward_deep_psp(self, input: Tensor, target: Tensor) -> Tensor:
-        """ " finished, checked,
+        """
 
         Parameters
         ----------
@@ -480,7 +478,7 @@ class AsymmetricLoss(nn.Module):
         return self.loss
 
     def _forward_alibaba_miil(self, input: Tensor, target: Tensor) -> Tensor:
-        """ " finished, checked,
+        """
 
         Parameters
         ----------
