@@ -5,34 +5,31 @@ the main differences to a normal Unet are that
 1. at the bottom, subtraction (and concatenation) is used
 2. uses triple convolutions at each block, instead of double convolutions
 3. dropout is used between certain convolutional layers ("cba" layers indeed)
+
 """
 
 from copy import deepcopy
-from collections import OrderedDict
 from itertools import repeat
-from typing import Union, Optional, Sequence, List, Tuple, NoReturn
-from numbers import Real
+from typing import NoReturn, Optional, Sequence, Union
 
 import numpy as np
 import torch
-from torch import nn
-from torch import Tensor
-import torch.nn.functional as F
+from torch import Tensor, nn
 
 from ...cfg import CFG, DEFAULTS
-from ...utils.utils_nn import (
-    compute_deconv_output_shape,
-    compute_module_size,
-    SizeMixin,
-    CkptMixin,
-)
-from ...utils.misc import dict_to_str
-from ...models._nets import (
-    Conv_Bn_Activation,
-    MultiConv,
+from ...models._nets import (  # noqa: F401
     BranchedConv,
+    Conv_Bn_Activation,
     DownSample,
-    ZeroPadding,
+    MultiConv,
+)
+from ...utils.misc import dict_to_str, add_docstring
+from ...utils.utils_nn import (
+    CkptMixin,
+    SizeMixin,
+    compute_deconv_output_shape,
+    compute_sequential_output_shape,
+    compute_sequential_output_shape_docstring,
 )
 
 if DEFAULTS.torch_dtype == torch.float64:
@@ -64,7 +61,7 @@ class TripleConv(MultiConv):
         out_activation: bool = True,
         **config,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -111,7 +108,7 @@ class TripleConv(MultiConv):
         )
 
 
-class DownTripleConv(SizeMixin, nn.Sequential):
+class DownTripleConv(nn.Sequential, SizeMixin):
     """ """
 
     __DEBUG__ = False
@@ -129,7 +126,7 @@ class DownTripleConv(SizeMixin, nn.Sequential):
         mode: str = "max",
         **config,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -169,7 +166,7 @@ class DownTripleConv(SizeMixin, nn.Sequential):
             DownSample(
                 down_scale=self.__down_scale,
                 in_channels=self.__in_channels,
-                batch_norm=False,
+                norm=False,
                 mode=mode,
             ),
         )
@@ -187,7 +184,7 @@ class DownTripleConv(SizeMixin, nn.Sequential):
         )
 
     def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -202,31 +199,15 @@ class DownTripleConv(SizeMixin, nn.Sequential):
         out = super().forward(input)
         return out
 
+    @add_docstring(compute_sequential_output_shape_docstring)
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
-
-        Parameters
-        ----------
-        seq_len: int,
-            length of the 1d sequence
-        batch_size: int, optional,
-            the batch size, can be None
-
-        Returns
-        -------
-        output_shape: sequence,
-            the output shape of this `DownDoubleConv` layer, given `seq_len` and `batch_size`
-        """
-        _seq_len = seq_len
-        for module in self:
-            output_shape = module.compute_output_shape(seq_len=_seq_len)
-            _, _, _seq_len = output_shape
-        return output_shape
+        """ """
+        return compute_sequential_output_shape(self, seq_len, batch_size)
 
 
-class DownBranchedDoubleConv(SizeMixin, nn.Module):
+class DownBranchedDoubleConv(nn.Module, SizeMixin):
     """
     the bottom block of the `subtract_unet`
     """
@@ -247,7 +228,7 @@ class DownBranchedDoubleConv(SizeMixin, nn.Module):
         mode: str = "max",
         **config,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -285,7 +266,7 @@ class DownBranchedDoubleConv(SizeMixin, nn.Module):
         self.down_sample = DownSample(
             down_scale=self.__down_scale,
             in_channels=self.__in_channels,
-            batch_norm=False,
+            norm=False,
             mode=mode,
         )
         self.branched_conv = BranchedConv(
@@ -300,7 +281,7 @@ class DownBranchedDoubleConv(SizeMixin, nn.Module):
         )
 
     def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -324,7 +305,7 @@ class DownBranchedDoubleConv(SizeMixin, nn.Module):
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -352,7 +333,7 @@ class DownBranchedDoubleConv(SizeMixin, nn.Module):
         return output_shape
 
 
-class UpTripleConv(SizeMixin, nn.Module):
+class UpTripleConv(nn.Module, SizeMixin):
     """
     Upscaling then double conv, with input of corr. down layer concatenated
     up sampling --> conv (conv --> (dropout -->) conv --> (dropout -->) conv)
@@ -453,7 +434,7 @@ class UpTripleConv(SizeMixin, nn.Module):
         )
 
     def forward(self, input: Tensor, down_output: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -473,7 +454,7 @@ class UpTripleConv(SizeMixin, nn.Module):
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -503,8 +484,8 @@ class UpTripleConv(SizeMixin, nn.Module):
         return output_shape
 
 
-class ECG_SUBTRACT_UNET(CkptMixin, SizeMixin, nn.Module):
-    """finished, checked,
+class ECG_SUBTRACT_UNET(nn.Module, CkptMixin, SizeMixin):
+    """
 
     entry 0433 of CPSC2019
     """
@@ -513,7 +494,7 @@ class ECG_SUBTRACT_UNET(CkptMixin, SizeMixin, nn.Module):
     __name__ = "ECG_SUBTRACT_UNET"
 
     def __init__(self, classes: Sequence[str], n_leads: int, config: dict) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -639,7 +620,7 @@ class ECG_SUBTRACT_UNET(CkptMixin, SizeMixin, nn.Module):
             kernel_size=self.config.out_filter_length,
             stride=1,
             groups=self.config.groups,
-            batch_norm=self.config.out_batch_norm,
+            norm=self.config.get("out_norm", self.config.get("out_batch_norm")),
             activation=None,
             kernel_initializer=self.config.kernel_initializer,
             kw_initializer=self.config.kw_initializer,
@@ -657,7 +638,7 @@ class ECG_SUBTRACT_UNET(CkptMixin, SizeMixin, nn.Module):
         self.sigmoid = nn.Sigmoid()
 
     def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -718,7 +699,7 @@ class ECG_SUBTRACT_UNET(CkptMixin, SizeMixin, nn.Module):
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
