@@ -6,34 +6,29 @@ References
 ----------
 [1] Moskalenko, Viktor, Nikolai Zolotykh, and Grigory Osipov. "Deep Learning for ECG Segmentation." International Conference on Neuroinformatics. Springer, Cham, 2019.
 [2] https://github.com/milesial/Pytorch-UNet/
+
 """
 
 from copy import deepcopy
-from collections import OrderedDict
-from itertools import repeat
-from typing import Union, Optional, Sequence, NoReturn
-from numbers import Real
+from typing import NoReturn, Optional, Sequence, Union
 
-import numpy as np
 import torch
-from torch import nn
-from torch import Tensor
 import torch.nn.functional as F
-from easydict import EasyDict as ED
+from torch import Tensor, nn
 
-from ...cfg import DEFAULTS
-from ...utils.utils_nn import (
-    compute_deconv_output_shape,
-    compute_module_size,
-    SizeMixin,
-    CkptMixin,
-)
-from ...utils.misc import dict_to_str
+from ...cfg import DEFAULTS, CFG
 from ...models._nets import (
     Conv_Bn_Activation,
-    MultiConv,
     DownSample,
-    ZeroPadding,
+    MultiConv,
+)
+from ...utils.misc import dict_to_str, add_docstring
+from ...utils.utils_nn import (
+    CkptMixin,
+    SizeMixin,
+    compute_deconv_output_shape,
+    compute_sequential_output_shape,
+    compute_sequential_output_shape_docstring,
 )
 
 if DEFAULTS.torch_dtype == torch.float64:
@@ -53,6 +48,7 @@ class DoubleConv(MultiConv):
     References
     ----------
     https://github.com/milesial/Pytorch-UNet/blob/master/unet/unet_parts.py
+
     """
 
     __DEBUG__ = False
@@ -70,7 +66,7 @@ class DoubleConv(MultiConv):
         mid_channels: Optional[int] = None,
         **config,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -96,6 +92,7 @@ class DoubleConv(MultiConv):
             other parameters, including
             activation choices, weight initializer, batch normalization choices, etc.
             for the convolutional layers
+
         """
         _mid_channels = mid_channels if mid_channels is not None else out_channels
         _out_channels = [_mid_channels, out_channels]
@@ -112,7 +109,7 @@ class DoubleConv(MultiConv):
         )
 
 
-class DownDoubleConv(SizeMixin, nn.Sequential):
+class DownDoubleConv(nn.Sequential, SizeMixin):
     """
     Downscaling with maxpool then double conv
     down sample (maxpool) --> double conv (conv --> conv)
@@ -122,6 +119,7 @@ class DownDoubleConv(SizeMixin, nn.Sequential):
     References
     ----------
     https://github.com/milesial/Pytorch-UNet/blob/master/unet/unet_parts.py
+
     """
 
     __DEBUG__ = False
@@ -140,7 +138,7 @@ class DownDoubleConv(SizeMixin, nn.Sequential):
         mode: str = "max",
         **config,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -166,6 +164,7 @@ class DownDoubleConv(SizeMixin, nn.Sequential):
             other parameters, including
             activation choices, weight initializer, batch normalization choices, etc.
             for the convolutional layers
+
         """
         super().__init__()
         self.__mode = mode.lower()
@@ -174,7 +173,7 @@ class DownDoubleConv(SizeMixin, nn.Sequential):
         self.__in_channels = in_channels
         self.__mid_channels = mid_channels if mid_channels is not None else out_channels
         self.__out_channels = out_channels
-        self.config = ED(deepcopy(config))
+        self.config = CFG(deepcopy(config))
         if self.__DEBUG__:
             print(
                 f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
@@ -185,7 +184,7 @@ class DownDoubleConv(SizeMixin, nn.Sequential):
             DownSample(
                 down_scale=self.__down_scale,
                 in_channels=self.__in_channels,
-                batch_norm=False,
+                norm=False,
                 mode=mode,
             ),
         )
@@ -204,7 +203,7 @@ class DownDoubleConv(SizeMixin, nn.Sequential):
         )
 
     def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -215,35 +214,20 @@ class DownDoubleConv(SizeMixin, nn.Sequential):
         -------
         output: Tensor,
             of shape (batch_size, n_channels, seq_len)
+
         """
         out = super().forward(input)
         return out
 
+    @add_docstring(compute_sequential_output_shape_docstring)
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
-
-        Parameters
-        ----------
-        seq_len: int,
-            length of the 1d sequence
-        batch_size: int, optional,
-            the batch size, can be None
-
-        Returns
-        -------
-        output_shape: sequence,
-            the output shape of this `DownDoubleConv` layer, given `seq_len` and `batch_size`
-        """
-        _seq_len = seq_len
-        for module in self:
-            output_shape = module.compute_output_shape(seq_len=_seq_len)
-            _, _, _seq_len = output_shape
-        return output_shape
+        """ """
+        return compute_sequential_output_shape(self, seq_len, batch_size)
 
 
-class UpDoubleConv(SizeMixin, nn.Module):
+class UpDoubleConv(nn.Module, SizeMixin):
     """
     Upscaling then double conv, with input of corr. down layer concatenated
     up sampling --> conv (conv --> conv)
@@ -252,6 +236,7 @@ class UpDoubleConv(SizeMixin, nn.Module):
     extra input
 
     channels are shrinked after up sampling
+
     """
 
     __DEBUG__ = False
@@ -277,7 +262,7 @@ class UpDoubleConv(SizeMixin, nn.Module):
         mid_channels: Optional[int] = None,
         **config,
     ) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -308,6 +293,7 @@ class UpDoubleConv(SizeMixin, nn.Module):
             other parameters, including
             activation choices, weight initializer, batch normalization choices, etc.
             for the deconvolutional layers
+
         """
         super().__init__()
         self.__up_scale = up_scale
@@ -319,7 +305,7 @@ class UpDoubleConv(SizeMixin, nn.Module):
         self.__deconv_filter_length = deconv_filter_length
         self.__mode = mode.lower()
         assert self.__mode in self.__MODES__
-        self.config = ED(deepcopy(config))
+        self.config = CFG(deepcopy(config))
         if self.__DEBUG__:
             print(
                 f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
@@ -344,13 +330,8 @@ class UpDoubleConv(SizeMixin, nn.Module):
                 scale_factor=self.__up_scale,
                 mode=mode,
             )
-        self.zero_pad = ZeroPadding(
-            in_channels=self.__in_channels,
-            out_channels=self.__in_channels + self.__in_channels // 2,
-            loc="head",
-        )
         self.conv = DoubleConv(
-            in_channels=2 * self.__in_channels,
+            in_channels=self.__in_channels + self.__in_channels // 2,
             out_channels=self.__out_channels,
             filter_lengths=filter_lengths,
             subsample_lengths=1,
@@ -360,7 +341,7 @@ class UpDoubleConv(SizeMixin, nn.Module):
         )
 
     def forward(self, input: Tensor, down_output: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -375,13 +356,13 @@ class UpDoubleConv(SizeMixin, nn.Module):
         -------
         output: Tensor,
             of shape (batch_size, n_channels'', seq_len')
+
         """
         output = self.up(input)
-        output = self.zero_pad(output)
 
         diff_sig_len = down_output.shape[-1] - output.shape[-1]
-
         output = F.pad(output, [diff_sig_len // 2, diff_sig_len - diff_sig_len // 2])
+
         # TODO: consider the case `groups` > 1 when concatenating
         output = torch.cat(
             [down_output, output], dim=1
@@ -393,7 +374,7 @@ class UpDoubleConv(SizeMixin, nn.Module):
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -406,6 +387,7 @@ class UpDoubleConv(SizeMixin, nn.Module):
         -------
         output_shape: sequence,
             the output shape of this `UpDoubleConv` layer, given `seq_len` and `batch_size`
+
         """
         _sep_len = seq_len
         if self.__mode == "deconv":
@@ -419,14 +401,12 @@ class UpDoubleConv(SizeMixin, nn.Module):
         else:
             output_shape = [batch_size, self.__in_channels, self.__up_scale * _sep_len]
         _, _, _seq_len = output_shape
-        output_shape = self.zero_pad.compute_output_shape(_seq_len, batch_size)
-        _, _, _seq_len = output_shape
         output_shape = self.conv.compute_output_shape(_seq_len, batch_size)
         return output_shape
 
 
-class ECG_UNET(CkptMixin, SizeMixin, nn.Module):
-    """finished, checked,
+class ECG_UNET(nn.Module, CkptMixin, SizeMixin):
+    """
 
     UNet for (multi-lead) ECG wave delineation
 
@@ -434,13 +414,14 @@ class ECG_UNET(CkptMixin, SizeMixin, nn.Module):
     ----------
     [1] Moskalenko, Viktor, Nikolai Zolotykh, and Grigory Osipov. "Deep Learning for ECG Segmentation." International Conference on Neuroinformatics. Springer, Cham, 2019.
     [2] https://github.com/milesial/Pytorch-UNet/
+
     """
 
     __DEBUG__ = False
     __name__ = "ECG_UNET"
 
     def __init__(self, classes: Sequence[str], n_leads: int, config: dict) -> NoReturn:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -451,13 +432,14 @@ class ECG_UNET(CkptMixin, SizeMixin, nn.Module):
         config: dict,
             other hyper-parameters, including kernel sizes, etc.
             ref. the corresponding config file
+
         """
         super().__init__()
         self.classes = list(classes)
         self.n_classes = len(classes)  # final out_channels
         self.__out_channels = self.n_classes
         self.__in_channels = n_leads
-        self.config = ED(deepcopy(config))
+        self.config = CFG(deepcopy(config))
         if self.__DEBUG__:
             print(
                 f"configuration of {self.__name__} is as follows\n{dict_to_str(self.config)}"
@@ -534,7 +516,7 @@ class ECG_UNET(CkptMixin, SizeMixin, nn.Module):
             kernel_size=self.config.out_filter_length,
             stride=1,
             groups=self.config.groups,
-            batch_norm=self.config.out_batch_norm,
+            norm=self.config.get("out_norm", self.config.get("out_batch_norm")),
             activation=None,
             kernel_initializer=self.config.kernel_initializer,
             kw_initializer=self.config.kw_initializer,
@@ -552,7 +534,7 @@ class ECG_UNET(CkptMixin, SizeMixin, nn.Module):
         self.sigmoid = nn.Sigmoid()
 
     def forward(self, input: Tensor) -> Tensor:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -563,6 +545,7 @@ class ECG_UNET(CkptMixin, SizeMixin, nn.Module):
         -------
         output: Tensor,
             of shape (batch_size, n_channels, seq_len)
+
         """
         to_concat = [self.init_conv(input)]
         # if self.__DEBUG__:
@@ -601,7 +584,7 @@ class ECG_UNET(CkptMixin, SizeMixin, nn.Module):
     def compute_output_shape(
         self, seq_len: Optional[int] = None, batch_size: Optional[int] = None
     ) -> Sequence[Union[int, None]]:
-        """finished, checked,
+        """
 
         Parameters
         ----------
@@ -614,6 +597,7 @@ class ECG_UNET(CkptMixin, SizeMixin, nn.Module):
         -------
         output_shape: sequence,
             the output shape of this model, given `seq_len` and `batch_size`
+
         """
         output_shape = (batch_size, seq_len, self.n_classes)
         return output_shape
