@@ -2,18 +2,12 @@
 PreProcessors as torch.nn.Module
 """
 
-from .preproc_manager import PreprocManager
 from .bandpass import BandPass
 from .baseline_remove import BaselineRemove
-from .normalize import (
-    Normalize,
-    MinMaxNormalize,
-    NaiveNormalize,
-    ZScoreNormalize,
-)
+from .normalize import MinMaxNormalize, NaiveNormalize, Normalize, ZScoreNormalize
+from .preproc_manager import PreprocManager
 from .resample import Resample
 
-
 __all__ = [
     "PreprocManager",
     "BandPass",
