@@ -1,33 +1,38 @@
 """
 """
-import os, sys, re, logging, datetime, random
-from pathlib import Path
-from functools import reduce
-from collections import namedtuple
-from glob import glob
+
+import datetime
+import logging
+import os
+import re
+import sys
+import signal
+import time
+import warnings
+import inspect
+from contextlib import contextmanager
 from copy import deepcopy
+from functools import reduce, wraps
+from glob import glob
+from numbers import Number, Real
+from pathlib import Path
 from typing import (
-    Union,
-    Optional,
-    NoReturn,
     Any,
-    List,
+    Callable,
     Dict,
-    Tuple,
-    Sequence,
     Iterable,
+    List,
+    NoReturn,
+    Optional,
+    Sequence,
+    Union,
+    Tuple,
 )
-from numbers import Real, Number
 
 import numpy as np
 import pandas as pd
-from scipy import interpolate
-from sklearn.utils import compute_class_weight
-from wfdb.io import _header
-from wfdb import Record, MultiRecord
-
-from ..cfg import CFG, DEFAULTS
 
+from ..cfg import DEFAULTS
 
 __all__ = [
     "get_record_list_recursive",
@@ -38,17 +43,9 @@ __all__ = [
     "diff_with_step",
     "ms2samples",
     "samples2ms",
-    "get_mask",
-    "class_weight_to_sample_weight",
     "plot_single_lead",
     "init_logger",
     "get_date_str",
-    "rdheader",
-    "ensure_lead_fmt",
-    "ensure_siglen",
-    "ECGWaveForm",
-    "masks_to_waveforms",
-    "mask_to_intervals",
     "list_sum",
     "read_log_txt",
     "read_event_scalars",
@@ -56,15 +53,17 @@ __all__ = [
     "default_class_repr",
     "ReprMixin",
     "MovingAverage",
-    "uniform",
     "nildent",
     "isclass",
-    "strafified_train_test_split",
+    "add_docstring",
+    "deprecate_kwargs",
+    "timeout",
+    "Timer",
 ]
 
 
 def get_record_list_recursive(db_dir: Union[str, Path], rec_ext: str) -> List[str]:
-    """finished, checked,
+    """
 
     get the list of records in `db_dir` recursively,
     for example, there are two folders "patient1", "patient2" in `db_dir`,
@@ -105,7 +104,7 @@ def get_record_list_recursive(db_dir: Union[str, Path], rec_ext: str) -> List[st
 
 
 def get_record_list_recursive2(db_dir: Union[str, Path], rec_pattern: str) -> List[str]:
-    """finished, checked,
+    """
 
     get the list of records in `db_dir` recursively,
     for example, there are two folders "patient1", "patient2" in `db_dir`,
@@ -147,7 +146,7 @@ def get_record_list_recursive2(db_dir: Union[str, Path], rec_pattern: str) -> Li
 def get_record_list_recursive3(
     db_dir: Union[str, Path], rec_patterns: Union[str, Dict[str, str]]
 ) -> Union[List[str], Dict[str, List[str]]]:
-    """finished, checked,
+    r"""
 
     get the list of records in `db_dir` recursively,
     for example, there are two folders "patient1", "patient2" in `db_dir`,
@@ -215,7 +214,7 @@ def get_record_list_recursive3(
 def dict_to_str(
     d: Union[dict, list, tuple], current_depth: int = 1, indent_spaces: int = 4
 ) -> str:
-    """finished, checked,
+    """
 
     convert a (possibly) nested dict into a `str` of json-like formatted form,
     this nested dict might also contain lists or tuples of dict (and of str, int, etc.)
@@ -237,7 +236,7 @@ def dict_to_str(
     """
     assert isinstance(d, (dict, list, tuple))
     if len(d) == 0:
-        s = f"{{}}" if isinstance(d, dict) else f"[]"
+        s = r"{}" if isinstance(d, dict) else "[]"
         return s
     # flat_types = (Number, bool, str,)
     flat_types = (
@@ -296,7 +295,7 @@ def dict_to_str(
 
 
 def str2bool(v: Union[str, bool]) -> bool:
-    """finished, checked,
+    """
 
     converts a "boolean" value possibly in the format of str to bool
 
@@ -327,7 +326,7 @@ def str2bool(v: Union[str, bool]) -> bool:
 
 
 def diff_with_step(a: np.ndarray, step: int = 1, **kwargs) -> np.ndarray:
-    """finished, checked,
+    """
 
     compute a[n+step] - a[n] for all valid n
 
@@ -354,7 +353,7 @@ def diff_with_step(a: np.ndarray, step: int = 1, **kwargs) -> np.ndarray:
 
 
 def ms2samples(t: Real, fs: Real) -> int:
-    """finished, checked,
+    """
 
     convert time `t` with units in ms to number of samples
 
@@ -376,7 +375,7 @@ def ms2samples(t: Real, fs: Real) -> int:
 
 
 def samples2ms(n_samples: int, fs: Real) -> Real:
-    """finished, checked,
+    """
 
     inverse function of `ms2samples`
 
@@ -397,100 +396,6 @@ def samples2ms(n_samples: int, fs: Real) -> Real:
     return t
 
 
-def get_mask(
-    shape: Union[int, Sequence[int]],
-    critical_points: np.ndarray,
-    left_bias: int,
-    right_bias: int,
-    return_fmt: str = "mask",
-) -> Union[np.ndarray, list]:
-    """finished, checked,
-
-    get the mask around the `critical_points`
-
-    Parameters
-    ----------
-    shape: int, or sequence of int,
-        shape of the mask (and the original data)
-    critical_points: ndarray,
-        indices (of the last dimension) of the points around which to be masked (value 1)
-    left_bias: int, non-negative
-        bias to the left of the critical points for the mask
-    right_bias: int, non-negative
-        bias to the right of the critical points for the mask
-    return_fmt: str, default "mask",
-        format of the return values,
-        "mask" for the usual mask,
-        can also be "intervals", which consists of a list of intervals
-
-    Returns
-    -------
-    mask: ndarray or list,
-        the mask array
-
-    """
-    if isinstance(shape, int):
-        shape = (shape,)
-    l_itv = [
-        [max(0, cp - left_bias), min(shape[-1], cp + right_bias)]
-        for cp in critical_points
-    ]
-    if return_fmt.lower() == "mask":
-        mask = np.zeros(shape=shape, dtype=int)
-        for itv in l_itv:
-            mask[..., itv[0] : itv[1]] = 1
-    elif return_fmt.lower() == "intervals":
-        mask = l_itv
-    return mask
-
-
-def class_weight_to_sample_weight(
-    y: np.ndarray, class_weight: Union[str, List[float], np.ndarray, dict] = "balanced"
-) -> np.ndarray:
-    """finished, checked,
-
-    transform class weight to sample weight
-
-    Parameters
-    ----------
-    y: ndarray,
-        the label (class) of each sample
-    class_weight: str, or list, or ndarray, or dict, default "balanced",
-        the weight for each sample class,
-        if is "balanced", the class weight will automatically be given by
-        if `y` is of string type, then `class_weight` should be a dict,
-        if `y` is of numeric type, and `class_weight` is array_like,
-        then the labels (`y`) should be continuous and start from 0
-
-    Returns
-    -------
-    sample_weight: ndarray,
-        the array of sample weight
-
-    """
-    if not class_weight:
-        sample_weight = np.ones_like(y, dtype=float)
-        return sample_weight
-
-    try:
-        sample_weight = y.copy().astype(int)
-    except:
-        sample_weight = y.copy()
-        assert (
-            isinstance(class_weight, dict) or class_weight.lower() == "balanced"
-        ), "if `y` are of type str, then class_weight should be \042balanced\042 or a dict"
-
-    if isinstance(class_weight, str) and class_weight.lower() == "balanced":
-        classes = np.unique(y).tolist()
-        cw = compute_class_weight("balanced", classes=classes, y=y)
-        trans_func = lambda s: cw[classes.index(s)]
-    else:
-        trans_func = lambda s: class_weight[s]
-    sample_weight = np.vectorize(trans_func)(sample_weight)
-    sample_weight = sample_weight / np.max(sample_weight)
-    return sample_weight
-
-
 def plot_single_lead(
     t: np.ndarray,
     sig: np.ndarray,
@@ -561,7 +466,7 @@ def init_logger(
     mode: str = "a",
     verbose: int = 0,
 ) -> logging.Logger:
-    """finished, checked,
+    """
 
     Parameters
     ----------
@@ -623,7 +528,7 @@ def init_logger(
 
 
 def get_date_str(fmt: Optional[str] = None):
-    """finished, checked,
+    """
 
     Parameters
     ----------
@@ -641,379 +546,22 @@ def get_date_str(fmt: Optional[str] = None):
     return date_str
 
 
-def rdheader(header_data: Union[str, Sequence[str]]) -> Union[Record, MultiRecord]:
-    """finished, checked,
-
-    modified from `wfdb.rdheader`
-
-    Parameters
-    ----------
-    head_data: str, or sequence of str,
-        path of the .hea header file, or lines of the .hea header file
-
+def list_sum(lst: Sequence[list]) -> list:
     """
-    if isinstance(header_data, str):
-        if not header_data.endswith(".hea"):
-            _header_data = header_data + ".hea"
-        else:
-            _header_data = header_data
-        if not os.path.isfile(_header_data):
-            raise FileNotFoundError
-        with open(_header_data, "r") as f:
-            _header_data = f.read().splitlines()
-    elif isinstance(header_data, Sequence):
-        _header_data = header_data
-    else:
-        raise TypeError(
-            f"header_data must be str or sequence of str, but got {type(header_data)}"
-        )
-    # Read the header file. Separate comment and non-comment lines
-    header_lines, comment_lines = [], []
-    for line in _header_data:
-        striped_line = line.strip()
-        # Comment line
-        if striped_line.startswith("#"):
-            comment_lines.append(striped_line)
-        # Non-empty non-comment line = header line.
-        elif striped_line:
-            # Look for a comment in the line
-            ci = striped_line.find("#")
-            if ci > 0:
-                header_lines.append(striped_line[:ci])
-                # comment on same line as header line
-                comment_lines.append(striped_line[ci:])
-            else:
-                header_lines.append(striped_line)
-
-    # Get fields from record line
-    record_fields = _header._parse_record_line(header_lines[0])
-
-    # Single segment header - Process signal specification lines
-    if record_fields["n_seg"] is None:
-        # Create a single-segment WFDB record object
-        record = Record()
-
-        # There are signals
-        if len(header_lines) > 1:
-            # Read the fields from the signal lines
-            signal_fields = _header._parse_signal_lines(header_lines[1:])
-            # Set the object's signal fields
-            for field in signal_fields:
-                setattr(record, field, signal_fields[field])
-
-        # Set the object's record line fields
-        for field in record_fields:
-            if field == "n_seg":
-                continue
-            setattr(record, field, record_fields[field])
-    # Multi segment header - Process segment specification lines
-    else:
-        # Create a multi-segment WFDB record object
-        record = MultiRecord()
-        # Read the fields from the segment lines
-        segment_fields = _header._read_segment_lines(header_lines[1:])
-        # Set the object's segment fields
-        for field in segment_fields:
-            setattr(record, field, segment_fields[field])
-        # Set the objects' record fields
-        for field in record_fields:
-            setattr(record, field, record_fields[field])
-
-        # Determine whether the record is fixed or variable
-        if record.seg_len[0] == 0:
-            record.layout = "variable"
-        else:
-            record.layout = "fixed"
-
-    # Set the comments field
-    record.comments = [line.strip(" \t#") for line in comment_lines]
-
-    return record
-
-
-def ensure_lead_fmt(
-    values: Sequence[Real], n_leads: int = 12, fmt: str = "lead_first"
-) -> np.ndarray:
-    """finished, checked,
-
-    ensure the `n_leads`-lead (ECG) signal to be of the format of `fmt`
 
     Parameters
     ----------
-    values: sequence,
-        values of the `n_leads`-lead (ECG) signal
-    n_leads: int, default 12,
-        number of leads
-    fmt: str, default "lead_first", case insensitive,
-        format of the output values, can be one of
-        "lead_first" (alias "channel_first"), "lead_last" (alias "channel_last")
-
-    Returns
-    -------
-    out_values: ndarray,
-        ECG signal in the format of `fmt`
-
-    """
-    out_values = np.array(values)
-    lead_dim = np.where(np.array(out_values.shape) == n_leads)[0]
-    if not any([[0] == lead_dim or [1] == lead_dim]):
-        raise ValueError(f"not valid {n_leads}-lead signal")
-    lead_dim = lead_dim[0]
-    if (lead_dim == 1 and fmt.lower() in ["lead_first", "channel_first"]) or (
-        lead_dim == 0 and fmt.lower() in ["lead_last", "channel_last"]
-    ):
-        out_values = out_values.T
-        return out_values
-    return out_values
-
-
-def ensure_siglen(
-    values: Sequence[Real],
-    siglen: int,
-    fmt: str = "lead_first",
-    tolerance: Optional[float] = None,
-) -> np.ndarray:
-    """finished, checked,
-
-    ensure the (ECG) signal to be of length `siglen`,
-    strategy:
-        If `values` has length greater than `siglen`,
-        the central `siglen` samples will be adopted;
-        otherwise, zero padding will be added to both sides.
-        If `tolerance` is given,
-        then if the length of `values` is longer than `siglen` by more than `tolerance`,
-        the `values` will be sliced to have multiple of `siglen` samples.
-
-    Parameters
-    ----------
-    values: sequence,
-        values of the `n_leads`-lead (ECG) signal
-    siglen: int,
-        length of the signal supposed to have
-    fmt: str, default "lead_first", case insensitive,
-        format of the input and output values, can be one of
-        "lead_first" (alias "channel_first"), "lead_last" (alias "channel_last")
-
-    Returns
-    -------
-    out_values: ndarray,
-        ECG signal in the format of `fmt` and of fixed length `siglen`,
-        of ndim=3 if `tolerence` is given, otherwise ndim=2
-
-    """
-    if fmt.lower() in ["channel_last", "lead_last"]:
-        _values = np.array(values).T
-    else:
-        _values = np.array(values).copy()
-    original_siglen = _values.shape[1]
-    n_leads = _values.shape[0]
-
-    if tolerance is None or original_siglen <= siglen * (1 + tolerance):
-        if original_siglen >= siglen:
-            start = (original_siglen - siglen) // 2
-            end = start + siglen
-            out_values = _values[..., start:end]
-        else:
-            pad_len = siglen - original_siglen
-            pad_left = pad_len // 2
-            pad_right = pad_len - pad_left
-            out_values = np.concatenate(
-                [
-                    np.zeros((n_leads, pad_left)),
-                    _values,
-                    np.zeros((n_leads, pad_right)),
-                ],
-                axis=1,
-            )
-
-        if fmt.lower() in ["channel_last", "lead_last"]:
-            out_values = out_values.T
-        if tolerance is not None:
-            out_values = out_values[np.newaxis, ...]
-
-        return out_values
-
-    forward_len = int(round(siglen * tolerance))
-    out_values = np.array(
-        [
-            _values[..., idx * forward_len : idx * forward_len + siglen]
-            for idx in range((original_siglen - siglen) // forward_len + 1)
-        ]
-    )
-    if fmt.lower() in ["channel_last", "lead_last"]:
-        out_values = np.moveaxis(out_values, 1, -1)
-    return out_values
-
-
-ECGWaveForm = namedtuple(
-    typename="ECGWaveForm",
-    field_names=["name", "onset", "offset", "peak", "duration"],
-)
-
-
-def masks_to_waveforms(
-    masks: np.ndarray,
-    class_map: Dict[str, int],
-    fs: Real,
-    mask_format: str = "channel_first",
-    leads: Optional[Sequence[str]] = None,
-) -> Dict[str, List[ECGWaveForm]]:
-    """finished, checked,
-
-    convert masks into lists of waveforms
-
-    Parameters
-    ----------
-    masks: ndarray,
-        wave delineation in the form of masks,
-        of shape (n_leads, seq_len), or (seq_len,)
-    class_map: dict,
-        class map, mapping names to waves to numbers from 0 to n_classes-1,
-        the keys should contain "pwave", "qrs", "twave"
-    fs: real number,
-        sampling frequency of the signal corresponding to the `masks`,
-        used to compute the duration of each waveform
-    mask_format: str, default "channel_first",
-        format of the mask, used only when `masks.ndim = 2`
-        "channel_last" (alias "lead_last"), or
-        "channel_first" (alias "lead_first")
-    leads: str or list of str, optional,
-        the names of leads corresponding to the channels of the `masks`
-
-    Returns
-    -------
-    waves: dict,
-        each item value is a list containing the `ECGWaveForm`s corr. to the lead;
-        each item key is from `leads` if `leads` is set,
-        otherwise would be "lead_1", "lead_2", ..., "lead_n"
-
-    """
-    if masks.ndim == 1:
-        _masks = masks[np.newaxis, ...]
-    elif masks.ndim == 2:
-        if mask_format.lower() not in [
-            "channel_first",
-            "lead_first",
-        ]:
-            _masks = masks.T
-        else:
-            _masks = masks.copy()
-    else:
-        raise ValueError(
-            f"masks should be of dim 1 or 2, but got a {masks.ndim}d array"
-        )
-
-    _leads = (
-        [f"lead_{idx+1}" for idx in range(_masks.shape[0])] if leads is None else leads
-    )
-    assert len(_leads) == _masks.shape[0]
-
-    _class_map = CFG(deepcopy(class_map))
-
-    waves = CFG({lead_name: [] for lead_name in _leads})
-    for channel_idx, lead_name in enumerate(_leads):
-        current_mask = _masks[channel_idx, ...]
-        for wave_name, wave_number in _class_map.items():
-            if wave_name.lower() not in [
-                "pwave",
-                "qrs",
-                "twave",
-            ]:
-                continue
-            current_wave_inds = np.where(current_mask == wave_number)[0]
-            if len(current_wave_inds) == 0:
-                continue
-            np.where(np.diff(current_wave_inds) > 1)
-            split_inds = np.where(np.diff(current_wave_inds) > 1)[0].tolist()
-            split_inds = sorted(split_inds + [i + 1 for i in split_inds])
-            split_inds = [0] + split_inds + [len(current_wave_inds) - 1]
-            for i in range(len(split_inds) // 2):
-                itv_start = current_wave_inds[split_inds[2 * i]]
-                itv_end = current_wave_inds[split_inds[2 * i + 1]] + 1
-                w = ECGWaveForm(
-                    name=wave_name.lower(),
-                    onset=itv_start,
-                    offset=itv_end,
-                    peak=np.nan,
-                    duration=1000 * (itv_end - itv_start) / fs,  # ms
-                )
-                waves[lead_name].append(w)
-        waves[lead_name].sort(key=lambda w: w.onset)
-    return waves
-
-
-def mask_to_intervals(
-    mask: np.ndarray,
-    vals: Optional[Union[int, Sequence[int]]] = None,
-    right_inclusive: bool = False,
-) -> Union[list, dict]:
-    """finished, checked,
-
-    Parameters
-    ----------
-    mask: ndarray,
-        1d mask
-    vals: int or sequence of int, optional,
-        values in `mask` to obtain intervals
-    right_inclusive: bool, default False,
-        if True, the intervals will be right inclusive
-        otherwise, right exclusive
-
-    Returns
-    -------
-    intervals: dict or list,
-        the intervals corr. to each value in `vals` if `vals` is `None` or `Sequence`;
-        or the intervals corr. to `vals` if `vals` is int.
-        each interval is of the form `[a,b]`
-
-    """
-    if vals is None:
-        _vals = list(set(mask))
-    elif isinstance(vals, int):
-        _vals = [vals]
-    else:
-        _vals = vals
-    # assert set(_vals) & set(mask) == set(_vals)
-    bias = 0 if right_inclusive else 1
-
-    intervals = {v: [] for v in _vals}
-    for v in _vals:
-        valid_inds = np.where(np.array(mask) == v)[0]
-        if len(valid_inds) == 0:
-            continue
-        split_indices = np.where(np.diff(valid_inds) > 1)[0]
-        split_indices = split_indices.tolist() + (split_indices + 1).tolist()
-        split_indices = sorted([0] + split_indices + [len(valid_inds) - 1])
-        for idx in range(len(split_indices) // 2):
-            intervals[v].append(
-                [
-                    valid_inds[split_indices[2 * idx]],
-                    valid_inds[split_indices[2 * idx + 1]] + bias,
-                ]
-            )
-
-    if isinstance(vals, int):
-        intervals = intervals[vals]
-
-    return intervals
-
-
-def list_sum(l: Sequence[list]) -> list:
-    """finished, checked,
-
-    Parameters
-    ----------
-    l: sequence of list,
+    lst: sequence of list,
         the sequence of lists to obtain the summation
 
     Returns
     -------
     l_sum: list,
-        sum of `l`,
-        i.e. if l = [list1, list2, ...], then l_sum = list1 + list2 + ...
+        sum of `lst`,
+        i.e. if lst = [list1, list2, ...], then l_sum = list1 + list2 + ...
 
     """
-    l_sum = reduce(lambda a, b: a + b, l, [])
+    l_sum = reduce(lambda a, b: a + b, lst, [])
     return l_sum
 
 
@@ -1022,7 +570,7 @@ def read_log_txt(
     epoch_startswith: str = "Train epoch_",
     scalar_startswith: Union[str, Iterable[str]] = "train/|test/",
 ) -> pd.DataFrame:
-    """finished, checked,
+    """
 
     read from log txt file, in case tensorboard not working
 
@@ -1050,14 +598,14 @@ def read_log_txt(
         field_pattern = f"""^({"|".join(scalar_startswith)})"""
     summary = []
     new_line = None
-    for l in content:
-        if l.startswith(epoch_startswith):
+    for line in content:
+        if line.startswith(epoch_startswith):
             if new_line:
                 summary.append(new_line)
-            epoch = re.findall("[\d]+", l)[0]
+            epoch = re.findall("[\\d]+", line)[0]
             new_line = {"epoch": epoch}
-        if re.findall(field_pattern, l):
-            field, val = l.split(":")
+        if re.findall(field_pattern, line):
+            field, val = line.split(":")
             field = field.strip()
             val = float(val.strip())
             new_line[field] = val
@@ -1069,7 +617,7 @@ def read_log_txt(
 def read_event_scalars(
     fp: str, keys: Optional[Union[str, Iterable[str]]] = None
 ) -> Union[pd.DataFrame, Dict[str, pd.DataFrame]]:
-    """finished, checked,
+    """
 
     read scalars from event file, in case tensorboard not working
 
@@ -1089,7 +637,7 @@ def read_event_scalars(
     """
     try:
         from tensorflow.python.summary.event_accumulator import EventAccumulator
-    except:
+    except Exception:
         from tensorboard.backend.event_processing.event_accumulator import (
             EventAccumulator,
         )
@@ -1118,7 +666,7 @@ def read_event_scalars(
 
 
 def dicts_equal(d1: dict, d2: dict) -> bool:
-    """finished, checked,
+    """
 
     Parameters
     ----------
@@ -1184,7 +732,7 @@ def dicts_equal(d1: dict, d2: dict) -> bool:
 
 
 def default_class_repr(c: object, align: str = "center", depth: int = 1) -> str:
-    """finished, checked,
+    """
 
     Parameters
     ----------
@@ -1240,7 +788,7 @@ class ReprMixin(object):
 
 
 class MovingAverage(object):
-    """finished, checked, to be improved,
+    """to be improved,
 
     moving average
 
@@ -1384,30 +932,8 @@ class MovingAverage(object):
         return smoothed
 
 
-def uniform(low: Real, high: Real, num: int) -> List[float]:
-    """finished, checked,
-
-    Parameters
-    ----------
-    low: real number,
-        lower bound of the interval of the uniform distribution
-    high: real number,
-        upper bound of the interval of the uniform distribution
-    num: int,
-        number of random numbers to generate
-
-    Returns
-    -------
-    arr: list of float,
-        array of randomly generated numbers with uniform distribution
-
-    """
-    arr = [random.uniform(low, high) for _ in range(num)]
-    return arr
-
-
 def nildent(text: str) -> str:
-    """finished, checked,
+    """
 
     kill all leading white spaces in each line of `text`,
     while keeping all lines (including empty)
@@ -1423,14 +949,14 @@ def nildent(text: str) -> str:
         processed text
 
     """
-    new_text = "\n".join([l.lstrip() for l in text.splitlines()]) + (
+    new_text = "\n".join([line.lstrip() for line in text.splitlines()]) + (
         "\n" if text.endswith("\n") else ""
     )
     return new_text
 
 
 def isclass(obj: Any) -> bool:
-    """finished, checked,
+    """
 
     Parameters
     ----------
@@ -1449,65 +975,232 @@ def isclass(obj: Any) -> bool:
         return False
 
 
-def strafified_train_test_split(
-    df: pd.DataFrame, strafified_cols: Sequence[str], test_ratio: float = 0.2
-) -> Tuple[pd.DataFrame, pd.DataFrame]:
+def add_docstring(doc: str, mode: str = "replace") -> Callable:
     """
-    Do stratified train-test split on the dataframe,
+    decorator to add docstring to a function
 
     Parameters
     ----------
-    df: pd.DataFrame,
-        dataframe to be split
-    strafified_cols: sequence of str,
-        columns to be strafified, assuming each column is a categorical variable
-        each class in any of the columns will be
-        split into train and test sets with an approximate ratio of `test_ratio`
-    test_ratio: float, default 0.2,
-        ratio of test set to the whole dataframe
-
-    Returns
-    -------
-    df_train: pd.DataFrame,
-        the dataframe of the train set
-    df_test: pd.DataFrame,
-        the dataframe of the test set
-
-    For example,
-    if one has a dataframe with columns `sex`, `nationality`, etc.,
-    assuming `sex` includes `male`, `female`; `nationality` includes `Chinese`, `American`,
-    and sets `strafified_cols = ["sex", "nationality"]` with `test_ratio = 0.2`,
-    then approximately 20% of the male and 20% of the female subjects
-    will be put into the test set,
-    and **at the same time**, approximately 20% of the Chinese and 20% of the Americans
-    lie in the test set as well.
+    doc: str,
+        the docstring to be added
+    mode: str, default "replace",
+        the mode of the docstring,
+        can be "replace", "append" or "prepend",
+        case insensitive
 
     """
-    df_inspection = df[strafified_cols].copy()
-    for item in strafified_cols:
-        all_entities = df_inspection[item].unique().tolist()
-        entities_dict = {e: str(i) for i, e in enumerate(all_entities)}
-        df_inspection[item] = df_inspection[item].apply(lambda e: entities_dict[e])
 
-    inspection_col_name = "Inspection" * (
-        max([len(c) for c in strafified_cols]) // 10 + 1
-    )
-    df_inspection[inspection_col_name] = ""
-    for idx, row in df_inspection.iterrows():
-        cn = "-".join([row[sc] for sc in strafified_cols])
-        df_inspection.loc[idx, inspection_col_name] = cn
-    item_names = df_inspection[inspection_col_name].unique().tolist()
-    item_indices = {
-        n: df_inspection.index[df_inspection[inspection_col_name] == n].tolist()
-        for n in item_names
-    }
-    for n in item_names:
-        random.shuffle(item_indices[n])
+    def decorator(func: Callable) -> Callable:
+        """ """
 
-    test_indices = []
-    for n in item_names:
-        item_test_indices = item_indices[n][: round(test_ratio * len(item_indices[n]))]
-        test_indices += item_test_indices
-    df_test = df.loc[df.index.isin(test_indices)].reset_index(drop=True)
-    df_train = df.loc[~df.index.isin(test_indices)].reset_index(drop=True)
-    return df_train, df_test
+        @wraps(func)
+        def wrapper(*args, **kwargs) -> Callable:
+            """ """
+            return func(*args, **kwargs)
+
+        pattern = "(\\s^\n){1,}"
+        if mode.lower() == "replace":
+            wrapper.__doc__ = doc
+        elif mode.lower() == "append":
+            tmp = re.sub(pattern, "", wrapper.__doc__)
+            new_lines = 1 - (len(tmp) - len(tmp.rstrip("\n")))
+            tmp = re.sub(pattern, "", doc)
+            new_lines -= len(tmp) - len(tmp.lstrip("\n"))
+            new_lines = max(0, new_lines) * "\n"
+            wrapper.__doc__ += new_lines + doc
+        elif mode.lower() == "prepend":
+            tmp = re.sub(pattern, "", doc)
+            new_lines = 1 - (len(tmp) - len(tmp.rstrip("\n")))
+            tmp = re.sub(pattern, "", wrapper.__doc__)
+            new_lines -= len(tmp) - len(tmp.lstrip("\n"))
+            new_lines = max(0, new_lines) * "\n"
+            wrapper.__doc__ = doc + new_lines + wrapper.__doc__
+        else:
+            raise ValueError(f"mode {mode} is not supported")
+        return wrapper
+
+    return decorator
+
+
+def deprecate_kwargs(l_kwargs: Sequence[Sequence[str]]):
+    """
+
+    decorator to deprecate old kwargs in a function
+
+    Parameters
+    ----------
+    l_kwargs: Sequence[Sequence[str]],
+        a list of kwargs to be deprecated,
+        each element is a sequence of length 2,
+        of the form (new_kwarg, old_kwarg)
+
+    """
+
+    def decorator(func: Callable) -> Callable:
+        """ """
+
+        @wraps(func)
+        def wrapper(*args, **kwargs) -> Callable:
+            """ """
+            old_kwargs = deepcopy(kwargs)
+            for new_kw, old_kw in l_kwargs:
+                if new_kw in kwargs:
+                    old_kwargs.pop(new_kw, None)
+                    old_kwargs[old_kw] = kwargs[new_kw]
+                elif old_kw in kwargs:
+                    warnings.warn(
+                        f"key word argument \042{old_kw}\042 is deprecated, use \042{new_kw}\042 instead"
+                    )
+            return func(*args, **old_kwargs)
+
+        func_params = list(inspect.signature(func).parameters.values())
+        func_param_names = list(inspect.signature(func).parameters.keys())
+        for new_kw, old_kw in l_kwargs:
+            idx = func_param_names.index(old_kw)
+            func_params[idx] = func_params[idx].replace(name=new_kw)
+            wrapper.__doc__ = func.__doc__.replace(old_kw, new_kw)
+        wrapper.__signature__ = inspect.Signature(parameters=func_params)
+        return wrapper
+
+    return decorator
+
+
+@contextmanager
+def timeout(duration: float):
+    """
+    A context manager that raises a `TimeoutError` after a specified time.
+
+    Parameters
+    ----------
+    duration: float,
+        the time duration in seconds,
+        should be non-negative,
+        0 for no timeout
+
+    References
+    ----------
+    https://stackoverflow.com/questions/492519/timeout-on-a-function-call
+
+    """
+    if np.isinf(duration):
+        duration = 0
+    elif duration < 0:
+        raise ValueError("duration must be non-negative")
+    elif duration > 0:  # granularity is 1 second, so round up
+        duration = max(1, int(duration))
+
+    def timeout_handler(signum, frame):
+        raise TimeoutError(f"block timedout after {duration} seconds")
+
+    signal.signal(signal.SIGALRM, timeout_handler)
+    signal.alarm(duration)
+    yield
+    signal.alarm(0)
+
+
+class Timer(ReprMixin):
+    """
+
+    Context manager to time the execution of a block of code.
+
+    Usage
+    -----
+    >>> with Timer("task name", verbose=2) as timer:
+    >>>     do_something()
+    >>>     timer.add_time("subtask 1", level=2)
+    >>>     do_subtask_1()
+    >>>     timer.stop_timer("subtask 1")
+    >>>     timer.add_time("subtask 2", level=2)
+    >>>     do_subtask_2()
+    >>>     timer.stop_timer("subtask 2")
+    >>>     do_something_else()
+
+    """
+
+    __name__ = "Timer"
+
+    def __init__(self, name: Optional[str] = None, verbose: int = 0) -> NoReturn:
+        """
+
+        Parameters
+        ----------
+        name: str, optional
+            the name of the timer, defaults to "default timer"
+        verbose: int, default 0
+            the verbosity level of the timer,
+
+        """
+        self.name = name or "default timer"
+        self.verbose = verbose
+        self.timers = {self.name: 0.0}
+        self.ends = {self.name: 0.0}
+        self.levels = {self.name: 1}
+
+    def __enter__(self) -> "Timer":
+        self.timers = {self.name: time.perf_counter()}
+        self.ends = {self.name: 0.0}
+        self.levels = {self.name: 1}
+        return self
+
+    def __exit__(self, *args) -> NoReturn:
+        for k in self.timers:
+            self.stop_timer(k)
+            self.timers[k] = self.ends[k] - self.timers[k]
+
+    def add_timer(self, name: str, level: int = 1) -> NoReturn:
+        """
+        add a new timer for some subtask
+
+        Parameters
+        ----------
+        name: str,
+            the name of the timer to be added
+        level: int, default 1
+            the verbosity level of the timer,
+
+        """
+        self.timers[name] = time.perf_counter()
+        self.ends[name] = 0
+        self.levels[name] = level
+
+    def stop_timer(self, name: str) -> NoReturn:
+        """
+        stop a timer
+
+        Parameters
+        ----------
+        name: str,
+            the name of the timer to be stopped
+
+        """
+        if self.ends[name] == 0:
+            self.ends[name] = time.perf_counter()
+            if self.verbose >= self.levels[name]:
+                time_cost, unit = self._simplify_time_expr(
+                    self.ends[name] - self.timers[name]
+                )
+                print(f"{name} took {time_cost:.4f} {unit}")
+
+    def _simplify_time_expr(self, time_cost: float) -> Tuple[float, str]:
+        """
+        simplify the time expression
+
+        Parameters
+        ----------
+        time_cost: float,
+            the time cost, with units in seconds
+
+        Returns
+        -------
+        time_cost: float,
+            the time cost,
+        unit: str,
+            the unit of the time cost
+
+        """
+        if time_cost <= 0.1:
+            return 1000 * time_cost, "ms"
+        return time_cost, "s"
+
+    def extra_repr_keys(self) -> List[str]:
+        return ["name", "verbose"]
