@@ -13,13 +13,11 @@ TODO:
 """
 import time
 from copy import deepcopy
-from functools import reduce
 from numbers import Real
-from typing import Union, Optional, Any, List, Tuple, Sequence
+from typing import Any, List, Optional, Sequence, Tuple, Union
 
 import numpy as np
 
-
 __all__ = [
     "get_optimal_covering",
     "overlaps",
@@ -35,11 +33,9 @@ __all__ = [
     "find_max_cont_len",
     "interval_len",
     "generalized_interval_len",
-    "diff_with_step",
     "find_extrema",
     "is_intersect",
     "max_disjoint_covering",
-    "mask_to_intervals",
 ]
 
 
@@ -49,7 +45,7 @@ GeneralizedInterval = Union[Sequence[Interval], type(EMPTY_SET)]
 
 
 def overlaps(interval: Interval, another: Interval) -> int:
-    """finished, checked,
+    """
 
     Return the amount of overlap, in bp between interval and anohter.
     If >0, the number of bp of overlap
@@ -109,7 +105,7 @@ def validate_interval(
 def in_interval(
     val: Real, interval: Interval, left_closed: bool = True, right_closed: bool = False
 ) -> bool:
-    """finished, checked,
+    """
 
     check whether val is inside interval or not
 
@@ -143,7 +139,7 @@ def in_generalized_interval(
     left_closed: bool = True,
     right_closed: bool = False,
 ) -> bool:
-    """finished, checked,
+    """
 
     check whether val is inside generalized_interval or not
 
@@ -174,7 +170,7 @@ def get_confidence_interval(
     confidence: float = 0.95,
     **kwargs: Any,
 ) -> np.ndarray:
-    """finished, checked,
+    """
 
     Parameters
     ----------
@@ -209,7 +205,7 @@ def get_confidence_interval(
 def intervals_union(
     interval_list: GeneralizedInterval, join_book_endeds: bool = True
 ) -> GeneralizedInterval:
-    """finished, checked,
+    """
 
     find the union (ordered and non-intersecting) of all the intervals in `interval_list`,
     which is a list of intervals in the form [a,b], where a,b need not be ordered
@@ -227,12 +223,11 @@ def intervals_union(
         the union of the intervals in `interval_list`
 
     """
-    interval_sort_key = lambda i: i[0]
     # list_add = lambda list1, list2: list1+list2
     processed = [item for item in interval_list if len(item) > 0]
     for item in processed:
         item.sort()
-    processed.sort(key=interval_sort_key)
+    processed.sort(key=lambda i: i[0])
     # end_points = reduce(list_add, processed)
     merge_flag = True
     while merge_flag:
@@ -279,7 +274,7 @@ def generalized_intervals_union(
     interval_list: Union[List[GeneralizedInterval], Tuple[GeneralizedInterval]],
     join_book_endeds: bool = True,
 ) -> GeneralizedInterval:
-    """finished, checked,
+    """
 
     calculate the union of a list (or tuple) of `GeneralizedInterval`s
 
@@ -304,7 +299,7 @@ def generalized_intervals_union(
 def intervals_intersection(
     interval_list: GeneralizedInterval, drop_degenerate: bool = True
 ) -> Interval:
-    """finished, checked,
+    """
 
     calculate the intersection of all intervals in interval_list
 
@@ -341,7 +336,7 @@ def generalized_intervals_intersection(
     another_generalized_interval: GeneralizedInterval,
     drop_degenerate: bool = True,
 ) -> GeneralizedInterval:
-    """finished, checked,
+    """
 
     calculate the intersection of generalized_interval and another_generalized_interval,
     which are both generalized intervals
@@ -384,7 +379,7 @@ def generalized_intervals_intersection(
 def generalized_interval_complement(
     total_interval: Interval, generalized_interval: GeneralizedInterval
 ) -> GeneralizedInterval:
-    """finished, checked, to be improved,
+    """to be improved,
 
     TODO: the case `total_interval` is a `GeneralizedInterval`
 
@@ -426,7 +421,7 @@ def get_optimal_covering(
     traceback: bool = False,
     **kwargs: Any,
 ) -> Tuple[GeneralizedInterval, list]:
-    """finished, checked,
+    """
 
     compute an optimal covering (disjoint union of intervals) that covers `to_cover` such that
     each interval in the covering is of length at least `min_len`,
@@ -498,8 +493,7 @@ def get_optimal_covering(
     for interval in to_cover_intervals:
         interval.sort()
 
-    interval_sort_key = lambda i: i[0]
-    to_cover_intervals.sort(key=interval_sort_key)
+    to_cover_intervals.sort(key=lambda i: i[0])
 
     if verbose >= 2:
         print(f"to_cover_intervals after sorted = {to_cover_intervals}")
@@ -622,7 +616,7 @@ def get_optimal_covering(
 
 
 def find_max_cont_len(sublist: Interval, tot_rng: Real) -> dict:
-    """finished, checked,
+    """
 
     find the maximum length of continuous (consecutive) sublists of `sublist`,
     whose element are integers within the range from 0 to `tot_rng`,
@@ -664,7 +658,7 @@ def find_max_cont_len(sublist: Interval, tot_rng: Real) -> dict:
 
 
 def interval_len(interval: Interval) -> Real:
-    """finished, checked,
+    """
 
     compute the length of an interval. 0 for the empty interval []
 
@@ -684,7 +678,7 @@ def interval_len(interval: Interval) -> Real:
 
 
 def generalized_interval_len(generalized_interval: GeneralizedInterval) -> Real:
-    """finished, checked,
+    """
 
     compute the length of a generalized interval. 0 for the empty interval []
 
@@ -702,34 +696,6 @@ def generalized_interval_len(generalized_interval: GeneralizedInterval) -> Real:
     return gi_len
 
 
-def diff_with_step(a: Sequence, step: int = 1, **kwargs) -> np.ndarray:
-    """finished, checked,
-
-    compute a[n+step] - a[n] for all valid n
-
-    Parameters
-    ----------
-    a: array_like,
-        the input data
-    step: int, default 1,
-        the step to compute the difference
-    kwargs: dict,
-
-    Returns
-    -------
-    d: ndarray:
-        the difference array
-
-    """
-    _a = np.array(a)
-    if step >= len(_a):
-        raise ValueError(
-            f"step ({step}) should be less than the length ({len(_a)}) of `a`"
-        )
-    d = _a[step:] - _a[:-step]
-    return d
-
-
 def find_extrema(signal: Optional[Sequence] = None, mode: str = "both") -> np.ndarray:
     """
     Locate local extrema points in a signal. Based on Fermat's Theorem
@@ -813,7 +779,7 @@ def max_disjoint_covering(
     with_traceback: bool = True,
     verbose: int = 0,
 ) -> Tuple[GeneralizedInterval, List[int]]:
-    """finished, checked,
+    """
 
     find the largest (the largest interval length) covering of a sequence of intervals
 
@@ -918,59 +884,3 @@ def max_disjoint_covering(
     else:
         covering_inds = []
     return covering, covering_inds
-
-
-def mask_to_intervals(
-    mask: np.ndarray,
-    vals: Optional[Union[int, Sequence[int]]] = None,
-    right_inclusive: bool = False,
-) -> Union[list, dict]:
-    """finished, checked,
-
-    Parameters
-    ----------
-    mask: ndarray,
-        1d mask
-    vals: int or sequence of int, optional,
-        values in `mask` to obtain intervals
-    right_inclusive: bool, default False,
-        if True, the intervals will be right inclusive
-        otherwise, right exclusive
-
-    Returns
-    -------
-    intervals: dict or list,
-        the intervals corr. to each value in `vals` if `vals` is `None` or `Sequence`;
-        or the intervals corr. to `vals` if `vals` is int.
-        each interval is of the form `[a,b]`
-
-    """
-    if vals is None:
-        _vals = list(set(mask))
-    elif isinstance(vals, int):
-        _vals = [vals]
-    else:
-        _vals = vals
-    # assert set(_vals) & set(mask) == set(_vals)
-    bias = 0 if right_inclusive else 1
-
-    intervals = {v: [] for v in _vals}
-    for v in _vals:
-        valid_inds = np.where(np.array(mask) == v)[0]
-        if len(valid_inds) == 0:
-            continue
-        split_indices = np.where(np.diff(valid_inds) > 1)[0]
-        split_indices = split_indices.tolist() + (split_indices + 1).tolist()
-        split_indices = sorted([0] + split_indices + [len(valid_inds) - 1])
-        for idx in range(len(split_indices) // 2):
-            intervals[v].append(
-                [
-                    valid_inds[split_indices[2 * idx]],
-                    valid_inds[split_indices[2 * idx + 1]] + bias,
-                ]
-            )
-
-    if isinstance(vals, int):
-        intervals = intervals[vals]
-
-    return intervals
