@@ -4,20 +4,18 @@ utilities for nn models
 """
 
 import re
+from copy import deepcopy
 from itertools import repeat
 from math import floor
-from copy import deepcopy
-from typing import Union, Sequence, List, Tuple, Optional, NoReturn
 from numbers import Real
+from typing import List, Optional, Sequence, Tuple, Union
 
 import numpy as np
 import torch
-from torch import Tensor
-from torch import nn
+from torch import Tensor, nn
 
 from ..cfg import DEFAULTS
 
-
 if DEFAULTS.torch_dtype == torch.float64:
     torch.set_default_tensor_type(torch.DoubleTensor)
 
@@ -29,6 +27,8 @@ __all__ = [
     "compute_deconv_output_shape",
     "compute_maxpool_output_shape",
     "compute_avgpool_output_shape",
+    "compute_sequential_output_shape",
+    "compute_sequential_output_shape_docstring",
     "compute_module_size",
     "default_collate_fn",
     "compute_receptive_field",
@@ -41,7 +41,7 @@ __all__ = [
 def extend_predictions(
     preds: Sequence, classes: List[str], extended_classes: List[str]
 ) -> np.ndarray:
-    """finished, checked,
+    """
 
     extend the prediction arrays to prediction arrays in larger range of classes
 
@@ -95,7 +95,7 @@ def compute_output_shape(
     dilation: Union[Sequence[int], int] = 1,
     channel_last: bool = False,
 ) -> Tuple[Union[int, None]]:
-    """finished, checked,
+    """
 
     compute the output shape of a (transpose) convolution/maxpool/avgpool layer
 
@@ -154,13 +154,18 @@ def compute_output_shape(
         "convolution",
     ]:
         # as function of dilation, kernel_size
-        minus_term = lambda d, k: d * (k - 1) + 1
+        def minus_term(d, k):
+            return d * (k - 1) + 1
+
         out_channels = num_filters
     elif lt in [
         "maxpool",
         "maxpooling",
     ]:
-        minus_term = lambda d, k: d * (k - 1) + 1
+
+        def minus_term(d, k):
+            return d * (k - 1) + 1
+
         out_channels = input_shape[-1] if channel_last else input_shape[1]
     elif lt in [
         "avgpool",
@@ -168,7 +173,10 @@ def compute_output_shape(
         "averagepool",
         "averagepooling",
     ]:
-        minus_term = lambda d, k: k
+
+        def minus_term(d, k):
+            return k
+
         out_channels = input_shape[-1] if channel_last else input_shape[1]
     elif lt in [
         "deconv",
@@ -449,7 +457,7 @@ def compute_deconv_output_shape(
     dilation: Union[Sequence[int], int] = 1,
     channel_last: bool = False,
 ) -> Tuple[Union[int, None]]:
-    """finished, checked,
+    """
 
     compute the output shape of a transpose convolution layer
 
@@ -495,10 +503,61 @@ def compute_deconv_output_shape(
     return output_shape
 
 
+compute_sequential_output_shape_docstring = """
+
+    Parameters
+    ----------
+    seq_len: int,
+        length of the 1d sequence
+    batch_size: int, optional,
+        the batch size, can be None
+
+    Returns
+    -------
+    output_shape: sequence,
+        the output shape, given `seq_len` and `batch_size`
+
+    """
+
+
+def compute_sequential_output_shape(
+    model: nn.Sequential,
+    seq_len: Optional[int] = None,
+    batch_size: Optional[int] = None,
+) -> Sequence[Union[int, None]]:
+    """
+
+    compute the output shape of a sequential model
+
+    Parameters
+    ----------
+    model: nn.Sequential,
+        the sequential model
+    seq_len: int,
+        length of the 1d sequence
+    batch_size: int, optional,
+        the batch size, can be None
+
+    Returns
+    -------
+    output_shape: sequence,
+        the output shape, given `seq_len` and `batch_size`
+
+    """
+    assert issubclass(
+        type(model), nn.Sequential
+    ), f"model should be nn.Sequential, but got {type(model)}"
+    _seq_len = seq_len
+    for module in model:
+        output_shape = module.compute_output_shape(_seq_len, batch_size)
+        _, _, _seq_len = output_shape
+    return output_shape
+
+
 def compute_module_size(
     module: nn.Module, human: bool = False, dtype: str = "float32"
 ) -> Union[int, str]:
-    """finished, checked,
+    """
 
     compute the size (number of parameters) of a module
 
@@ -540,7 +599,7 @@ def compute_receptive_field(
     dilations: Union[Sequence[int], int] = 1,
     input_len: Optional[int] = None,
 ) -> Union[int, float]:
-    r""" finished, checked,
+    r"""
 
     computes the (generic) receptive field of feature map of certain channel,
     from certain flow (if not merged, different flows, e.g. shortcut, must be computed separately),
@@ -617,7 +676,7 @@ def compute_receptive_field(
 
 
 def default_collate_fn(batch: Sequence[Tuple[np.ndarray, ...]]) -> Tuple[Tensor, ...]:
-    """finished, checked,
+    """
 
     collate functions for model training
 
@@ -637,7 +696,7 @@ def default_collate_fn(batch: Sequence[Tuple[np.ndarray, ...]]) -> Tuple[Tensor,
     """
     try:
         n_fields = len(batch[0])
-    except:
+    except Exception:
         raise ValueError("No data")
     ret = []
     for i in range(n_fields):
@@ -648,58 +707,15 @@ def default_collate_fn(batch: Sequence[Tuple[np.ndarray, ...]]) -> Tuple[Tensor,
     return tuple(ret)
 
 
-def intervals_iou(itv_a: Tensor, itv_b: Tensor, iou_type="iou") -> Tensor:
-    """NOT finished, NOT checked,
+if torch.__version__ >= "1.15.0":
 
-    1d analogue of the 2d bounding boxes IoU,
-    for 1d "object detection" models
+    def _true_divide(dividend, divisor):
+        return torch.true_divide(dividend, divisor)
 
-    Parameters
-    ----------
-    itv_a, itv_b: Tensor,
-        of shape (N, 2), (K, 2) resp.
-    iou_type: str, default "iou", case insensitive
-        type of IoU
+else:
 
-    """
-    raise NotImplementedError
-    left_intersect = torch.max(itv_a[:, np.newaxis, :1], itvb[..., :1]).squeeze(
-        -1
-    )  # shape (N,K)
-    right_intersect = torch.min(itv_a[:, np.newaxis, 1:], itvb[..., 1:]).squeeze(
-        -1
-    )  # shape (N,K)
-
-    left_union = torch.min(itv_a[:, np.newaxis, :1], itvb[..., :1]).squeeze(
-        -1
-    )  # shape (N,K)
-    right_union = torch.max(itv_a[:, np.newaxis, 1:], itvb[..., 1:]).squeeze(
-        -1
-    )  # shape (N,K)
-
-    en = (left_intersect < right_intersect).type(left_intersect.type())
-    len_intersect = (right_intersect - left_intersect) * en
-    len_union = right_union - left_union
-
-    iou = _true_divide(len_intersect, len_union)
-
-    if iou_type.lower() == "iou":
-        return iou
-
-    cen_a = torch.mean(itv_a, dim=-1, keepdim=True)  # shape (N,1,1)
-    cen_b = torch.mean(itv_b, dim=-1, keepdim=False)  # shape (K,1)
-    cen_dist = torch.abs(itv_a - itv_b).squeeze(-1)  # shape (N,K)
-
-    diou = iou - _true_divide(cen_dist, len_union)
-
-    if iou_type.lower() == "diou":
-        return diou
-
-    len_a = a[..., 1] - a[..., 0]
-    len_b = b[..., 1] - b[..., 0]
-
-    if iou_type.lower() == "ciou":
-        raise NotImplementedError
+    def _true_divide(dividend, divisor):
+        return dividend / divisor
 
 
 def _adjust_cnn_filter_lengths(
@@ -708,7 +724,7 @@ def _adjust_cnn_filter_lengths(
     ensure_odd: bool = True,
     pattern: str = "filter_length|filt_size",
 ) -> dict:
-    """finished, checked,
+    """
 
     adjust the filter lengths (kernel sizes) in the config for convolutional neural networks,
     according to the new sampling frequency
@@ -742,9 +758,9 @@ def _adjust_cnn_filter_lengths(
             if isinstance(v, (Sequence, np.ndarray)):  # DO NOT use `Iterable`
                 config[k] = [
                     _adjust_cnn_filter_lengths(
-                        {"filter_length": l, "fs": config["fs"]}, fs, ensure_odd
+                        {"filter_length": fl, "fs": config["fs"]}, fs, ensure_odd
                     )["filter_length"]
-                    for l in v
+                    for fl in v
                 ]
             elif isinstance(v, Real):
                 # DO NOT use `int`, which might not work for numpy array elements
@@ -769,7 +785,7 @@ def adjust_cnn_filter_lengths(
     ensure_odd: bool = True,
     pattern: str = "filter_length|filt_size",
 ) -> dict:
-    """finished, checked,
+    """
 
     adjust the filter lengths in the config for convolutional neural networks,
     according to the new sampling frequency
@@ -797,7 +813,7 @@ def adjust_cnn_filter_lengths(
 
 
 class SizeMixin(object):
-    """finished, checked,
+    """
 
     mixin class for size related methods
 
@@ -846,7 +862,7 @@ class SizeMixin(object):
 
 
 class CkptMixin(object):
-    """finished, checked,
+    """
 
     mixin class for loading from checkpoint class methods
     """
@@ -855,7 +871,7 @@ class CkptMixin(object):
     def from_checkpoint(
         cls, path: str, device: Optional[torch.device] = None
     ) -> Tuple[nn.Module, dict]:
-        """finished, checked,
+        """
 
         load a model from a checkpoint
 
