@@ -5,17 +5,18 @@ including spatial, temporal, spatio-temporal domains
 """
 
 from copy import deepcopy
-from typing import Union, Optional, List, Tuple, Sequence, Iterable, NoReturn
 from numbers import Real
+from typing import Iterable, Optional, Sequence, Tuple, Union
 
 import numpy as np
 from scipy import interpolate
-from scipy.signal import peak_prominences, butter, filtfilt
+from scipy.signal import butter, filtfilt, peak_prominences
+
+from .utils_data import ensure_siglen
 
 
 __all__ = [
     "smooth",
-    "MovingAverage",
     "resample_irregular_timeseries",
     "detect_peaks",
     "remove_spikes_naive",
@@ -93,7 +94,7 @@ def smooth(
     if radius < 3:
         return x
 
-    if not window in ["flat", "hanning", "hamming", "bartlett", "blackman"]:
+    if window not in ["flat", "hanning", "hamming", "bartlett", "blackman"]:
         raise ValueError(
             "Window is on of 'flat', 'hanning', 'hamming', 'bartlett', 'blackman'"
         )
@@ -115,146 +116,6 @@ def smooth(
     return y
 
 
-class MovingAverage(object):
-    """
-
-    moving average
-
-    References
-    ----------
-    [1] https://en.wikipedia.org/wiki/Moving_average
-
-    """
-
-    def __init__(self, data: Sequence, **kwargs) -> NoReturn:
-        """
-        Parameters
-        ----------
-        data: sequence,
-            the series data to compute its moving average
-
-        """
-        self.data = np.array(data)
-        self.verbose = kwargs.get("verbose", 0)
-
-    def cal(self, method: str, **kwargs) -> np.ndarray:
-        """
-        Parameters
-        ----------
-        method: str,
-            method for computing moving average, can be one of
-            - 'sma', 'simple', 'simple moving average'
-            - 'ema', 'ewma', 'exponential', 'exponential weighted', 'exponential moving average', 'exponential weighted moving average'
-            - 'cma', 'cumulative', 'cumulative moving average'
-            - 'wma', 'weighted', 'weighted moving average'
-
-        """
-        m = method.lower().replace("_", " ")
-        if m in ["sma", "simple", "simple moving average"]:
-            func = self._sma
-        elif m in [
-            "ema",
-            "ewma",
-            "exponential",
-            "exponential weighted",
-            "exponential moving average",
-            "exponential weighted moving average",
-        ]:
-            func = self._ema
-        elif m in ["cma", "cumulative", "cumulative moving average"]:
-            func = self._cma
-        elif m in ["wma", "weighted", "weighted moving average"]:
-            func = self._wma
-        else:
-            raise NotImplementedError
-        return func(**kwargs)
-
-    def _sma(self, window: int = 5, center: bool = False, **kwargs) -> np.ndarray:
-        """
-        simple moving average
-
-        Parameters
-        ----------
-        window: int, default 5,
-            window length of the moving average
-        center: bool, default False,
-            if True, when computing the output value at each point, the window will be centered at that point;
-            otherwise the previous `window` points of the current point will be used
-
-        """
-        smoothed = []
-        if center:
-            hw = window // 2
-            window = hw * 2 + 1
-        for n in range(window):
-            smoothed.append(np.mean(self.data[: n + 1]))
-        prev = smoothed[-1]
-        for n, d in enumerate(self.data[window:]):
-            s = prev + (d - self.data[n]) / window
-            prev = s
-            smoothed.append(s)
-        smoothed = np.array(smoothed)
-        if center:
-            smoothed[hw:-hw] = smoothed[window - 1 :]
-            for n in range(hw):
-                smoothed[n] = np.mean(self.data[: n + hw + 1])
-                smoothed[-n - 1] = np.mean(self.data[-n - hw - 1 :])
-        return smoothed
-
-    def _ema(self, weight: float = 0.6, **kwargs) -> np.ndarray:
-        """
-        exponential moving average,
-        which is also the function used in Tensorboard Scalar panel,
-        whose parameter `smoothing` is the `weight` here
-
-        Parameters
-        ----------
-        weight: float, default 0.6,
-            weight of the previous data point
-
-        """
-        smoothed = []
-        prev = self.data[0]
-        for d in self.data:
-            s = prev * weight + (1 - weight) * d
-            prev = s
-            smoothed.append(s)
-        smoothed = np.array(smoothed)
-        return smoothed
-
-    def _cma(self, **kwargs) -> np.ndarray:
-        """
-        cumulative moving average
-
-        """
-        smoothed = []
-        prev = 0
-        for n, d in enumerate(self.data):
-            s = prev + (d - prev) / (n + 1)
-            prev = s
-            smoothed.append(s)
-        smoothed = np.array(smoothed)
-        return smoothed
-
-    def _wma(self, window: int = 5, **kwargs) -> np.ndarray:
-        """
-        weighted moving average
-
-        Parameters
-        ----------
-        window: int, default 5,
-            window length of the moving average
-
-        """
-        # smoothed = []
-        # total = []
-        # numerator = []
-        conv = np.arange(1, window + 1)[::-1]
-        deno = np.sum(conv)
-        smoothed = np.convolve(conv, self.data, mode="same") / deno
-        return smoothed
-
-
 def resample_irregular_timeseries(
     s: np.ndarray,
     output_fs: Real = 2,
@@ -264,7 +125,7 @@ def resample_irregular_timeseries(
     interp_kw: dict = {},
     verbose: int = 0,
 ) -> np.ndarray:
-    """finished, checked,
+    """
 
     resample the 2d irregular timeseries `s` into a 1d or 2d regular time series with frequency `output_fs`,
     elements of `s` are in the form [time, value], where the unit of `time` is ms
@@ -629,7 +490,7 @@ def _plot(x, mph, mpd, threshold, edge, valley, ax, ind):
 def remove_spikes_naive(
     sig: np.ndarray, threshold: Real = 20, inplace: bool = True
 ) -> np.ndarray:
-    """finished, checked,
+    """
 
     remove `spikes` from `sig` using a naive method proposed in entry 0416 of CPSC2019
 
@@ -796,7 +657,7 @@ def get_ampl(
     window: Real = 0.2,
     critical_points: Optional[Sequence] = None,
 ) -> Union[float, np.ndarray]:
-    """finished, checked,
+    """
 
     get amplitude of a signal (near critical points if given)
 
@@ -877,8 +738,8 @@ def normalize(
     sig_fmt: str = "channel_first",
     per_channel: bool = False,
 ) -> np.ndarray:
-    """ finished, checked,
-    
+    r"""
+
     perform z-score normalization on `sig`,
     to make it has fixed mean and standard deviation,
     or perform min-max normalization on `sig`,
@@ -913,12 +774,12 @@ def normalize(
         "channel_first" (alias "lead_first")
     per_channel: bool, default False,
         if True, normalization will be done per channel
-        
+
     Returns
     -------
     nm_sig: ndarray,
         the normalized signal
-        
+
     NOTE
     ----
     in cases where normalization is infeasible (std = 0),
@@ -938,7 +799,7 @@ def normalize(
     if not per_channel:
         assert isinstance(mean, Real) and isinstance(
             std, Real
-        ), f"mean and std should be real numbers in the non per-channel setting"
+        ), "mean and std should be real numbers in the non per-channel setting"
     assert sig_fmt.lower() in [
         "channel_first",
         "lead_first",
